//include	"stdio.h"
#include	"Luna2Pennant.h"



// constructor
Luna2Pennant::Luna2Pennant(OBJHANDLE hObj, int fmodel): VESSEL2(hObj, fmodel)
{
	bFlying = true;
	timer = 1;

	// select mesh, approximately one oit of the is COA, rest are inscriptions
	if ( DRand(0, 50) > 40 )
		AddMesh("r7_VL\\Luna2PennantCoA");
	else
		AddMesh("r7_VL\\Luna2PennantText");
}


void Luna2Pennant::clbkSetClassCaps(FILEHANDLE cfg)
{
	// physics and geometry
	SetSize(0.25);
	SetPMI(_V(0.1, 0.1, 0.1));
	SetCrossSections(_V(0.001, 0.001, 0.01));
	SetCW (0.2, 0.2, 0.2, 0.2); 
	SetRotDrag (_V(0.1, 0.1, 0.1));
	SetLiftCoeffFunc(0);
	SetPitchMomentScale(1e-6);
	SetBankMomentScale(1e-6);
	SetEmptyMass(0.1);
//	SetSurfaceFrictionCoeff(1e5, 1e5);
	SetTouchdownPoints (_V( 1, 0, 0), _V( -1, -1, 0), _V(-1, 1, 0));
//	SetClipRadius(0.1);
	SetVisibilityLimit(0.00001);  // 0.00001: from 100 m    from 0.00001: 1 km
}


// called when reading the scenario file - this does not happen on runtime creation
void Luna2Pennant::clbkLoadStateEx (FILEHANDLE scn, void *status)
{
	//Read the scenario file line by line
    char *line;
	while (oapiReadScenario_nextline (scn, line)) 
	{
		// are we landed? 
		if (!_strnicmp (line, "STATUS", 6))
		{
			if (!_strnicmp (line+7, "Landed", 6))
			bFlying = false;
		}
		
		ParseScenarioLineEx (line, status);
	}
}


void Luna2Pennant::clbkPostStep(double simt, double SimDT, double mjd)
{
	if ( (timer -= SimDT) > 0)
		return;
	
	if (bFlying)
	{
		if (GroundContact())
		{
			// clone wreck status from the current one
			VESSELSTATUS vs;
			GetStatus(vs);

			double Longitude, Latitude, Radius;
			GetEquPos(Longitude, Latitude, Radius);

			vs.status = 1;
			vs.vdata[0].x = Longitude;
			vs.vdata[0].y = Latitude;
			vs.vdata[0].z = 0;

			DefSetState(&vs);

			bFlying = false;
		}
	}
}


DLLCLBK VESSEL *ovcInit(OBJHANDLE hvessel, int flightmodel)
{
	return new Luna2Pennant(hvessel, flightmodel);
}

DLLCLBK void ovcExit(VESSEL *vessel)
{
	if (vessel) 
		delete (Luna2Pennant*) vessel;
}


