#include "..\r7_vessel2m_VL\vessel2m.h"



// geometry and physics 
const		double		EMPTY_MASS = 61.3;
const		double		EXPLOSIVE_MASS = 300.0;          
const		VECTOR3		PMI = {0.25, 0.25, 0.25};
const		VECTOR3		CS = {1.86, 1.86, 1.86};

// antenna rotation speed
const double ANTENNA_ROTATION_SPEED = 70*RAD;  // rad/s
const double TAPE_EXTENSION_SPEED = 70*RAD;  
const double TAPE_ROTATION_SPEED = 70*RAD;  

const float ANTENNA_ROTATION_ANGLE = 85*(float)PI/180;

// antennas rotation centers
const		VECTOR3		ANTEN00_REF = { 0.358, 0, 0.204};
const		VECTOR3		ANTEN01_REF = {0, -0.358, 0.204};
const		VECTOR3		ANTEN02_REF = {-0.358, 0, 0.204};
const		VECTOR3		ANTEN03_REF = { 0, 0.358, 0.204};

// antennas rotation axis
const		VECTOR3		ANTEN00_AXIS = { 0, 1, 0};
const		VECTOR3		ANTEN01_AXIS = {-1, 0, 0};
const		VECTOR3		ANTEN02_AXIS = {0, -1, 0};
const		VECTOR3		ANTEN03_AXIS = { 1, 0, 0};

// sound
const int SND_PYRO = 1;
const int SND_CLANG = 2;
const int SND_MOTOR = 3;


class Luna2_VL: public VESSEL2M
{
public:
	Luna2_VL(OBJHANDLE, int);

	// callbacks
	void clbkSetClassCaps(FILEHANDLE);
	void clbkPostStep(double, double, double);
	void clbkPostCreation();
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);

	// VESSEL2M overrides
	void clbkSaveState (FILEHANDLE scn);
	void ParseScenarioLineEx(char *line, void *status);
	void FinishConfiguration ();
	void Explode();

private:

	OBJHANDLE hMoon;			//+--

	// all animations
	double timer;								//+++   antenna opening delay
	double SimDt;								//---   delta time step, for animation
	bool bAnimate;								//+--
	double tempAnimationPercent;				//+--
    enum animation_type { AT_NONE, AT_ANTENNA, AT_TAPE_EXT, AT_TAPE_ROT } CurrentAnimation;
	void DefineAnimations();
	void Animate();

	// antenna animation
	double AntennaRotationPercent;					
	UINT AntennaRotation;						
	void AnimateAntennaRotation();
	void AntennaRotationStart();
	void AntennaRotationEnd();

	// tape extension animation
	double TapeExtensionPercent;					
	UINT TapeExtension;						
	void AnimateTapeExtension();
	void TapeExtensionEnd();

	// tape rotation animation
	double TapeRotationPercent;					
	UINT TapeRotation;						
	void AnimateTapeRotation();
	void TapeRotationEnd();

	// pennants
	bool bAllowFocus;
	bool bSwitchFocus;

	// sound
	int SoundLibID;					//---
};
