#include	"../math/mymath.h"
#include	"stdio.h"
#include	"Luna3.h"



// constructor
Luna3_VL::Luna3_VL(OBJHANDLE hObj, int fmodel): VESSEL2M(hObj, fmodel)
{
	// VESSEL2M crash parameters
	VvCrash = DRand(10, 50); 
	VhCrash = DRand(30, 60);
	VMaxBlast = DRand(100, 1000); 
	QMax = DRand(10000, 50000);

	// mesh
	int MeshID = AddMesh("r7_VL\\Luna3");
	SetMeshVisibilityMode (MeshID, MESHVIS_ALWAYS);

	// define animation and its defaults
	timer = 0;						// initial animations delay timer
	bAnimate = false;		
	CurrentAnimation = AT_NONE;
	tempAnimationPercent = 0;
	DefineAnimations();

	// start with all animations in the initial states
	AntennaRotationPercent = 0;
	TapeExtensionPercent = 0;
	TapeRotationPercent = 0;
	CoverRotationPercent = 0;

	// var vars
	hMoon = oapiGetGbodyByName ("Moon");
	hEarth = oapiGetGbodyByName ("Earth");
	MaxTimeAccel = 0;
	PastAlt = 0;
	xplus = _V( 1,  0,  0);
	bTrackMoon = false;
	bApKillRot = false;
	bApKillRotStopped = false;

	// taking photo
	PhotoStep = PH_NONE;
	PhotoTimer = 0;
	ScanLine = 0;
	Aperture = 0;
	SavedAperture = 0;
	SavedHUD = -1;
	SavedMFDMode0 = MFD_NONE;
	SavedMFDMode1 = MFD_NONE;

	PhotoCount = 0;
	CurrentPhoto = PhotoCount;		

	// transmitting photos
	ForcedSignalLevel = 0;
	AutoTransmitPhoto = 0;

	for (int i=0; i<MAX_PHOTO_COUNT; i++)
	{
		bTransmittedPhotos[i] = false;
		bSuccessfulPhotos[i] = false;
		surfPhotos[i] = 0;
	}

	surfTempPhoto = 0;
	surfBtnL = 0;			
	surfBtnR = 0;			
	surfBtnTr = 0;			
	surfSignal = 0;			
	
	// internal camera position
	SetCameraOffset(_V(0, 0, 0.3)); 
	
	// flight program
	bManualControl = false;
	ChkDoType = CDT_DETACHED;		// initial check-do mode, default to attached 

	// force photo dirs creation
	CreateDirectory("Photo", NULL);
	CreateDirectory("Photo\\Luna3", NULL);
	CreateDirectory("Photo\\Luna3\\Acquired", NULL);
	CreateDirectory("Photo\\Luna3\\Received", NULL);
}


void Luna3_VL::clbkSetClassCaps(FILEHANDLE cfg)
{
	// physics and geometry
	SetSize(1.0);
	SetPMI(PMI);
	SetCrossSections(CS);
	SetCW (0.2, 0.2, 0.2, 0.2); 
	SetRotDrag (_V(0.1, 0.1, 0.1));
	SetLiftCoeffFunc(0);
	SetPitchMomentScale(1e-6);
	SetBankMomentScale(1e-6);
	SetEmptyMass(EMPTY_MASS);
	SetSurfaceFrictionCoeff(1e5, 1e5);
	SetTouchdownPoints (_V( 0, 1, 2), _V( 2, -1, -2), _V(-2, -2, -2));

		// gas RCS group
	THRUSTER_HANDLE thRollLeft[2], thRollRight[2], thYawLeft[2], thYawRight[2], thPitchDown[2], thPitchUp[2];
	GasRCSTank = CreatePropellantResource(GAS_RCS_FUEL_MASS);

	// YAW
	thYawLeft[0] = CreateThruster (_V(GAS_YAW_X_L_0, 0, GAS_YAW_Z_L_0), _V( cos(GAS_YAW_ANGLE_X), 0, -cos(GAS_YAW_ANGLE_Z)), THRUST_GAS_RSC, GasRCSTank, GAS_RCS_ISP);
	thYawLeft[1] = CreateThruster (_V(GAS_YAW_X_L_1, 0, GAS_YAW_Z_L_1), _V( cos(GAS_YAW_ANGLE_X), 0,  cos(GAS_YAW_ANGLE_Z)), THRUST_GAS_RSC, GasRCSTank, GAS_RCS_ISP);
	CreateThrusterGroup (thYawLeft, 2, THGROUP_ATT_YAWLEFT);

	thYawRight[0] = CreateThruster (_V(GAS_YAW_X_R_0, 0, GAS_YAW_Z_R_0), _V(-cos(GAS_YAW_ANGLE_X), 0, cos(GAS_YAW_ANGLE_Z)), THRUST_GAS_RSC, GasRCSTank, GAS_RCS_ISP);
	thYawRight[1] = CreateThruster (_V(GAS_YAW_X_R_1, 0, GAS_YAW_Z_R_1), _V(-cos(GAS_YAW_ANGLE_X), 0,-cos(GAS_YAW_ANGLE_Z)), THRUST_GAS_RSC, GasRCSTank, GAS_RCS_ISP);
	CreateThrusterGroup (thYawRight, 2, THGROUP_ATT_YAWRIGHT);

	// PITCH
	thPitchDown[0] = CreateThruster (_V(-GAS_X, -D_GAS_Y, GAS_Z), _V(0,  1, 0), THRUST_GAS_RSC, GasRCSTank, GAS_RCS_ISP);
	thPitchDown[1] = CreateThruster (_V( GAS_X, -D_GAS_Y, GAS_Z), _V(0,  1, 0), THRUST_GAS_RSC, GasRCSTank, GAS_RCS_ISP);
	CreateThrusterGroup (thPitchDown, 2, THGROUP_ATT_PITCHDOWN);

	thPitchUp[0] = CreateThruster (_V(-GAS_X,  D_GAS_Y, GAS_Z), _V(0, -1, 0), THRUST_GAS_RSC, GasRCSTank, GAS_RCS_ISP);
	thPitchUp[1] = CreateThruster (_V( GAS_X,  D_GAS_Y, GAS_Z), _V(0, -1, 0), THRUST_GAS_RSC, GasRCSTank, GAS_RCS_ISP);
	CreateThrusterGroup (thPitchUp, 2, THGROUP_ATT_PITCHUP);

	// ROLL
	thRollLeft[0] = CreateThruster (_V( GAS_X, -D_GAS_Y, GAS_Z), _V(0,  1, 0), THRUST_GAS_RSC, GasRCSTank, GAS_RCS_ISP);
	thRollLeft[1] = CreateThruster (_V(-GAS_X,  D_GAS_Y, GAS_Z), _V(0, -1, 0), THRUST_GAS_RSC, GasRCSTank, GAS_RCS_ISP);
	CreateThrusterGroup (thRollLeft, 2, THGROUP_ATT_BANKLEFT);

	thRollRight[0] = CreateThruster (_V(-GAS_X, -D_GAS_Y, GAS_Z), _V(0,  1, 0), THRUST_GAS_RSC, GasRCSTank, GAS_RCS_ISP);
	thRollRight[1] = CreateThruster (_V( GAS_X,  D_GAS_Y, GAS_Z), _V(0, -1, 0), THRUST_GAS_RSC, GasRCSTank, GAS_RCS_ISP);
	CreateThrusterGroup (thRollRight, 2, THGROUP_ATT_BANKRIGHT);

	// particles
	PARTICLESTREAMSPEC GasExhaustStream = { 0,			// flags, reserved
		0.005,			// srcsize, particle size at creation, m
		50.0,			// srcrate, average particle generation rate, Hz   
		1.0,			// V0, average particle emission velocity, m/s
		-0.05,			// srcspread, emission velocity distribution randomisation
		0.001,			// lifetime, average particle lifetime [s]
		0.5,			// growthrate, particle growth rate [m/s]
		0.1,			// atmslowdown, deceleration rate b in atmosphere, defined as v = v0 e- bdt 
		PARTICLESTREAMSPEC::EMISSIVE, PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5, PARTICLESTREAMSPEC::ATM_PLOG, 0, 0 };
	
	AddExhaustStream(thYawLeft[0], &GasExhaustStream);
	AddExhaustStream(thYawLeft[1], &GasExhaustStream);
	AddExhaustStream(thYawRight[0], &GasExhaustStream);
	AddExhaustStream(thYawRight[1], &GasExhaustStream);
	AddExhaustStream(thPitchDown[0], &GasExhaustStream);
	AddExhaustStream(thPitchDown[1], &GasExhaustStream);
	AddExhaustStream(thPitchUp[0], &GasExhaustStream);
	AddExhaustStream(thPitchUp[1], &GasExhaustStream);
	
	AddExhaustStream(thRollLeft[0], &GasExhaustStream);
	AddExhaustStream(thRollLeft[1], &GasExhaustStream);
	AddExhaustStream(thRollRight[0], &GasExhaustStream);
	AddExhaustStream(thRollRight[1], &GasExhaustStream);
}


void Luna3_VL::clbkSaveState(FILEHANDLE scn)
{

	// do inherited VESSEL2M stuff before writing custom stuff
	VESSEL2M::clbkSaveState (scn);
	oapiWriteScenario_string (scn, "=========== LUNA-3 vars", "");

	if (timer > 0)
		oapiWriteScenario_float(scn, "TIMER", timer);

	// animating right now or animation finished!
	if (CurrentAnimation > AT_NONE)
	{
		// which animation is current/last?
		oapiWriteScenario_int (scn, "ANTENNAS_PHASE", CurrentAnimation);

		if (CurrentAnimation == AT_ANTENNA)
			oapiWriteScenario_float (scn, "ANTENNAS_STATUS", AntennaRotationPercent);
		else if (CurrentAnimation == AT_TAPE_EXT)
			oapiWriteScenario_float (scn, "ANTENNAS_STATUS", TapeExtensionPercent);
		else if (CurrentAnimation == AT_TAPE_ROT)
			oapiWriteScenario_float (scn, "ANTENNAS_STATUS", TapeRotationPercent);
		else if (CurrentAnimation == AT_COVER_ROT)
			oapiWriteScenario_float (scn, "ANTENNAS_STATUS", CoverRotationPercent);
	}

	if (bManualControl)
		oapiWriteScenario_int(scn, "MANUAL", bManualControl);

	// photo 
	if (bTrackMoon)
		oapiWriteScenario_int (scn, "TRACK_MOON", bTrackMoon);

	if (PhotoStep > PH_NONE)
		oapiWriteScenario_int (scn, "PHOTO_STEP", PhotoStep);

	if (PhotoCount > 0)
		oapiWriteScenario_int (scn, "PHOTO_COUNT", PhotoCount);

	if (CurrentPhoto != PhotoCount)
		oapiWriteScenario_int (scn, "CURRENT_PHOTO", CurrentPhoto);

	if (PhotoTimer > 0)
		oapiWriteScenario_float (scn, "PHOTO_TIMER", PhotoTimer);

	if (Aperture > 0)
		oapiWriteScenario_float (scn, "APERTURE", Aperture);

	if (SavedAperture > 0)
		oapiWriteScenario_float (scn, "SAVED_APERTURE", SavedAperture);

	if (ScanLine > 0)
		oapiWriteScenario_int (scn, "SCANLINE", ScanLine);

	// transmitted photos, convert to string for better scenario readability
	char TransmittedPhotosString[12];
	int SaveTransmitted = 0;
	TransmittedPhotosString[11] = 0;
	for (int i=0; i<MAX_PHOTO_COUNT; i++) 	
	{
        TransmittedPhotosString[i] = bTransmittedPhotos[i]+48;
		SaveTransmitted += bTransmittedPhotos[i];
	}

	if (SaveTransmitted > 0)
		oapiWriteScenario_string (scn, "TRANSMITTED", TransmittedPhotosString);

	// successful photos, convert to string for better scenario readability
	char SuccessfulPhotosString[12];
	int SaveSuccessful = 0;
	SuccessfulPhotosString[11] = 0;
	for (int i=0; i<MAX_PHOTO_COUNT; i++) 	
	{
        SuccessfulPhotosString[i] = bSuccessfulPhotos[i]+48;
		SaveSuccessful += bSuccessfulPhotos[i];
	}

	if (SaveSuccessful > 0)
		oapiWriteScenario_string (scn, "SUCCESSFUL", SuccessfulPhotosString);

	if (PhotoStep == PH_TRANSMIT)
	{
		oapiWriteScenario_int (scn, "PERCENT_LOST", PercentLost);
		oapiWriteScenario_int (scn, "COLOR_RANGE", ColorRange);
		oapiWriteScenario_int (scn, "START_COLOR_RANGE", ColorRangeStart);
	}

	if (AutoTransmitPhoto > 0)
		oapiWriteScenario_int (scn, "AUTO_TRANSMIT_PHOTO", AutoTransmitPhoto);

	if (ForcedSignalLevel > 0)
		oapiWriteScenario_float (scn, "_DEBUG_SIGNAL_LEVEL", ForcedSignalLevel);

	// flight phase
	if (ChkDoType > CDT_DETACHED)
		oapiWriteScenario_int (scn, "FLIGHT_PHASE", ChkDoType);
}


void Luna3_VL::ParseScenarioLineEx(char *line, void *status)
{
	if (!strnicmp (line, "TIMER", 5))
		sscanf (line+5, "%lf", &timer);

	if (!strnicmp (line, "MANUAL", 6))
		sscanf (line+6, "%i", &bManualControl);

	// antennas state or incomplete animation

	else if (!strnicmp (line, "ANTENNAS_PHASE", 14))
		sscanf (line+14, "%d", &CurrentAnimation);

	else if (!strnicmp (line, "ANTENNAS_STATUS", 15))
		sscanf (line+15, "%lf", &tempAnimationPercent);

	else if (!strnicmp (line, "FLIGHT_PHASE", 12))
		sscanf (line+12, "%d", &ChkDoType);

	// photo 
	else if (!strnicmp (line, "TRACK_MOON", 10))
		sscanf (line+10, "%d", &bTrackMoon);

	else if (!strnicmp (line, "PHOTO_STEP", 10))
		sscanf (line+10, "%d", &PhotoStep);

	else if (!strnicmp (line, "PHOTO_COUNT", 11))
	{
		sscanf (line+11, "%d", &PhotoCount);
		CurrentPhoto = PhotoCount;
	}

	else if (!strnicmp (line, "CURRENT_PHOTO", 13))
		sscanf (line+13, "%d", &CurrentPhoto);

	else if (!strnicmp (line, "AUTO_TRANSMIT_PHOTO", 19))
		sscanf (line+19, "%d", &AutoTransmitPhoto);

	else if (!strnicmp (line, "PHOTO_TIMER", 11))
		sscanf (line+11, "%lf", &PhotoTimer);

	else if (!strnicmp (line, "APERTURE", 8))
		sscanf (line+8, "%lf", &Aperture);

	else if (!strnicmp (line, "SAVED_APERTURE", 14))
		sscanf (line+14, "%lf", &SavedAperture);

	else if (!strnicmp (line, "SCANLINE", 8))
		sscanf (line+8, "%d", &ScanLine);

	// transmitted photos
	else if (!strnicmp (line, "TRANSMITTED", 11))
	{
		char TransmittedPhotosString[12];
		sscanf (line+11, "%s", TransmittedPhotosString);

		for (int i=0; i<MAX_PHOTO_COUNT; i++) 	
	        bTransmittedPhotos[i] = (TransmittedPhotosString[i]-48) != 0;
	}

	// successful photos
	else if (!strnicmp (line, "SUCCESSFUL", 10))
	{
		char SuccessfulPhotosString[12];
		sscanf (line+10, "%s", SuccessfulPhotosString);

		for (int i=0; i<MAX_PHOTO_COUNT; i++) 	
	        bSuccessfulPhotos[i] = (SuccessfulPhotosString[i]-48) != 0;
	}

	else if (!strnicmp (line, "PERCENT_LOST", 12))
		sscanf (line+12, "%d", &PercentLost);

	else if (!strnicmp (line, "COLOR_RANGE", 11))
		sscanf (line+11, "%d", &ColorRange);
		
	else if (!strnicmp (line, "START_COLOR_RANGE", 17))
		sscanf (line+17, "%d", &ColorRangeStart);

	else if (!strnicmp (line, "_DEBUG_SIGNAL_LEVEL", 19))
		sscanf (line+19, "%lf", &ForcedSignalLevel);

	// other...
	else 
		VESSEL2M::ParseScenarioLineEx(line, status);
}


// called after all vessel in scn are created, but before first timestep
void Luna3_VL::FinishConfiguration()
{
	// update animation percents
	switch (CurrentAnimation)
	{
		case AT_COVER_ROT :
		{
			CoverRotationPercent = tempAnimationPercent;
			TapeRotationPercent = 1;
			TapeExtensionPercent = 1;
			AntennaRotationPercent = 1;
			break;
		}

		case AT_TAPE_ROT :
		{
			TapeRotationPercent = tempAnimationPercent;
			TapeExtensionPercent = 1;
			AntennaRotationPercent = 1;
			break;
		}
		
		case AT_TAPE_EXT :
		{
			TapeExtensionPercent = tempAnimationPercent;
			AntennaRotationPercent = 1;
			break;
		}

		case AT_ANTENNA :
		{
			AntennaRotationPercent = tempAnimationPercent;
			break;
		}
	}

	// fast-forward animations
	SetAnimation(AntennaRotation, AntennaRotationPercent);
	SetAnimation(TapeExtension, TapeExtensionPercent);
	SetAnimation(TapeRotation, TapeRotationPercent);
	SetAnimation(CoverRotation, CoverRotationPercent);

	// is animation still running?
	bAnimate = ( (tempAnimationPercent > 0) && (tempAnimationPercent < 1) );
	
	// load any previously taken photos, if any. As a minimum, load test table
	for (int i=0; i<=PhotoCount; i++)
	{
		// load image from either received or acquired images
		char fileName[45];

		if (bTransmittedPhotos[i] == true)
            sprintf(fileName, "PHOTO\\Luna3\\Received\\photo%d.bmp", i);
		else
			sprintf(fileName, "PHOTO\\Luna3\\Acquired\\photo%d.bmp", i);
			
		surfPhotos[i] = oapiCreateSurface (LOADBMP (fileName));

		// if there is no photo file, load tv table instead
		if (surfPhotos[i] == NULL)
			surfPhotos[i] = oapiCreateSurface (LOADBMP ("Textures\\r7_VL\\TvTable.bmp"));
	}

	// temp surface for taking photos in-flight
	surfTempPhoto = oapiCreateSurface (LOADBMP ("Textures\\r7_VL\\TvTable.bmp"));

	// show photo in the photo panel
	oapiTriggerPanelRedrawArea (0, PHOTO_AREA);

	if (!bManualControl)
		SetAutopilotControl();

	//VESSEL2M stuff
	VESSEL2M::FinishConfiguration();
}


void Luna3_VL::GetVesselMessage(VesselMessage VM)
{
	// receive autopilot status from block E
	if (VM == VM_AUTOPILOT_OFF)
		bManualControl = true;

	// VESSEL2M stuff
	else
		VESSEL2M::GetVesselMessage(VM);
}


int Luna3_VL::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{

	// only process keydown events
	if (!down) 
		return 0; 

	// shift combinations are reserved
	if (KEYMOD_SHIFT (kstate)) 
		return 0; 

	// control combinations are reserved
	if (KEYMOD_CONTROL (kstate)) 
		return 0; 

	// open photo covers
	if (key == OAPI_KEY_C) 
		ManualTryCoverRotationStart();

	// track Moon
	else if (key == OAPI_KEY_K) 
		ManualTryToggleMoonTracking();

	// take  photo
	else if (key == OAPI_KEY_P) 
		ManualTryTakePhoto();

	// load transmission panel
	else if (key == OAPI_KEY_G) 
		ManualTryStartTransmission();

	// toggle between autopilot and manual control 
	else if (key == OAPI_KEY_M) 
		ToggleManualAndAutoplilot();

	// manual debug
	else if (key == OAPI_KEY_D) 
		DebugSomething();

	return 0;
}


int Luna3_VL::clbkConsumeDirectKey (char *keystate)
{
	if (!bManualControl)
	{
		// any of the following keys should be disregarded in autopilot mode
		if	(
				KEYDOWN (keystate, OAPI_KEY_NUMPAD8) + 
				KEYDOWN (keystate, OAPI_KEY_NUMPAD4) +
				KEYDOWN (keystate, OAPI_KEY_NUMPAD6) +
				KEYDOWN (keystate, OAPI_KEY_NUMPAD2) +
				KEYDOWN (keystate, OAPI_KEY_NUMPAD1) +
				KEYDOWN (keystate, OAPI_KEY_NUMPAD3)
			)
		{
			ShowAnnotationMessage("Manual controls disabled. To enable, use M key");
			return 1;
		}
	}
	
	return 0;
}


void Luna3_VL::clbkPostStep(double simt, double SimDT, double mjd)
{
	// common processing for all VESSEL2M-derived objects
	TimestepForVESSEL2M();

	// store timestep vars for use in other places
	SimDt = SimDT;
	Alt = GetAltitude();

	// perform generic condition check (condition and action depend on flight phase)
	CurrentConditionCheck();

	// run animation
	if (bAnimate)
		Animate();

	// calculate orientation and set thrusters accordingly
	if (bTrackMoon)
	{
		CalcCurrentOrientLevels();
		CalcCurrentOmegaLevels();
		SetOrientThrusters();
	}

	// photo process step
	if (PhotoStep > PH_NONE)
		StepPhoto();

	// limit time acceleration depending on the moon's proximity or dynamic operations
	if (MaxTimeAccel > 0)
		if (oapiGetTimeAcceleration() > MaxTimeAccel)
			oapiSetTimeAcceleration(MaxTimeAccel);
}

	// ===== AUTOPILOTS =====


void Luna3_VL::ToggleManualAndAutoplilot()
{
	// do not allow toggling during pre-burn!
	if (bAttached)
	{
		ShowAnnotationMessage("Manual/Autopilot controls can't be toggled until separation!");
		return;
	}

	// toggle flag
	bManualControl = !bManualControl;

	// implement switching to the appropriate mode
	if (bManualControl)
	{
		ShowAnnotationMessage("Manual controls enabled. To return to autopilot, use M key");
		sprintf(oapiDebugString(), "");
	}
	else
	{
		ShowAnnotationMessage("Manual controls disabled. To enable, use M key");
		sprintf(oapiDebugString(), "");

		SetAutopilotControl();
	}
}


void Luna3_VL::SetAutopilotControl()
{
	// disable any possible residual navmodes
	DeactivateNavmode(NAVMODE_KILLROT);
	DeactivateNavmode(NAVMODE_HLEVEL);
	DeactivateNavmode(NAVMODE_RETROGRADE);
	DeactivateNavmode(NAVMODE_NORMAL);
	DeactivateNavmode(NAVMODE_ANTINORMAL);
	DeactivateNavmode(NAVMODE_HOLDALT);
	DeactivateNavmode(NAVMODE_PROGRADE);

	// in photo window
	if (ChkDoType == CDT_PHOTOEND)
	{
		// if some film left...
		if (PhotoCount < MAX_PHOTO_COUNT)
		{
			// ... and no success yet - ...
			bool bSuccess = false;
			for (int i=1; i<=PhotoCount; i++)
				if (bSuccessfulPhotos[i])
					bSuccess = true;

			// track moon (and take photo)
			if (!bSuccess)
				bTrackMoon = true;
		}

		// if photo cover not open yet - open it!
		if (CoverRotationPercent < 1)
			CoverRotationStart();
	}
}


void Luna3_VL::ManualTryToggleMoonTracking()
{
	// some additional limitation when trying to toggle it "ON"
	if (!bTrackMoon)
	{
		if (ChkDoType != CDT_PHOTOEND)
		{
			ShowAnnotationMessage("Auto tracking of the Moon is not available outside photo window!");
			return;
		}

		if (oapiGetTimeAcceleration() > 1)
		{
			ShowAnnotationMessage("Auto tracking of the Moon is disabled during time acceleration.");
			return;
		}
	}

	// flip the flag
	bTrackMoon = !bTrackMoon;

	// do some actions
	if (bTrackMoon)
		ShowAnnotationMessage("Auto tracking of the Moon is ON");
	else
	{
		ShowAnnotationMessage("Auto tracking of the Moon is OFF");

		// shut off any running thrusters
		ResetOrientThrusters();
	
		// cleanup any residual tracking output
		sprintf(oapiDebugString(), "" );
	}
}


void Luna3_VL::CalcCurrentOrientLevels()
{
	// override "generic" time accel limitation with more strict one
	if (oapiGetTimeAcceleration() > 1)
		oapiSetTimeAcceleration(1);

	// direction to Moon - global frame
	VECTOR3 DirMoonGlobal;
	GetRelativePos(hMoon, DirMoonGlobal);

	// get yaw-pitch angles
	LocalRot(DirMoonGlobal, CurrentOrientLevels);
	CurrentOrientLevels /= length(CurrentOrientLevels);

	// swap pitch and yaw, pitch goes to x, yaw to y
	double tmp = CurrentOrientLevels.x;
	CurrentOrientLevels.x = CurrentOrientLevels.y;
	CurrentOrientLevels.y = tmp;

	// normalize yaw-pitch sensors to 0+-1 range
	CurrentOrientLevels /= 2;
	if (CurrentOrientLevels.z > 0)
	{
		if (CurrentOrientLevels.x > 0)
			CurrentOrientLevels.x = 1 - CurrentOrientLevels.x;
		else 
			CurrentOrientLevels.x = -1 - CurrentOrientLevels.x;

		if (CurrentOrientLevels.y > 0)
			CurrentOrientLevels.y = 1 - CurrentOrientLevels.y;
		else 
			CurrentOrientLevels.y = -1 - CurrentOrientLevels.y;
	}

	// get roll orientation
	VECTOR3 DirXHorizon;
	HorizonRot(xplus, DirXHorizon);

	// normalize roll sensor to 0+-1 range
	double signZ = -fabs(DirXHorizon.z) / DirXHorizon.z;
	CurrentOrientLevels.z = (1 - DirXHorizon.x)/2 * signZ;

	// limit epsilon-zero sensitivity: when very small, set level to 0
	const double ANGLE_LEVEL_EPS = 0.001;

	if (fabs(CurrentOrientLevels.x) < ANGLE_LEVEL_EPS)
		CurrentOrientLevels.x = 0;
	if (fabs(CurrentOrientLevels.y) < ANGLE_LEVEL_EPS)
		CurrentOrientLevels.y = 0;
	if (fabs(CurrentOrientLevels.z) < ANGLE_LEVEL_EPS)
		CurrentOrientLevels.z = 0;

}

void Luna3_VL::CalcCurrentOmegaLevels()
{
	// get current omegas, normalize to degrees
	GetAngularVel(omegas);
	omegas *= DEG;

	// calc omega levels, separately on each channel
	CalcOneCurrentOmegaLevel(CH_PITCH);
	CalcOneCurrentOmegaLevel(CH_YAW);
	CalcOneCurrentOmegaLevel(CH_ROLL);
}


void Luna3_VL::CalcOneCurrentOmegaLevel(orientation_channel Channel)
{
	// assign channel-specific values
	double *pAngle;
	double *pOmega;
	double *pOmegaLevel;
	double signOmega;

	if (Channel == CH_PITCH)
	{
		pAngle = &CurrentOrientLevels.x;
		pOmega = &omegas.x;
		pOmegaLevel = &OmegaLevels.x;
		signOmega = 1;
	}
	else if (Channel == CH_YAW)
	{
		pAngle = &CurrentOrientLevels.y;
		pOmega = &omegas.y;
		pOmegaLevel = &OmegaLevels.y;
		signOmega = -1;
	}
	else 
	{
		pAngle = &CurrentOrientLevels.z;
		pOmega = &omegas.z;
		pOmegaLevel = &OmegaLevels.z;
		signOmega = 1;
	}

	// start neutral
	*pOmegaLevel = 0;

	// set omega limits, depending on angle
	double omegaMax = 5;
	double omegaMin = 4.5;

	if (fabs(*pAngle) < 0.1)
	{
		omegaMax = 3;
		omegaMin = 2.5;
	}
	
	if (fabs(*pAngle) < 0.01)
	{
		omegaMax = 0.5;
		omegaMin = 0.45;
	}

	// going too fast? break: set level to *-1
	if (fabs(*pOmega) >= omegaMax)
		*pOmegaLevel = -signPM(*pOmega);

	// going wrong direction? speed up to the right direction 
	else if (signPM(*pOmega) == (signPM(*pAngle)*signOmega) )
		*pOmegaLevel = -signPM(*pOmega);

	// going right direction, but too slow? speed up to the min speed
	else if (fabs(*pOmega) <= omegaMin)
		*pOmegaLevel = signPM(*pOmega);
}


void Luna3_VL::ResetOrientThrusters()
{
	// set all thrusters to neutral
	SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
	SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
	SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
	SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
	SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT, 0);
	SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT, 0);
}


void Luna3_VL::SetOrientThrusters()
{
	// screen output
	if (oapiGetFocusObject() == GetHandle()) 
		sprintf(oapiDebugString(), "pitch: %.3f  yaw: %.3f  roll: %.3f", CurrentOrientLevels.x, CurrentOrientLevels.y, CurrentOrientLevels.z);

	// start with everything neutral
	ResetOrientThrusters();

	// still stopping...
	if ( (bApKillRot) && (!bApKillRotStopped) )
		return;

	// stopped?
	if (bApKillRotStopped)
	{
//		DeactivateNavmode(NAVMODE_KILLROT);
		bApKillRotStopped = false;
		ManualTryToggleMoonTracking();		

		// autopilot: take photo (if there is film and did not take any good shots yet)
		if (!bManualControl) 
			if (PhotoCount < MAX_PHOTO_COUNT)
			{
				bool bSuccess = false;
				for (int i=1; i<=PhotoCount; i++)
					if (bSuccessfulPhotos[i])
						bSuccess = true;

				if (!bSuccess)
					StartPhoto();
			}

		return;
	}

	// time to stop?
	if (length(CurrentOrientLevels) <= 0.001)
	{
		if (length(omegas) > 0.001)
		{
			bApKillRot = true;
			ActivateNavmode(NAVMODE_KILLROT);
		}

		return;
	}

	// set pitch-yaw
	SetChannelThrustersLevel(THGROUP_ATT_PITCHUP, THGROUP_ATT_PITCHDOWN, OmegaLevels.x);
	SetChannelThrustersLevel(THGROUP_ATT_YAWLEFT, THGROUP_ATT_YAWRIGHT, OmegaLevels.y);
	SetChannelThrustersLevel(THGROUP_ATT_BANKRIGHT, THGROUP_ATT_BANKLEFT, OmegaLevels.z);
}


void Luna3_VL::SetChannelThrustersLevel(THGROUP_TYPE THG_POSITIVE, THGROUP_TYPE THG_NEGATIVE, double OmegaLevel)
{
	if (OmegaLevel > 0)
	{
		SetThrusterGroupLevel(THG_POSITIVE, OmegaLevel);
		SetThrusterGroupLevel(THG_NEGATIVE, 0);
	}
	else 
	{
		SetThrusterGroupLevel(THG_POSITIVE, 0);
		SetThrusterGroupLevel(THG_NEGATIVE, -OmegaLevel);
	}
}


void Luna3_VL::clbkNavMode (int mode, bool active)
{
	if ( (mode == NAVMODE_KILLROT) && (bApKillRot) && (mode == !active) )
	{
		bApKillRot = false;
		bApKillRotStopped = true;
		return;
	}

	// in manual mode, allow everything
	if (bManualControl) 
		return;

	// in autopilot: anything but KillRot:
	if (mode != NAVMODE_KILLROT)
	{
		// deactivate: allow silently
		if (!active) 
			return;

		// activate: override by immediately deactivating it, complain on-screen!
		else
		{
			DeactivateNavmode(mode);
			ShowAnnotationMessage("Manual controls disabled. To enable, use M key");
		}
	}

	// in autoplilot: KillRot

	// deactivate killrot: always allow, no harm
	if (!active) 
		return;

	// activate
	else
	{
		// allowed? do it silently
		if (bApKillRot)
			return;

		// not allowed? restore and complain
		else
		{
			DeactivateNavmode(mode);
			ShowAnnotationMessage("Manual controls disabled. To enable, use M key");
		}
	}
}


	// ===== PHOTOS =====


// indicate that we are starting a photo snap process, switch to internal view
void Luna3_VL::StartPhoto()
{
	ShowAnnotationMessage("Taking photo...");

	// force focus
	if (oapiGetFocusObject() != GetHandle())
		oapiSetFocusObject(GetHandle());

	// force camera mode to internal view
	double OriginalCameraMode = oapiCameraMode();
	if (OriginalCameraMode != CAM_COCKPIT)
		oapiCameraAttach(GetHandle(), 0); // internal

	// force generic cocpit by pressing F8 to switch from panel view
	if (oapiCockpitMode() == COCKPIT_PANELS)
	{
		keybd_event(0, 0x42, KEYEVENTF_SCANCODE, 0);
		keybd_event(0, 0x42, KEYEVENTF_SCANCODE | KEYEVENTF_KEYUP, 0);
	}
	else // save original visual environment: HUD, MFDs
	{
		SavedHUD = oapiGetHUDMode();
		SavedMFDMode0 = oapiGetMFDMode(MFD_LEFT);
		SavedMFDMode1 = oapiGetMFDMode(MFD_RIGHT);
	}

	// save current camera aperture
	SavedAperture = oapiCameraAperture()*DEG;

	// introduce a small delay...
	PhotoTimer = 1;
	PhotoStep = PH_WAIT_1;
}



void Luna3_VL::StepPhoto()
{
	// update timer
	PhotoTimer -= SimDt;

	// override "generic" time accel limitation with more strict one
	if (oapiGetTimeAcceleration() > 1)
		oapiSetTimeAcceleration(1);

	// short delay before zooming in
	if (PhotoStep == PH_WAIT_1)
	{
		if (PhotoTimer < 0)
		{
			// set new aperture, remove HUD and MFDs
			Aperture = SavedAperture;
			oapiSetHUDMode(HUD_NONE);
			oapiOpenMFD(MFD_NONE, MFD_LEFT);
			oapiOpenMFD(MFD_NONE, MFD_RIGHT);

			// start zooming in
			PhotoStep = PH_ZOOM;
		}
	}

	else if (PhotoStep == PH_ZOOM)
	{
		// decreasing aperture until good for photo
		if (Aperture > 5)
		{
			Aperture -= 20 * SimDt;
			oapiCameraSetAperture(Aperture*RAD);
			PlayVesselWave3(SoundLibID, SND_MOTOR, LOOP);
			oapiCameraAttach(GetHandle(), 0); // re-force internal camera - to force Moon re-render
			return;
		}

		// if we got here - take photo to clipboard!
		Aperture = 5;
		StopVesselWave3(SoundLibID, SND_MOTOR);
		PlayVesselWave3(SoundLibID, SND_PHOTO);

		// take screenshot photo to clipboard via Alt+PrtScr
		keybd_event(VK_MENU, 0, 0, 0);
		keybd_event(VK_SNAPSHOT, 0, 0, 0);
		keybd_event(VK_SNAPSHOT, 0, KEYEVENTF_KEYUP, 0);
		keybd_event(VK_MENU, 0, KEYEVENTF_KEYUP, 0);

		// mission success criteria: check if the photo is good, mark as such
		CalcCurrentOrientLevels();
		bSuccessfulPhotos[CurrentPhoto+1] = 
			(
				(ChkDoType == CDT_PHOTOEND) &&
				( (fabs(CurrentOrientLevels.x) + fabs(CurrentOrientLevels.y)) < 0.01 ) 
			);

		// introduce a small delay
		PhotoTimer = 1;
		PhotoStep = PH_WAIT_2;
	}
	
	// short delay before pasting frame from clipboard
	else if (PhotoStep == PH_WAIT_2)
	{
		if (PhotoTimer < 0)
		{
			// notification 
			ShowAnnotationMessage("Photo frame acquired, processing...");

			// restore visuals
			oapiCameraSetAperture(SavedAperture*RAD);
			SavedAperture = 0;
			Aperture = 0;

			oapiOpenMFD(SavedMFDMode0, MFD_LEFT);
			oapiOpenMFD(SavedMFDMode1, MFD_RIGHT);
			SavedMFDMode0 = MFD_NONE;
			SavedMFDMode1 = MFD_NONE;

			if (SavedHUD >= HUD_NONE)
			{
				oapiSetHUDMode(SavedHUD);
				SavedHUD = -1;
			}

			// paste screenshot
			OpenClipboard(NULL);
			HBITMAP hbmp = NULL;
			hbmp = (HBITMAP)GetClipboardData(CF_BITMAP);
			SURFHANDLE temp  = oapiCreateSurface (hbmp, false);
			CloseClipboard();

			// get screenshot dimensions
			BITMAP bm;
			GetObject(hbmp, sizeof(bm), &bm);
			int W = bm.bmWidth;
			int H = bm.bmHeight;

			// copy the cropped portion of the screenshot to a photo surface
			hDCphoto = oapiGetDC (surfTempPhoto);
			HDC hDCtemp = oapiGetDC (temp); 
			BitBlt(hDCphoto, 0, 0, 600, 600, hDCtemp, (W-600)/2, (H-600)/2, SRCCOPY); 
			oapiReleaseDC (temp, hDCtemp);
			oapiDestroySurface (temp);

			// proceed to the next step
			ScanLine = 0;
			PhotoStep = PH_DEVELOP;
		}
	}

	// convert photo to grayscale
	else if (PhotoStep == PH_DEVELOP)
	{
		// convert pixel by pixel, pretty slow, so two scanlines at a timestep
		for(int x=0; x<600; x++)
		{
			unsigned long px =GetPixel(hDCphoto, x, ScanLine); 
            int c = (int)(GetRValue(px)*0.114 + GetGValue(px)*0.587 + GetBValue(px)*0.299);
			SetPixel(hDCphoto, x, ScanLine, RGB(c, c, c));
		}
		
		ScanLine++;

		for(int x=0; x<600; x++)
		{
			unsigned long px =GetPixel(hDCphoto, x, ScanLine); 
            int c = (int)(GetRValue(px)*0.114 + GetGValue(px)*0.587 + GetBValue(px)*0.299);
			SetPixel(hDCphoto, x, ScanLine, RGB(c, c, c));
		}

		// report processing progress
		sprintf(oapiDebugString(), "Photo frame %d is being processed: %d %%", PhotoCount+1, ScanLine/6 );

		// last scanline? prepare for marking
		if (ScanLine++ >= 600)
		{
			sprintf(oapiDebugString(), "");
			ScanLine = 0;
		    SelectObject(hDCphoto, GetStockObject(WHITE_PEN));
			PhotoStep = PH_MARK;
		}
	}

	// put marker crosses, one row per timestep
	else if (PhotoStep == PH_MARK)
	{
		for(int xx=0; xx<5; xx ++)
		{
			// horizontal line of the cross
			MoveToEx(hDCphoto, xx*100+95, ScanLine*100+100, (LPPOINT) NULL);
			LineTo  (hDCphoto, xx*100+105, ScanLine*100+100  );

			// vertical line of the cross
			MoveToEx(hDCphoto, xx*100+100, ScanLine*100+95, (LPPOINT) NULL);
			LineTo  (hDCphoto, xx*100+100, ScanLine*100+105);
		 }

		 if (ScanLine++ == 4)
		 {
			// save photo to file
			char filename[45];
            sprintf(filename, "PHOTO\\Luna3\\Acquired\\photo%d.bmp", PhotoCount+1);

			HDC dcSave = CreateCompatibleDC(hDCphoto);
			HBITMAP bmpSave = CreateCompatibleBitmap(hDCphoto, 600, 600);
			SelectObject(dcSave, bmpSave);
			BitBlt(dcSave, 0, 0, 600, 600, hDCphoto, 0, 0, SRCCOPY); 

			SaveBitmapFile(dcSave, bmpSave, filename);

			 // clean up
			oapiReleaseDC (surfTempPhoto, hDCphoto); 
			ScanLine = 0;

			// update photo count
			PhotoCount++;

			// set current photo to the new one
			CurrentPhoto = PhotoCount;

			// append photo to the array, from newly saved file
			surfPhotos[CurrentPhoto] = oapiCreateSurface (LOADBMP (filename));

		 	// introduce a small delay
			PhotoTimer = 1;
			PhotoStep = PH_WAIT_3;
		 }
	}

	// final delay before showing the photo
	else if (PhotoStep == PH_WAIT_3)
	{
		if (PhotoTimer < 0)
		{
			ShowAnnotationMessage("Photo successfully aquired!");

			// done with the photo frame!
			PhotoTimer = 0;
			PhotoStep = PH_NONE;

			// press F8 to switch to panel view...
			keybd_event(0, 0x42, KEYEVENTF_SCANCODE, 0);
			keybd_event(0, 0x42, KEYEVENTF_SCANCODE | KEYEVENTF_KEYUP, 0);
		}
	}

	// photo transmission, a function of its own
	else if (PhotoStep == PH_TRANSMIT)
		StepTransmission();
}


void Luna3_VL::ManualTryTakePhoto()
{
	if (PhotoStep > PH_NONE)
		ShowAnnotationMessage("Photo session is already in progress, please wait...");

	else if (PhotoCount > 9)
		ShowAnnotationMessage("No more film for photos.");

	else if (CoverRotationPercent < 1)
		ShowAnnotationMessage("Photo camera cover is not open.");

	else if (oapiGetTimeAcceleration() > 1)
		ShowAnnotationMessage("Photo function is disabled during time acceleration.");

	else if (!bManualControl)
		ShowAnnotationMessage("Manual photo function is disabled in autopilot mode.");

	else
		StartPhoto();
}


	// ===== TRANSMISSION =====


void Luna3_VL::StartTransmission()
{
	ShowAnnotationMessage("Opening photo transmission panel...");

	// force focus
	if (oapiGetFocusObject() != GetHandle())
		oapiSetFocusObject(GetHandle());

	// force camera mode to internal view
	double OriginalCameraMode = oapiCameraMode();
	if (OriginalCameraMode != CAM_COCKPIT)
		oapiCameraAttach(GetHandle(), 0); // internal

	// force generic cocpit by pressing F8 to switch from panel view
	if (oapiCockpitMode() == COCKPIT_PANELS)
	{
		keybd_event(0, 0x42, KEYEVENTF_SCANCODE, 0);
		keybd_event(0, 0x42, KEYEVENTF_SCANCODE | KEYEVENTF_KEYUP, 0);
	}

	// force panel view by pressing F8 
	keybd_event(0, 0x42, KEYEVENTF_SCANCODE, 0);
	keybd_event(0, 0x42, KEYEVENTF_SCANCODE | KEYEVENTF_KEYUP, 0);
}


// PhotoStep == PH_TRANSMIT
void Luna3_VL::StepTransmission()
{
	// radio sound
	PlayVesselWave3(SoundLibID, SND_RADIO, LOOP);

	// report processing progress
	sprintf(oapiDebugString(), "Photo frame %d is being transmitted: %d %%", CurrentPhoto, ScanLine/6 );

	hDCphoto = oapiGetDC (surfPhotos[CurrentPhoto]);

	//random scanline dropout
	int DropChance = IRand(0, 100);  // 0 to 100%
	bool bBadScanline = DropChance <= (PercentLost/5);

	// pixel, channel
	unsigned long px;
	int c;

	// bad scanline
	if (bBadScanline)
	{
		for(int x=0; x<600; x++)
		{
			c = IRand(ColorRangeStart, ColorRangeStart+ColorRange);
			px = RGB(c, c, c);
			SetPixel(hDCphoto, x, ScanLine, px);
		}
	}

	// good scanline
	else
	{
		for(int x=0; x<600; x++)
		{
			// original pixel
			px = GetPixel(hDCphoto, x, ScanLine); 

			// reduce pixel contrast
			c = GetRValue(px);  // one channel, all three are identical in grayscale

			c = (int)((double)c * (double)ColorRange / 256.0);
			c += ColorRangeStart;

			px = RGB(c, c, c);
			
			// replace pixel
			SetPixel(hDCphoto, x, ScanLine, px);
		}
	}

	// redraw
	oapiReleaseDC (surfPhotos[CurrentPhoto], hDCphoto); 
	oapiTriggerPanelRedrawArea (0, PHOTO_AREA);

	// last scanline? done, save to file, refresh screen, do other related stuff
	if (ScanLine++ >= 600)
	{
		// stop sound
		StopVesselWave3(SoundLibID, SND_RADIO);

		// save photo to file
		char filename[45];
        sprintf(filename, "PHOTO\\Luna3\\Received\\photo%d.bmp", CurrentPhoto);

		hDCphoto = oapiGetDC (surfPhotos[CurrentPhoto]);
		HDC dcSave = CreateCompatibleDC(hDCphoto);
		HBITMAP bmpSave = CreateCompatibleBitmap(hDCphoto, 600, 600);
		SelectObject(dcSave, bmpSave);
		BitBlt(dcSave, 0, 0, 600, 600, hDCphoto, 0, 0, SRCCOPY); 
		oapiReleaseDC (surfPhotos[CurrentPhoto], hDCphoto); 

		SaveBitmapFile(dcSave, bmpSave, filename);

		// mark photo as transmitted
		bTransmittedPhotos[CurrentPhoto] = true;

		// clean up
		ScanLine = 0;

		// clean up screen message
		sprintf(oapiDebugString(), "");

		// send success annotation message
		ShowAnnotationMessage("Photo successfully transmitted!");

		// stop the transmission
		PhotoStep = PH_NONE;

		oapiTriggerPanelRedrawArea (0, BTN_LEFT);
		oapiTriggerPanelRedrawArea (0, BTN_RIGHT);

		// for autopilot: repeat for the second photo (just modify AutoTransmitPhoto, UpdateSignalLevel function will do the rest)
		if (AutoTransmitPhoto == 0)
		{
			for (int i=1; i<=MAX_PHOTO_COUNT; i++)
			{
				if (bSuccessfulPhotos[i])
				{
					AutoTransmitPhoto = i; 
					continue;
				}
			}
		}


		// show success banner here, if yes
		int SuccessCount = 0;
	
		for (int i=1; i<=PhotoCount; i++)
			if (bTransmittedPhotos[i] && bSuccessfulPhotos[i])
				SuccessCount++;

		if (SuccessCount == 1)
			ShowAnnotationMessage("Congratulations! Mission successfully accomplished!");
	}
}


void Luna3_VL::ManualTryStartTransmission()
{
	if (PhotoStep > PH_NONE)
		ShowAnnotationMessage("Photo session is already in progress, please wait...");

	if (ChkDoType != CDT_TRANSEND)
		ShowAnnotationMessage("Transmission is not possible outside transmission window!");

	else
		StartTransmission();
}

void Luna3_VL::UpdateSignalLevel()
{
	// elev level
	double Lon, Lat, Rad;
	GetEquPos(Lon, Lat, Rad);
	
	// ground track length, rad
	double Dist;
	DistanceFromCoordPairRad(Lat, Lon, LAT_EUP, LON_EUP, &Dist);
	
	// direct dist to Eupatoria, m
	double HH = Alt + RADIUS0;
	double L = sqrt(RADIUS0*RADIUS0 + HH*HH - 2*RADIUS0*HH*cos(Dist));

	// elevation 
	double sinElev = (HH / L) * sin(Dist);
	double Elev = asin(sinElev) * DEG;
	Elev = 90 - Elev;

	// check quadrants/negatives
	if (Dist*DEG > 90)
		Elev = 0;

	if (Elev < ELEV_TRANSM_MIN)
		ElevLevel = 0;
	else
		ElevLevel = (Elev - ELEV_TRANSM_MIN) / (90 - ELEV_TRANSM_MIN);

	// dist level
	if (L > ALT_TRANSM_MAX)
		AltLevel = 0;
	else if (L < ALT_TRANSM_MIN)
		AltLevel = 1;
	else
		AltLevel = (ALT_TRANSM_MAX - L) / (ALT_TRANSM_MAX - ALT_TRANSM_MIN);

	// produce signal level, normalize to 0 to 1
	if ( (AltLevel * ElevLevel) == 0 )
		SignalLevel = 0;
	else
		SignalLevel = (AltLevel + ElevLevel) / 2;

	// debug-only override
	if (ForcedSignalLevel > 0)
		SignalLevel = ForcedSignalLevel;

	// if not in panel view, show signal level
	if ((oapiCockpitMode() == COCKPIT_PANELS) && (oapiCameraMode() == CAM_COCKPIT) )
		sprintf(oapiDebugString(), "");
	else
		sprintf(oapiDebugString(), "Signal level: %f", SignalLevel);

	// autopilot: start transmission when allowed
	if (!bManualControl)
		if (!bTransmittedPhotos[AutoTransmitPhoto])
			if (PhotoStep == PH_NONE)
				if (SignalLevel > 0.1)
				{
					// stop time accel, if any
					if (oapiGetTimeAcceleration() > 1)
						oapiSetTimeAcceleration(1);

					CurrentPhoto = AutoTransmitPhoto;

					StartTransmission();
					StartTransmitting();
				}
}


// rad, rad, rad, rad. rad
void Luna3_VL::DistanceFromCoordPairRad(double Lat1, double Lon1, double Lat2, double Lon2, double* pDistance)
{
	double t = sin(Lat1)*sin(Lat2) + cos(Lat1)*cos(Lat2)*cos(Lon2-Lon1);
	if (t > 1)
		t = 1;
	else if (t < -1)
		t = -1;

	*pDistance = acos(t);
}



	// ===== CHECKs_DOs =====

// this should be kept effective, as it is called in everty timestep
void Luna3_VL::CurrentConditionCheck()
{
	switch (ChkDoType)
	{
		// are we still attached to the rocket? 
		case CDT_DETACHED:
			CheckDetached();
			return;

		case CDT_MOONINFL:
			CheckMoonInfluence();
			return;

		case CDT_MOONPERI:
			CheckMoonPericenter();
			return;

		case CDT_PHOTOSTART:
			CheckPhotoStart();
			return;

		case CDT_PHOTOEND:
			CheckPhotoEnd();
			return;

		case CDT_EARTHINFL:
			CheckEarthInfluence();
			return;

		case CDT_TRANSSTART:
			CheckTransmissionStart();
			return;

		case CDT_TRANSEND:
			CheckTransmissionEnd();
			return;

		// nothing, CDT_INERT
		default: 
			return;
	}
}

// are we still attached to the rocket? 
void Luna3_VL::CheckDetached()
{
	if (bAttached)
	{
		// this would indicate our separation from the rocket
		if (GetAttachmentStatus(GetAttachmentHandle(TOPARENT, 0)) == NULL)
			DoDetach();
	}
}

void Luna3_VL::DoDetach()
{
	// pyro sound
	PlayVesselWave3(SoundLibID, SND_PYRO);
			
	// start antennas timer
	timer = 10;

	bAttached = false;

	ChkDoType = CDT_MOONINFL;
}


void Luna3_VL::CheckMoonInfluence()
{
	// update antenna timer
	if (timer > 0)
	{
		timer -= SimDt;
		if (timer <= 0)
			AntennaRotationStart();
	}

	if ( GetSurfaceRef() == hMoon )
		DoEnterMoonInfluence();
}

void Luna3_VL::DoEnterMoonInfluence()
{
	ShowAnnotationMessage("Entering Moon gravity...");

	ChkDoType = CDT_MOONPERI;
}

void Luna3_VL::CheckMoonPericenter()
{
	// depending on Moon proximity, set time accel limit, consider possible impact
	if (Alt < 30000)
		MaxTimeAccel = 1;
	else if (Alt < 300000)
		MaxTimeAccel = 10;
	else if (Alt < 3000000)
		MaxTimeAccel = 100;
	else if (Alt < 30000000)
		MaxTimeAccel = 1000;
	else
		MaxTimeAccel = 10000;

	if (PastAlt > 0)
		if ( PastAlt < Alt )
			DoPassMoonPericenter();

	// update previous step's alt
	PastAlt = Alt;
}

void Luna3_VL::DoPassMoonPericenter()
{
	ShowAnnotationMessage("Passing Moon trajectory pericenter...");

	// reset time accel limit
	MaxTimeAccel = 0;

	ChkDoType = CDT_PHOTOSTART;
}

void Luna3_VL::CheckPhotoStart()
{
	if (Alt > 63500000)
		DoEnterPhotoWindow();
}

void Luna3_VL::DoEnterPhotoWindow()
{
	ShowAnnotationMessage("Entring photo window...");

	// remove time accel, only once
	oapiSetTimeAcceleration(1);

	// limit time accel until photo covers are open
	if (CoverRotationPercent < 1)
		MaxTimeAccel = 1;
	else
		MaxTimeAccel = 10000;

	// introduce a small delay for photo cover
	timer = 5;

	ChkDoType = CDT_PHOTOEND;
}


void Luna3_VL::CheckPhotoEnd()
{
	// update photo cover timer, open cover if not already done
	if (timer > 0)
	{
		timer -= SimDt;
		if (timer <= 0)
		{
			CoverRotationStart();
			MaxTimeAccel = 10000;

			// autopilot: start looking for the Moon!
			if (!bManualControl)
				bTrackMoon = true;
		}
	}

	if (Alt > 67000000)
		DoLeavePhotoWindow();
}

void Luna3_VL::DoLeavePhotoWindow()
{
	// stop tracking moon if it is ON
	bTrackMoon = false;
	ResetOrientThrusters();

	ShowAnnotationMessage("Leaving photo window...");

	// cleanup any residual tracking output
	sprintf(oapiDebugString(), "" );

	// allow any time accel
	MaxTimeAccel = 0;

	ChkDoType = CDT_EARTHINFL;
}

void Luna3_VL::CheckEarthInfluence()
{
	if ( GetSurfaceRef() == hEarth )
		DoEnterEarthInfluence();
}

void Luna3_VL::DoEnterEarthInfluence()
{
	ShowAnnotationMessage("Entering Earth gravity...");

	ChkDoType = CDT_TRANSSTART;
}

void Luna3_VL::CheckTransmissionStart()
{
	if (Alt < ALT_TRANSM_MAX)
		DoEnterTransmissionWindow();
}

void Luna3_VL::DoEnterTransmissionWindow()
{
	ShowAnnotationMessage("Entring transmission window...");

	// remove time accel, only once
	oapiSetTimeAcceleration(1);

	// limit time accel
	MaxTimeAccel = 1000;

	// autopilot: open transmission panel
	if (!bManualControl)
		StartTransmission();

	ChkDoType = CDT_TRANSEND;
}


void Luna3_VL::CheckTransmissionEnd()
{
	// update signal level
	UpdateSignalLevel();

	if (Alt > 50000000)
		DoLeaveTransmissionWindow();
}

void Luna3_VL::DoLeaveTransmissionWindow()
{
	ShowAnnotationMessage("Leaving transmission window...");

	sprintf(oapiDebugString(), "" );

	// reset time accel
	MaxTimeAccel = 0;

	// show failure banner here, if failed
	bool bMissionFailed = true;
	
	for (int i=1; i<=PhotoCount; i++)
		if (bTransmittedPhotos[i] && bSuccessfulPhotos[i])
			bMissionFailed = false;

	if (bMissionFailed)
		ShowAnnotationMessage("Ups! Mission failed: no Far Side photo... Keep trying!");

	// safing
	ChkDoType = CDT_INERT;
}



	// ===== ANIMATIONS =====


void Luna3_VL::DefineAnimations()
{
	// Antennas
	MGROUP_ROTATE *Antennas[4];

	static UINT grAntenna0[2] = {5, 6}; 
	static UINT grAntenna1[2] = {7, 8}; 
	static UINT grAntenna2[2] = {3, 4}; 
	static UINT grAntenna3[2] = {0, 9}; 

	Antennas[0] = new MGROUP_ROTATE(0, grAntenna0, 2, ANTEN00_REF, ANTEN00_AXIS,  ANTENNA_ROTATION_ANGLE);
	Antennas[1] = new MGROUP_ROTATE(0, grAntenna1, 2, ANTEN01_REF, ANTEN01_AXIS, -ANTENNA_ROTATION_ANGLE);
	Antennas[2] = new MGROUP_ROTATE(0, grAntenna2, 2, ANTEN02_REF, ANTEN02_AXIS,  ANTENNA_ROTATION_ANGLE);
	Antennas[3] = new MGROUP_ROTATE(0, grAntenna3, 2, ANTEN03_REF, ANTEN03_AXIS, -ANTENNA_ROTATION_ANGLE);

	AntennaRotation = CreateAnimation (0);

	AddAnimationComponent (AntennaRotation, 0, 1, Antennas[0]);
	AddAnimationComponent (AntennaRotation, 0, 1, Antennas[1]);
	AddAnimationComponent (AntennaRotation, 0, 1, Antennas[2]);
	AddAnimationComponent (AntennaRotation, 0, 1, Antennas[3]);

	// Tapes 
	static UINT grTapes1[1] = {18}; 
	static UINT grTapes2[1] = {1}; 

	// tapes extesion
	MGROUP_SCALE *TapesScale[2];

	TapesScale[0] = new MGROUP_SCALE(0, grTapes1, 1, _V( 0.146, 0, -0.567), _V(1, 1, 20));
	TapesScale[1] = new MGROUP_SCALE(0, grTapes2, 1, _V(-0.146, 0, -0.567), _V(1, 1, 20));

	TapeExtension = CreateAnimation (0);

	AddAnimationComponent (TapeExtension, 0, 1, TapesScale[0]);
	AddAnimationComponent (TapeExtension, 0, 1, TapesScale[1]);

	// tapes rotation
	MGROUP_ROTATE *TapesRot[2];

	TapesRot[0] = new MGROUP_ROTATE(0, grTapes1, 1, _V( 0.146, 0, -0.567), _V(0, 1, 0), (float)(-45*RAD));
	TapesRot[1] = new MGROUP_ROTATE(0, grTapes2, 1, _V(-0.146, 0, -0.567), _V(0, 1, 0), (float) (45*RAD));

	TapeRotation = CreateAnimation (0);

	AddAnimationComponent (TapeRotation, 0, 1, TapesRot[0]);
	AddAnimationComponent (TapeRotation, 0, 1, TapesRot[1]);

	// Covers
	MGROUP_ROTATE *Covers[2];

	static UINT grCovers0[1] = {14}; 
	static UINT grCovers1[1] = {13}; 

	Covers[0] = new MGROUP_ROTATE(0, grCovers0, 1, _V(-0.09,   0.096, 0.617), _V(0.7071, 0.7071, 0), (float)(-165*RAD));
	Covers[1] = new MGROUP_ROTATE(0, grCovers1, 1, _V( 0.096, -0.09,  0.617), _V(0.7071, 0.7071, 0), (float)( 165*RAD));

	CoverRotation = CreateAnimation (0);

	AddAnimationComponent (CoverRotation, 0, 1, Covers[0]);
	AddAnimationComponent (CoverRotation, 0, 1, Covers[1]);
}



void Luna3_VL::Animate()
{
	if (CurrentAnimation == AT_ANTENNA)
		AnimateAntennaRotation();
	
	else if (CurrentAnimation == AT_TAPE_EXT)
		AnimateTapeExtension();

	else if (CurrentAnimation == AT_TAPE_ROT)
		AnimateTapeRotation();

	else if (CurrentAnimation == AT_COVER_ROT)
		AnimateCoverRotation();
}


void Luna3_VL::AnimateAntennaRotation()
{
	double dPercent = ANTENNA_ROTATION_SPEED * SimDt;

	if (AntennaRotationPercent < 1.0)
		AntennaRotationPercent = min (1.0, AntennaRotationPercent + dPercent);
	else
		AntennaRotationEnd();

	SetAnimation (AntennaRotation, AntennaRotationPercent);
}


void Luna3_VL::AnimateTapeExtension()
{
	double dPercent = TAPE_EXTENSION_SPEED * SimDt;

	if (TapeExtensionPercent < 1.0)
		TapeExtensionPercent = min (1.0, TapeExtensionPercent + dPercent);
	else
		TapeExtensionEnd();

	SetAnimation (TapeExtension, TapeExtensionPercent);
}


void Luna3_VL::AnimateTapeRotation()
{
	double dPercent = TAPE_ROTATION_SPEED * SimDt;

	if (TapeRotationPercent < 1.0)
		TapeRotationPercent = min (1.0, TapeRotationPercent + dPercent);
	else
		TapeRotationEnd();

	SetAnimation (TapeRotation, TapeRotationPercent);
}


void Luna3_VL::AntennaRotationStart()
{
	// ignore multipe restarts
	if (bAnimate)
		return;

	CurrentAnimation = AT_ANTENNA;

	// pyro sound
	PlayVesselWave3(SoundLibID, SND_PYRO);

	// finally, allow animation
	bAnimate = true;
}


// same as TapeExtensionStart()
void Luna3_VL::AntennaRotationEnd()
{
	PlayVesselWave3(SoundLibID, SND_CLANG);

	CurrentAnimation = AT_TAPE_EXT;

	PlayVesselWave3(SoundLibID, SND_MOTOR, LOOP);
}


// same as TapeRotationStart()
void Luna3_VL::TapeExtensionEnd()
{
	StopVesselWave3(SoundLibID, SND_MOTOR);

	CurrentAnimation = AT_TAPE_ROT;
}


void Luna3_VL::TapeRotationEnd()
{
	bAnimate = false;
}


void Luna3_VL::ManualTryCoverRotationStart()
{
	// ignore multipe restarts
	if (bAnimate)
		return;

	// ignore if antennas are not open yet
	if (CurrentAnimation != AT_TAPE_ROT)
		return;

	// ignore if time accel is on
	if (oapiGetTimeAcceleration() > 1)
	{
		ShowAnnotationMessage("Camera cover opening is disabled during time acceleration.");
		return;
	}

	// ignore if autopilot is on
	if (!bManualControl)
	{
		ShowAnnotationMessage("Manual override of camera cover is disabled in autopilot mode.");
		return;
	}

	// start
	CoverRotationStart();
}

void Luna3_VL::CoverRotationStart()
{
	CurrentAnimation = AT_COVER_ROT;

	// motor sound
	PlayVesselWave3(SoundLibID, SND_MOTOR, LOOP);

	// finally, allow animation
	bAnimate = true;
}


void Luna3_VL::AnimateCoverRotation()
{
	double dPercent = COVER_ROTATION_SPEED * SimDt;

	if (CoverRotationPercent < 1.0)
		CoverRotationPercent = min (1.0, CoverRotationPercent + dPercent);
	else
		CoverRotationEnd();

	SetAnimation (CoverRotation, CoverRotationPercent);
}


void Luna3_VL::CoverRotationEnd()
{
	StopVesselWave3(SoundLibID, SND_MOTOR);

	bAnimate = false;
}


// ----------- PANELS --------------

void Luna3_VL::ReleaseSurfaces()
{
	if (surfBtnL != NULL)
	{
		oapiDestroySurface(surfBtnL);
		surfBtnL = 0;
	}

	if (surfBtnR != NULL)
	{
		oapiDestroySurface(surfBtnR);
		surfBtnR = 0;
	}

	if (surfBtnTr != NULL)
	{
		oapiDestroySurface(surfBtnTr);
		surfBtnTr = 0;
	}

	if (surfSignal != NULL)
	{
		oapiDestroySurface(surfSignal);
		surfSignal = 0;
	}
}


bool Luna3_VL::clbkLoadPanel (int id)
{
	if ( (PhotoStep > PH_NONE) && (PhotoStep < PH_TRANSMIT) )
	{
		ShowAnnotationMessage("Photo panel is busy processing the previous photo, please wait...");
		return false;
	}

	// save original visual environment
	SavedHUD = oapiGetHUDMode();
	oapiSetHUDMode(HUD_NONE);

	// not in transmission window? show bare non-interactive "photo film" panel
	if (ChkDoType != CDT_TRANSEND)
	{
		// do panel stuff
		HBITMAP hBmp = LOADBMP ("Textures\\r7_VL\\Panel1.bmp");
		oapiRegisterPanelBackground (hBmp, PANEL_ATTACH_TOP);
		oapiRegisterPanelArea (PHOTO_AREA, _R(100, 100, 700, 700), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_BACKGROUND);
		oapiRegisterPanelArea (NUMBER_AREA, _R(70, 670, 90, 699), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_BACKGROUND);
	}

	// in transmission window? show interactive "transmission center" panel
	else 
	{
		// cleanup from any previous load panel
		ReleaseSurfaces();

		// background
		HBITMAP hBmp = LOADBMP ("Textures\\r7_VL\\Panel2.bmp");
		oapiRegisterPanelBackground (hBmp, PANEL_ATTACH_TOP);

		// photo area and number area
		oapiRegisterPanelArea (PHOTO_AREA, _R(100, 100, 700, 700), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_BACKGROUND);
		oapiRegisterPanelArea (NUMBER_AREA, _R(70, 670, 90, 699), PANEL_REDRAW_USER, PANEL_MOUSE_IGNORE, PANEL_MAP_BACKGROUND);

		// left-right buttons
		surfBtnL = oapiCreateSurface (LOADBMP ("Textures\\r7_VL\\BtnL.bmp"));
		surfBtnR = oapiCreateSurface (LOADBMP ("Textures\\r7_VL\\BtnR.bmp"));
		oapiRegisterPanelArea (BTN_LEFT,  _R(198, 718, 223, 744), PANEL_REDRAW_USER, PANEL_MOUSE_DOWN, PANEL_MAP_BACKGROUND);
		oapiRegisterPanelArea (BTN_RIGHT, _R(578, 718, 603, 744), PANEL_REDRAW_USER, PANEL_MOUSE_DOWN, PANEL_MAP_BACKGROUND);

		// transmit button
		surfBtnTr = oapiCreateSurface (LOADBMP ("Textures\\r7_VL\\BtnTr.bmp"));
		oapiRegisterPanelArea (BTN_TR,  _R(338, 718, 457, 754), PANEL_REDRAW_ALWAYS, PANEL_MOUSE_DOWN, PANEL_MAP_BACKGROUND);

		// signal level indicator
		surfSignal = oapiCreateSurface (LOADBMP ("Textures\\r7_VL\\Signal.bmp"));
		oapiRegisterPanelArea (SIGNAL_AREA,  _R(300, 60, 505, 70), PANEL_REDRAW_ALWAYS, PANEL_MOUSE_IGNORE, PANEL_MAP_BACKGROUND);
	}

	return true;
}


bool Luna3_VL::clbkLoadGenericCockpit ()
{
	// restore visuals
	if (SavedHUD >= HUD_NONE)
	{
		oapiSetHUDMode(SavedHUD);
		SavedHUD = -1;
	}

	return true;
}

bool Luna3_VL::clbkPanelRedrawEvent (int id, int event, SURFHANDLE surf)
{
	// signal level indicator, put at the top as most frequent updater
	if (id == SIGNAL_AREA) 
	{
		oapiBlt (surf, surfSignal, 2, 2, 0, 0, (int)(SignalLevel*200), 6);
		return true;
	}

	// paint start button grey, also redaws in every step
	else if (id == BTN_TR) 
	{
		// important compound condition!
		bTransmitEnabled = 
			(
				(SignalLevel >= 0.1) &&
				(PhotoStep == PH_NONE) &&
				(bTransmittedPhotos[CurrentPhoto] == false)
			);

		if (bTransmitEnabled)	
			oapiBlt (surf, surfBtnTr, 0, 0, 0, 0, 118, 34);

		return true;
	}

	// paint photo area
	else if (id == PHOTO_AREA) 
	{
		oapiBlt (surf, surfPhotos[CurrentPhoto], 0, 0, 0, 0, 600, 600);
		return true;
	}

	// paint frame number area
	else if (id == NUMBER_AREA) 
	{
		char tempStr[5];
		sprintf(tempStr, " %d ", CurrentPhoto);
		HDC DC = oapiGetDC(surf);
        SetTextColor(DC, 0x00585961); 
        SetBkColor(DC,  0x00000000); 
		TextOut(DC, 0, 0, tempStr, 3); 
		oapiReleaseDC (surf, DC);
		return true;
	}

	// paint left button black
	else if (id == BTN_LEFT) 
	{
		if ( (CurrentPhoto > 0) && (PhotoStep == PH_NONE) )
			oapiBlt (surf, surfBtnL, 0, 0, 0, 0, 25, 26);
		return true;
	}

	// paint right button black
	else if (id == BTN_RIGHT) 
	{
		if ( (CurrentPhoto < PhotoCount) && (PhotoStep == PH_NONE) ) 
			oapiBlt (surf, surfBtnR, 0, 0, 0, 0, 25, 26);

		return true;
	}

	return false;
}


bool Luna3_VL::clbkPanelMouseEvent (int id, int event, int mx, int my)
{
	if (id == BTN_TR) 
	{
		if (bTransmitEnabled)
			StartTransmitting();

		return true;
	}

	// the rest of the mouse events are frames navigation buttons

	// ignore event if we are already transmitting
	if (PhotoStep != PH_NONE)
		return true;

	// store current photo number for a moment...
	int OldCurrentPhoto = CurrentPhoto;

	// modify current photo number 
	if (id == BTN_LEFT)
		CurrentPhoto--;
	else if (id == BTN_RIGHT)
		CurrentPhoto++;

	// enforce current photo number limits
	CurrentPhoto = max(0, CurrentPhoto);
	CurrentPhoto = min(CurrentPhoto, PhotoCount);

	// ignore event if current photo number has not changed
	if (OldCurrentPhoto == CurrentPhoto)
		return true;

	// update visuals
	oapiTriggerPanelRedrawArea (0, PHOTO_AREA);
	oapiTriggerPanelRedrawArea (0, NUMBER_AREA);
	oapiTriggerPanelRedrawArea (0, BTN_LEFT);
	oapiTriggerPanelRedrawArea (0, BTN_RIGHT);

	return true;
}


void Luna3_VL::StartTransmitting()
{
	// set transmission session vars
	ScanLine = 0;
	PercentLost = (int)((1-SignalLevel)*39/0.9) + 1; // 1 to 40
	ColorRange = (int) ((SignalLevel-0.1) * 150/0.9) + 50; // 50 to 200
	ColorRangeStart = (255 - ColorRange) / 3; // shift to the dark a little

	// flag the transmission step
	PhotoStep = PH_TRANSMIT;

	// disable navigation buttons
	oapiTriggerPanelRedrawArea (0, BTN_LEFT);
	oapiTriggerPanelRedrawArea (0, BTN_RIGHT);
}


		// SOUND
void Luna3_VL::clbkPostCreation()
{
	SoundLibID = ConnectToOrbiterSoundDLL3(GetHandle());

	SoundOptionOnOff3(SoundLibID, PLAYMAINTHRUST, true);
	SoundOptionOnOff3(SoundLibID, PLAYATTITUDETHRUST, true);
	SoundOptionOnOff3(SoundLibID, PLAYLANDINGANDGROUNDSOUND, true);
	SoundOptionOnOff3(SoundLibID, PLAYWINDAIRSPEED, true);
	SoundOptionOnOff3(SoundLibID, PLAYWINDAMBIANCEWHENLANDED, true);

	SoundOptionOnOff3(SoundLibID, PLAYCOUNTDOWNWHENTAKEOFF, false);
	SoundOptionOnOff3(SoundLibID, PLAYWHENATTITUDEMODECHANGE, false);
	SoundOptionOnOff3(SoundLibID, PLAYGPWS, false);
	SoundOptionOnOff3(SoundLibID, PLAYHOVERTHRUST, false);
	SoundOptionOnOff3(SoundLibID, PLAYDOCKINGSOUND, false);
	SoundOptionOnOff3(SoundLibID, PLAYRADARBIP, false);
	SoundOptionOnOff3(SoundLibID, PLAYDOCKLANDCLEARANCE, false);
	SoundOptionOnOff3(SoundLibID, PLAYCABINAIRCONDITIONING, false);
	SoundOptionOnOff3(SoundLibID, PLAYCABINRANDOMAMBIANCE, false);
	SoundOptionOnOff3(SoundLibID, PLAYRADIOATC, false);
	SoundOptionOnOff3(SoundLibID, DISPLAYTIMER, false);

	RequestLoadVesselWave3(SoundLibID, SND_PYRO, "Sound\\r7_VL\\Pyro.wav", BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(SoundLibID, SND_CLANG, "Sound\\r7_VL\\Clang.wav", BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(SoundLibID, SND_PHOTO, "Sound\\r7_VL\\Photo.wav", BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(SoundLibID, SND_MOTOR, "Sound\\r7_VL\\Motor.wav", BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(SoundLibID, SND_RADIO, "Sound\\r7_VL\\Radio.wav", BOTHVIEW_FADED_FAR);

	ReplaceStockSound3(SoundLibID, "Sound\\r7_VL\\Gas.wav", REPLACE_RCS_THRUST_SUSTAIN);
}


void Luna3_VL::DebugSomething()
{
	sprintf(oapiDebugString(), "test" );
}


void Luna3_VL::LocalRot(const VECTOR3 vecGlob, VECTOR3 &vecLoc)
{
	VESSELSTATUS2 vs;
	double dCosZ, dSinZ, dCosY, dSinY, dCosX, dSinX;
	vs.version = 2;
	vs.flag = 0;
	GetStatusEx((void*) &vs);

	dCosZ = cos(vs.arot.z);
	dSinZ = sin(vs.arot.z);
	dCosY = cos(vs.arot.y);
	dSinY = sin(vs.arot.y);
	dCosX = cos(vs.arot.x);
	dSinX = sin(vs.arot.x);

	vecLoc.x = vecGlob.x*dCosZ*dCosY + vecGlob.y*(-dSinZ*dCosX+dCosZ*dSinY*dSinX) + vecGlob.z*(dSinZ*dSinX+dCosZ*dSinY*dCosX);
	vecLoc.y = vecGlob.x*dSinZ*dCosY + vecGlob.y*(dCosZ*dCosX+dSinZ*dSinY*dSinX) + vecGlob.z*(-dCosZ*dSinX+dSinZ*dSinY*dCosX);
	vecLoc.z = vecGlob.x*-dSinY + vecGlob.y*dCosY*dSinX + vecGlob.z*dCosY*dCosX;
}



DLLCLBK VESSEL *ovcInit(OBJHANDLE hvessel, int flightmodel)
{
	return new Luna3_VL(hvessel, flightmodel);
}

DLLCLBK void ovcExit(VESSEL *vessel)
{
	if (vessel) delete (Luna3_VL*) vessel;
}





