#include	"math.h"
#include	"stdio.h"
#include	"Fairing.h"


r7_BlockE_fairing_VL::r7_BlockE_fairing_VL(OBJHANDLE hObj, int fmodel): VESSEL2M(hObj, fmodel)
{
	// VESSEL2M crash parameters
	VvCrash = DRand(5, 30); 
	VhCrash = DRand(20, 30);
	VMaxBlast = DRand(1000, 1); 
	QMax = DRand(3000, 10000);
}

void r7_BlockE_fairing_VL::clbkSetClassCaps(FILEHANDLE cfg)
{
	// physics and geometry
	SetSize(3.0);
	SetEmptyMass(EMPTY_MASS);
	SetCrossSections(_V(4, 5, 5));
	SetCW (0.1, 1.0, 0.5, 0.5); 
	SetRotDrag (_V(1.1, 1.1, 0.1));
	SetLiftCoeffFunc(0);
	SetPitchMomentScale(1e-6);
	SetBankMomentScale(1e-6);
	SetSurfaceFrictionCoeff(1e5, 1e5);
	SetTouchdownPoints (_V( 0, 0, 2), _V( 2, 1, -2), _V(-2, 2, -2));
	
	// attachment to R7
	ATTACHMENTHANDLE hToR7 = CreateAttachment(TOPARENT, APOINT, ADIR, AROT, "ToR7", NOTLOOSE);
	
	PROPELLANT_HANDLE ph_main = CreatePropellantResource(FUEL_MASS);
	THRUSTER_HANDLE		th_main[4];

	th_main[0] = CreateThruster(T01_REF, T01_DIR, MAX_THRUST, ph_main, MAX_ISP);
	th_main[1] = CreateThruster(T02_REF, T02_DIR, MAX_THRUST*2, ph_main, MAX_ISP);
	th_main[2] = CreateThruster(T03_REF, T03_DIR, MAX_THRUST, ph_main, MAX_ISP);
	th_main[3] = CreateThruster(T04_REF, T04_DIR, MAX_THRUST, ph_main, MAX_ISP);

	// for debuging exhaust placement; also, add more fuel
//	THGROUP_HANDLE thg_main = CreateThrusterGroup(th_main, 4, THGROUP_MAIN);

	PARTICLESTREAMSPEC ExhaustStream = { 0,			// flags, reserved
		0.1,			// srcsize, particle size at creation, m
		100,			// srcrate, average particle generation rate, Hz   
		10,				// V0, average particle emission velocity, m/s
		0.01,			// srcspread, emission velocity distribution randomisation
		0.1,			// lifetime, average particle lifetime [s]
		0.5,			// growthrate, particle growth rate [m/s]
		0.1,			// atmslowdown, deceleration rate b in atmosphere, defined as v = v0 e- bdt 
		PARTICLESTREAMSPEC::EMISSIVE, PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5, PARTICLESTREAMSPEC::ATM_PLIN, 0, 0 };

	AddExhaustStream(th_main[0], &ExhaustStream);
	AddExhaustStream(th_main[1], &ExhaustStream);
	AddExhaustStream(th_main[2], &ExhaustStream);
	AddExhaustStream(th_main[3], &ExhaustStream);

	// mesh
	AddMesh("r7_VL\\Fairing");
}

void r7_BlockE_fairing_VL::clbkPostStep(double simt, double SimDT, double mjd)
{
	// common processing for all VESSEL2M-derived objects
	TimestepForVESSEL2M();
}


DLLCLBK VESSEL *ovcInit(OBJHANDLE hvessel, int flightmodel)
{
	return new r7_BlockE_fairing_VL(hvessel, flightmodel);
}

DLLCLBK void ovcExit(VESSEL *vessel)
{
	if (vessel) delete (r7_BlockE_fairing_VL*) vessel;
}