//=============================================================================
//
//
//
//
//
//=============================================================================
#define		ORBITER_MODULE
#include	"math.h"
#include	"..\..\..\include\orbitersdk.h"
#include	"stdio.h"
#include	"../math/mymath.h"
#include	"../quaternion/vectors.h"
#include	"../quaternion/matrix.h"
#include	"../quaternion/quaternion.h"
#include	"Launchpad.h"

HINSTANCE		h_DLL;

const		double		MASS = 2e6;
const		double		PMASS = 1000;
const		double		H = -6+7.798;
const		double		R = 7.5;
const		double		R2 = 7.5;
const		double		HFerm01 = 4+7.798;
const		double		HFerm02 = 2+7.798;
const		double		PSANGLE = 60*RAD;

const		VECTOR3		APOINT = {0, H, 0};
VECTOR3		ADIR = {0, 1, 0};
VECTOR3		AROT = {1, 0, 0};
const		VECTOR3		FERMA01_OFS = {-R, HFerm01, 0};
const		VECTOR3		FERMA02_OFS = {-R2, HFerm02, 0};
const		VECTOR3		KOTL_OFF = {105, 0, 12};
const		VECTOR3		L_RING_OFF = {0, 2+7.798, 0};
const		VECTOR3		TABLE_OFF = {0, 0, 0};
float phiT = 57.33;
float lambdaT = 161.83;
float Inc = 65.6*RAD;
float Ha = 245*1000;
float Hp = 193*1000;
float Alaunch = 0;
int	  setazimut = 0;

const		int			READY = 0;
const		int			FREE = 1;
const		int			BALLISTIC = 0;
const		int			ORBIT = 1;
const		int			R7_BALLISTIC = 0;
const		int			R7_SOYUZ = 3;
const		int			R7_VOSTOK = 2;
const		int			R7_PS1 = 1;

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
DLLCLBK	void InitModule(HINSTANCE hModule)
{
	h_DLL = hModule;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
DLLCLBK void ExitModule(HINSTANCE hModule)
{

}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
bool InputLat(void *id, char *str, void *data)
{
	sscanf(str, "%f", &phiT);
	return true;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
bool InputLong(void *id, char *str, void *data)
{
	sscanf(str, "%f", &lambdaT);
	return true;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
bool InputInc(void *id, char *str, void *data)
{
	sscanf(str, "%f", &Inc);
	Inc = Inc*RAD;
	setazimut = false;
	return true;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
bool InputApogee(void *id, char *str, void *data)
{
	sscanf(str, "%f", &Ha);
	Ha = Ha*1000;
	return true;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
bool InputPerigee(void *id, char *str, void *data)
{
	sscanf(str, "%f", &Hp);
	Hp = Hp*1000;
	return true;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
bool InputAzimut(void *id, char *str, void *data)
{
	sscanf(str, "%f", &Alaunch);
	Alaunch = Alaunch*RAD;
	setazimut = true;
	return true;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
R7_launchpad::R7_launchpad(OBJHANDLE hObj, int fmodel): VESSEL2(hObj, fmodel)
{
	ptex = oapiRegisterParticleTexture("Contrail3");

	memset(&psc1, 0, sizeof(psc1));
	psc1.srcsize = 7.5;
	psc1.srcrate = 10;
	psc1.v0 = 40;
	psc1.srcspread = 0.01;
	psc1.lifetime = 4;
	psc1.growthrate = 10;
	psc1.atmslowdown = 0.1;
	psc1.ltype = PARTICLESTREAMSPEC::DIFFUSE;
	psc1.levelmap = PARTICLESTREAMSPEC::LVL_PSQRT;
	psc1.lmin = 0;
	psc1.lmax = 1.0;
	psc1.atmsmap = PARTICLESTREAMSPEC::ATM_PLIN;
	psc1.amin = 0.0;
	psc1.amax = 0.0;
	psc1.tex = ptex;

	A = 0*RAD;
	OmegaA = 5*RAD;

	Beta = 0;
	OmegaBeta = 5*RAD;

	Psi = 0;
	OmegaPsi = 0;

	PhiT = 57.33*RAD;
	LambdaT = 161.83*RAD;

	Ready01 = Ready02 = Ready03 = Ready04 = FirstStep = false;
	Start = false;

	LaunchState = READY;
	STime = 0;
	Time = 0;

	Far = 0;
	Natt = 0;	
	CalcLock = true;
	CalcStart = false;
	psi0 = 74*RAD;
	Alaunch = (34 + 37/60 + 59.2/3600)*RAD;

	FreeTime = 0;
	h0 = 0;

	plevel = 0;
	begin = 0;

	StartAtDate = false;
	locksad = false;

	IncScn = 0;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_launchpad::clbkLoadStateEx(FILEHANDLE scn, void *status)
{
	char *line;

	while (oapiReadScenario_nextline(scn, line))
	{
		if (!strnicmp (line, "ALAUNCH", 7))
		{
			sscanf (line+7, "%f", &Alaunch);
		}
		
		if (!strnicmp (line, "A", 1))
		{
			sscanf (line+1, "%f", &A);
		}
		
		if (!strnicmp (line, "BETA", 4))
		{
			sscanf (line+4, "%f", &Beta);
		}
		
		if (!strnicmp (line, "PSI", 3))
		{
			sscanf (line+3, "%f", &Psi);
		}

		if (!strnicmp (line, "OMEGAPSI", 8))
		{
			sscanf (line+8, "%f", &OmegaPsi);
		}
		
		if (!strnicmp (line, "READY01", 7))
		{
			sscanf (line+7, "%i", &Ready01);
		}
		
		if (!strnicmp (line, "READY02", 7))
		{
			sscanf (line+7, "%i", &Ready02);
		}
		
		if (!strnicmp (line, "READY03", 7))
		{
			sscanf (line+7, "%i", &Ready03);
		} 
		
		if (!strnicmp (line, "READY04", 7))
		{
			sscanf (line+7, "%i", &Ready04);
		}

		if (!strnicmp (line, "FIRSTSTEP", 9))
		{
			sscanf (line+9, "%i", &FirstStep);
		}
		
		if (!strnicmp (line, "START", 5))
		{
			sscanf (line+5, "%i", &Start);
		}
		
		if (!strnicmp (line, "STATE", 5))
		{
			sscanf (line+5, "%i", &LaunchState);
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "FAR", 3))
		{
			sscanf (line+3, "%i", &Far);
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "TIME", 4))
		{
			sscanf (line+4, "%f", &Time);
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "STIME", 5))
		{
			sscanf (line+5, "%f", &STime);
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "FTIME", 5))
		{
			sscanf (line+5, "%f", &FreeTime);
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "CONFIG", 6))
		{
			sscanf (line+6, "%i", &config);
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}
		
		if (!strnicmp (line, "SETAZIMUT", 9))
		{
			sscanf (line+9, "%i", &setazimut);
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "LAUNCHERTYPE", 12))
		{
			sscanf (line+12, "%i", &LauncherType);
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "LAUNCHER", 8))
		{
			sscanf (line+8, "%s", &LauncherName);
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "H0", 2))
		{
			float h1;
			sscanf (line+2, "%f", &h1);
			h0 = (double) h1;
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "PARLEVEL", 8))
		{
			float pl;
			sscanf (line+8, "%f", &pl);
			plevel = (double) pl;
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "BEGIN", 5))
		{
			sscanf (line+5, "%i", &begin);
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "STARTMJD", 8))
		{
			double mjd;
			sscanf (line+8, "%f", &mjd);
			StartMJD = mjd;
			StartAtDate = true;
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "LOCKSAD", 7))
		{
			sscanf (line+7, "%i", &locksad);
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "INC", 3))
		{
			sscanf (line+3, "%f", &IncScn);
		} 
		else 
		{
			ParseScenarioLineEx (line, status);
		}
	}
	
	if (LaunchState == READY)
	{
		//AttachChild(hR7Obj, hR7Att, R7->GetAttachmentHandle(true, 0));
	}

	hLauncher = oapiGetVesselByName(LauncherName);
	
	switch (LauncherType)
	{
	case R7_BALLISTIC:
		{
			hFerma01 = oapiLoadMeshGlobal("r7_base_frame");
			hFerma02 = oapiLoadMeshGlobal("r7_service_frame");
			SAngleMax = 97*RAD;

			AddMesh(hFerma01, &FERMA01_OFS); //0
			AddMesh(hFerma01, &FERMA01_OFS); //1
			AddMesh(hFerma01, &FERMA01_OFS); //2
			AddMesh(hFerma01, &FERMA01_OFS); //3
			AddMesh(hFerma02, &FERMA02_OFS); //4
			AddMesh(hFerma02, &FERMA02_OFS); //5
			AddMesh(oapiLoadMeshGlobal("r7_kotlovan"), &KOTL_OFF); //6
			AddMesh(oapiLoadMeshGlobal("r7_table"), &TABLE_OFF); //7
			AddMesh(oapiLoadMeshGlobal("r7_launch_ring"), &L_RING_OFF); //8
			break;
		}
	case R7_PS1:
		{
			hFerma01 = oapiLoadMeshGlobal("r7_base_frame");
			hFerma02 = oapiLoadMeshGlobal("r7_service_frame");
			SAngleMax = 97*RAD;

			AddMesh(hFerma01, &FERMA01_OFS); //0
			AddMesh(hFerma01, &FERMA01_OFS); //1
			AddMesh(hFerma01, &FERMA01_OFS); //2
			AddMesh(hFerma01, &FERMA01_OFS); //3
			AddMesh(hFerma02, &FERMA02_OFS); //4
			AddMesh(hFerma02, &FERMA02_OFS); //5
			AddMesh(oapiLoadMeshGlobal("r7_kotlovan"), &KOTL_OFF); //6
			AddMesh(oapiLoadMeshGlobal("r7_table"), &TABLE_OFF); //7
			AddMesh(oapiLoadMeshGlobal("r7_launch_ring"), &L_RING_OFF); //8

			Ha = 947*1000;
			Inc = 66.7*RAD;

			break;
		}
	case R7_SOYUZ:
		{
			hFerma01 = oapiLoadMeshGlobal("r7_base_frame");
			hFerma02 = oapiLoadMeshGlobal("r7_service_frame_soyuz_001");
			hFerma03 = oapiLoadMeshGlobal("r7_service_frame_soyuz_002");
			SAngleMax = 90*RAD;

			AddMesh(hFerma01, &FERMA01_OFS); //0
			AddMesh(hFerma01, &FERMA01_OFS); //1
			AddMesh(hFerma01, &FERMA01_OFS); //2
			AddMesh(hFerma01, &FERMA01_OFS); //3
			AddMesh(hFerma02, &FERMA02_OFS); //4
			AddMesh(hFerma03, &FERMA02_OFS); //5
			AddMesh(oapiLoadMeshGlobal("r7_kotlovan"), &KOTL_OFF); //6
			AddMesh(oapiLoadMeshGlobal("r7_table"), &TABLE_OFF); //7
			AddMesh(oapiLoadMeshGlobal("r7_launch_ring"), &L_RING_OFF); //8

			Ha = 245*1000;
			Inc = 51.7*RAD;

			break;
		}
	case R7_VOSTOK:
		{
			hFerma01 = oapiLoadMeshGlobal("r7_VL\\Launch_Base");
			hFerma02 = oapiLoadMeshGlobal("r7_VL\\Launch_Tower");
			SAngleMax = 90*RAD;

			AddMesh(hFerma01, &FERMA01_OFS); //0
			AddMesh(hFerma01, &FERMA01_OFS); //1
			AddMesh(hFerma01, &FERMA01_OFS); //2
			AddMesh(hFerma01, &FERMA01_OFS); //3
			AddMesh(hFerma02, &FERMA02_OFS); //4
			AddMesh(hFerma02, &FERMA02_OFS); //5
			AddMesh(oapiLoadMeshGlobal("r7_kotlovan"), &KOTL_OFF); //6
			AddMesh(oapiLoadMeshGlobal("r7_table"), &TABLE_OFF); //7
			AddMesh(oapiLoadMeshGlobal("r7_VL\\Launch_Ring"), &L_RING_OFF); //8

			Ha = 245*1000;
			Inc = 51.7*RAD;

			break;
		}
	}	//case

	if (IncScn > 0)
		Inc = IncScn*RAD;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_launchpad::clbkSaveState(FILEHANDLE scn)
{
	VESSEL2::clbkSaveState(scn);
	oapiWriteScenario_int(scn, "STATE", LaunchState);
	oapiWriteScenario_int(scn, "READY01", Ready01);
	oapiWriteScenario_int(scn, "READY02", Ready02);
	oapiWriteScenario_int(scn, "READY03", Ready03);
	oapiWriteScenario_int(scn, "READY04", Ready04);
	oapiWriteScenario_int(scn, "FIRSTSTEP", FirstStep);
	oapiWriteScenario_int(scn, "START", Start);
	oapiWriteScenario_float(scn, "A", A);
	oapiWriteScenario_float(scn, "BETA", Beta);
	oapiWriteScenario_float(scn, "PSI", Psi);
	oapiWriteScenario_float(scn, "OMEGAPSI", OmegaPsi);
	oapiWriteScenario_float(scn, "ALAUNCH", Alaunch);
	oapiWriteScenario_float(scn, "H0", h0);
	oapiWriteScenario_int(scn, "FAR", Far);
	oapiWriteScenario_float(scn, "TIME", Time);
	oapiWriteScenario_float(scn, "STIME", STime);
	oapiWriteScenario_float(scn, "FTIME", FreeTime);
	oapiWriteScenario_int(scn, "CONFIG", config);
	oapiWriteScenario_int(scn, "SETAZIMUT", setazimut);
	oapiWriteScenario_int(scn, "LAUNCHERTYPE", LauncherType);
	oapiWriteScenario_string(scn, "LAUNCHER", LauncherName);
	oapiWriteScenario_float(scn, "PARLEVEL", plevel);
	oapiWriteScenario_int(scn, "BEGIN", begin);
	if (StartAtDate) oapiWriteScenario_float(scn, "STARTMJD", StartMJD);
	oapiWriteScenario_int(scn, "LOCKSAD", locksad);

	if (IncScn > 0)
		oapiWriteScenario_float(scn, "INC", IncScn);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_launchpad::clbkSetClassCaps(FILEHANDLE cfg)
{
	SetEmptyMass(MASS);
	
	AddParticleStream(&psc1, _V(0, -10, 0), _V(1, 0, 0), &plevel);
	AddParticleStream(&psc1, _V(0, -10, 0), _V(0, 0, 1), &plevel);
	AddParticleStream(&psc1, _V(0, -10, 0), _V(0, 0, -1), &plevel);

	hR7Att = CreateAttachment(false,
		                      APOINT,
							  ADIR,
							  AROT,
							  "ForR7",
							  false);

	SetSurfaceFrictionCoeff(1e5, 1e5);
	SetTouchdownPoints(_V(0, 0.01, -1), _V(1, 0.01, 0), _V(-1, 0.01, 0));
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_launchpad::clbkPostStep(double simt, double SimDT, double mjd)
{
	// if launch pad is in its final stage, and nothing can change anymore - 
	// become completely intert and stop doing anything 
	// (like wasting cycles, printing obsolete messages, hogging focus, etc.)
	if (Ready04)
		return;

	if ( (Natt == 0) && (LaunchState == READY) )
	{
 		hR7Obj = GetAttachmentStatus(hR7Att);
		R7 = oapiGetVesselInterface(hR7Obj);
		hR7 = (R7_second_stage*) R7;
		Natt++;
	}
	
	AROT = _V(sin(A), 0, cos(A));
	SetAttachmentParams(hR7Att, APOINT, ADIR, AROT);
	
	if ( (begin) && (!Ready01) )
	{
		BeginStart();
	}
	
	if (Ready01)
	{
		if (!Ready04) AzimutRotation(Alaunch, SimDT);
	}

	if (Ready02)
	{
		if (!Ready04) ServiceFermsRotation(SAngleMax, SimDT);
	}
	
	if ( (Ready03) && (!Start) )
	{
		hR7->StartLaunchProg();
		Start = true;
	}

	bool Launch = true;
	if ( (LaunchState != FREE) )
	{
		Launch = hR7->LaunchpadFree();		
	}

	if (Launch)
	{
		FreeTime = FreeTime + SimDT;
		double Vdt = 1.5*G*SimDT;
		OmegaPsi = Vdt/R;
		/*if ( (!Ready04) && (FreeTime > 5*SimDT) )*/ //LaunchFermsRotation(55*RAD, SimDT);
		DetachChild(hR7Att, Vdt);		
		LaunchState = FREE;
	}

	if ( (LaunchState == READY) || (Time - STime <= 16) ) 
	{
		double fslevel = hR7->GetFirstStageLevel();
		plevel = exp(2*log(fslevel + 0.001));
		if (plevel > 1.0) plevel = 1.0;
	}
	else
	{
		plevel = 0;
	}

	if (CalcStart)
	{
		sprintf(oapiDebugString(), "Launch trajectory calculated. Please wait...");
		CalcLock = false;
		if ( (!setazimut) ) Alaunch = GetLaunchAzimut(phiT*RAD, lambdaT*RAD);
	}

	if ( (StartAtDate) && (!locksad) )
	{
		if (mjd - StartMJD >= 0)
			StartLaunchOperations();
		else
		{
			double d, h, m, s;
			d = StartMJD - mjd;
			h = (d - (double)((int) d))*24;
			m = (h - (double)((int) h))*60;
			s = (m - (double)((int) m))*60;
			sprintf(oapiDebugString(), "Time to launch %i:%i:%i:%i", (int) d, (int) h, (int) m, (int) s);
		}
	}
	else
	{
		if (!begin) sprintf(oapiDebugString(), "Start prerare stoped. Launch manualy by press <S>");
	}

	// moving this to the end of the procedure, as it might set the flag that turns off timestep processing.
	if (LaunchState == FREE) LaunchFermsRotation(55*RAD, SimDT);

	Time = Time + SimDT;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_launchpad::clbkVisualCreated(VISHANDLE visual, int refcount)
{
	if (refcount > 1) return;
	vis = visual;
	Far = 1;	

	Ferms01LoadRotation();
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_launchpad::clbkVisualDestroyed(VISHANDLE visual, int refcount)
{
	if (visual == vis) 
	{
		vis = NULL;
		Far = 0;
	}
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
int R7_launchpad::clbkConsumeDirectKey(char *keystate)
{
	
	if (KEYDOWN(keystate, OAPI_KEY_V))
	{
		CalcStart = true;
		sprintf(oapiDebugString(), "Launch trajectory calculated. Please wait...");
	}

	if (KEYDOWN(keystate, OAPI_KEY_S))
	{
		StartLaunchOperations();
	}
	
	switch (config)
	{
	case BALLISTIC:
		{
			if (KEYDOWN(keystate, OAPI_KEY_A))
			{
				oapiOpenInputBox("Latitude of taget input (from -90 to 90 deg.)", InputLat, 0, 25, 0);		
			}

			if (KEYDOWN(keystate, OAPI_KEY_N))
			{
				oapiOpenInputBox("Longtitude of taget input (from -180 to 180 deg.)", InputLong, 0, 25, 0);		
			}
			break;
		}
	case ORBIT:
		{
			if (KEYDOWN(keystate, OAPI_KEY_E))
			{
				oapiOpenInputBox("Orbit inc. input (not low than you current latitude)", InputInc, 0, 25, 0);		
			}

			if (KEYDOWN(keystate, OAPI_KEY_A))
			{
				oapiOpenInputBox("Orbit apogee input (km)", InputApogee, 0, 25, 0);		
			}

			/*if (KEYDOWN(keystate, OAPI_KEY_P))
			{
				oapiOpenInputBox("Orbit perigee input (km)", InputPerigee, 0, 25, 0);		
			}*/
			

			if (KEYDOWN(keystate, OAPI_KEY_W))
			{
				oapiOpenInputBox("Launch azimut (degres)", InputAzimut, 0, 25, 0);		
			}
			break;
		}
	}

	return 0;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
int R7_launchpad::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
	if ( (key == OAPI_KEY_H) && (!down) )
	{
		locksad = !locksad;		
		return 1;
	}	
	return 0;
}

//=============================================================================
//
//=============================================================================
DLLCLBK VESSEL *ovcInit(OBJHANDLE hvessel, int flightmodel)
{
	return new R7_launchpad(hvessel, flightmodel);
}

DLLCLBK void ovcExit(VESSEL *vessel)
{
	if (vessel) delete (R7_launchpad*) vessel;
}

//=============================================================================
//
//=============================================================================

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_launchpad::AzimutRotation(double Az, double dt)
{
	double eps = 1*RAD;
	double dA = Az - A;
	double A0 = A;
	
	if (ABS(dA) < eps)
	{
		A = Az;
		OmegaA = 0.0;
		Ready02 = true;
	}
	else
		if (!Ready02) A = A + OmegaA*dt*sign(dA);

	if (!vis) return;
	mt0.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt0.nmesh = 8;
	mt0.ngrp = -1;
	mt0.P.rotparam.angle = (float)(-(A - A0));
	mt0.P.rotparam.ref = _V(0, 0, 0);
	mt0.P.rotparam.axis = _V(0, 1, 0);
	if (Far) MeshgroupTransform(vis, mt0);
	
	mt1.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt1.nmesh = 0;//1;
	mt1.ngrp = -1;
	mt1.P.rotparam.angle = (float)(-(A - A0));
	mt1.P.rotparam.ref = _V(R, 0, 0);
	mt1.P.rotparam.axis = _V(0, 1, 0);
	if (Far) MeshgroupTransform(vis, mt1);

	mt2.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt2.nmesh = 1;//2;
	mt2.ngrp = -1;
	mt2.P.rotparam.angle = (float)(-(A - A0));
	mt2.P.rotparam.ref = _V(R, 0, 0);
	mt2.P.rotparam.axis = _V(0, 1, 0);
	if (Far) MeshgroupTransform(vis, mt2);

	mt3.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt3.nmesh = 2;///3;
	mt3.ngrp = -1;
	mt3.P.rotparam.angle = (float)(-(A - A0));
	mt3.P.rotparam.ref = _V(R, 0, 0);
	mt3.P.rotparam.axis = _V(0, 1, 0);
	if (Far) MeshgroupTransform(vis, mt3);

	mt4.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt4.nmesh = 3;//4;
	mt4.ngrp = -1;
	mt4.P.rotparam.angle = (float)(-(A - A0));
	mt4.P.rotparam.ref = _V(R, 0, 0);
	mt4.P.rotparam.axis = _V(0, 1, 0);
	if (Far) MeshgroupTransform(vis, mt4);

	mt5.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt5.nmesh = 4;//;
	mt5.ngrp = -1;
	mt5.P.rotparam.angle = (float)(-(A - A0));
	mt5.P.rotparam.ref = _V(R2, 0, 0);
	mt5.P.rotparam.axis = _V(0, 1, 0);
	if (Far) MeshgroupTransform(vis, mt5);

	mt6.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt6.nmesh = 5;//6;
	mt6.ngrp = -1;
	mt6.P.rotparam.angle = (float)(-(A - A0));
	mt6.P.rotparam.ref = _V(R2, 0, 0);
	mt6.P.rotparam.axis = _V(0, 1, 0);
	if (Far) MeshgroupTransform(vis, mt6);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
double R7_launchpad::GetLaunchAzimut(double phi, double lambda)
{
	double OMEGA0 = 2*PI/3600/24;
	double phi0, lambda0, r0;
	double Hk, alpha;
	double tp, tp0;
	double lambda1;
	double eps = 1e-3;
	double Vground;
	double e, E, E0, theta, theta0, _r0, sin_x, cos_x, Psi0, _v0, p, a, vk, azimut, alp, v;
	double Re = 6.371e6;
	double Me = 5.98e24;

	VECTOR3 V;
	
	switch (config)
	{
	case BALLISTIC:
		{
			GetEquPos(lambda0, phi0, r0);
			Hk = 225*1000;
			alpha = 14*RAD;
			tp = tp0 = 0;
			lambda1 = lambda;
			GetRelativeVel(oapiGetGbodyByName("Earth"), V);
			Vground = length(V);
			hR7->SetVground(Vground);

			do
			{
				tp0 = tp;
				lambda = lambda1 + OMEGA0*tp;
				Psi0 = acos(sin(phi0)*sin(phi) + cos(lambda - lambda0)*cos(phi0)*cos(phi));
				_r0 = 1 + Hk/Re;
				_v0 = 2*(1 + tan(alpha)*tan(alpha))*tan(Psi0/2)*tan(Psi0/2)/( (1 + _r0)*tan(Psi0/2)*tan(Psi0/2) + 2*tan(alpha)*tan(Psi0/2) + _r0 - 1 );
				vk = sqrt(GGRAV*Me*_v0/(Re + Hk));
				sin_x = sin(lambda - lambda0)*cos(phi)/sin(Psi0);
				cos_x = ( sin(phi) - cos(Psi0)*sin(phi0) )/sin(Psi0)/cos(phi0);
				azimut = atan2(sin_x, cos_x);

				V = _V( vk*cos(alpha)*sin(azimut) - Vground, vk*cos(alpha)*cos(azimut), vk*sin(alpha) );
				v = length(V);
				alp = asin(vk*sin(alpha)/v);
				sin_x = V.x/v/cos(alp);
				cos_x = V.y/v/cos(alp);
				azimut = atan2(sin_x, cos_x);

				p = (Re + Hk)*_v0*cos(alpha)*cos(alpha);
				e = sqrt( 1 - (2 - _v0)*_v0*cos(alpha)*cos(alpha) );
				theta0 = acos( (p/(Re + Hk) - 1)/e );
				theta = acos( (p/Re - 1)/e );
				a = (Re + Hk)/(2 - _v0);
				E0 = 2*atan( tan(theta0/2)*sqrt((1 - e)/(1 + e)) );
				E = 2*PI - 2*atan( tan(theta/2)*sqrt((1 - e)/(1 + e)) );
				tp = sqrt(a)*a*(E - E0 - e*(sin(E) - sin(E0)))/sqrt(GGRAV*Me);

			}
			while (ABS(tp - tp0) >= eps);

			break;
		}
	case ORBIT:
		{
			GetEquPos(lambda0, phi0, r0);
			double x = cos(Inc)/cos(phi0);
			if (fabs(x) > 1) x = sign(x);
			azimut = asin(x);			

			CalcStart = false;
			CalcLock = true;
			if (hR7->GetConfig() != 1)
				sprintf(oapiDebugString(), "Done. Azimuth = %f", azimut*DEG);
			else
				sprintf(oapiDebugString(), "Ok, the first statellite ready for start!");
			
			Vk0 = vk;
			break;
		}
	}

	return azimut;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_launchpad::Ferms01LoadRotation(void)
{
	if (!vis) return;

	mt0.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt0.nmesh = 8;
	mt0.ngrp = -1;
	mt0.P.rotparam.angle = (float)(-A);
	mt0.P.rotparam.ref = _V(0, 0, 0);
	mt0.P.rotparam.axis = _V(0, 1, 0);
	if (Far) MeshgroupTransform(vis, mt0);
	
	mt1.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt1.nmesh = 0;//1;
	mt1.ngrp = -1;
	mt1.P.rotparam.angle = (float)(-A);
	mt1.P.rotparam.ref = _V(R, 0, 0);
	mt1.P.rotparam.axis = _V(0, 1, 0);
	if (Far) MeshgroupTransform(vis, mt1);

	mt2.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt2.nmesh = 1;//2;
	mt2.ngrp = -1;
	mt2.P.rotparam.angle = (float)(-90*RAD - A);
	mt2.P.rotparam.ref = _V(R, 0, 0);
	mt2.P.rotparam.axis = _V(0, 1, 0);
	if (Far) MeshgroupTransform(vis, mt2);

	mt3.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt3.nmesh = 2;//3;
	mt3.ngrp = -1;
	mt3.P.rotparam.angle = (float)(-180*RAD - A);
	mt3.P.rotparam.ref = _V(R, 0, 0);
	mt3.P.rotparam.axis = _V(0, 1, 0);
	if (Far) MeshgroupTransform(vis, mt3);

	mt4.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt4.nmesh = 3;//4;
	mt4.ngrp = -1;
	mt4.P.rotparam.angle = (float)(-270*RAD - A);
	mt4.P.rotparam.ref = _V(R, 0, 0);
	mt4.P.rotparam.axis = _V(0, 1, 0);
	if (Far) MeshgroupTransform(vis, mt4);

	mt5.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt5.nmesh = 4;//5;
	mt5.ngrp = -1;
	mt5.P.rotparam.angle = (float)(-(A + 45*RAD));
	mt5.P.rotparam.ref = _V(R2, 0, 0);
	mt5.P.rotparam.axis = _V(0, 1, 0);
	if (Far) MeshgroupTransform(vis, mt5);

	mt6.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt6.nmesh = 5;//6;
	mt6.ngrp = -1;
	mt6.P.rotparam.angle = (float)(-(A + 225*RAD));
	mt6.P.rotparam.ref = _V(R2, 0, 0);
	mt6.P.rotparam.axis = _V(0, 1, 0);
	if (Far) MeshgroupTransform(vis, mt6);
	
	mt5.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt5.nmesh = 4;//5;
	mt5.ngrp = -1;
	mt5.P.rotparam.angle = (float)(Beta);
	mt5.P.rotparam.ref = _V(R2*(1 - cos(A + 45*RAD)), 0, R2*sin(A + 45*RAD));
	mt5.P.rotparam.axis = _V(sin(A + 45*RAD), 0, cos(A + 45*RAD));
	if (Far) MeshgroupTransform(vis, mt5);
	
	mt6.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt6.nmesh = 5;//6;
	mt6.ngrp = -1;
	mt6.P.rotparam.angle = (float)(Beta);
	mt6.P.rotparam.ref = _V(R2*(1 + cos(A + 45*RAD)), 0, -R2*sin(A + 45*RAD));
	mt6.P.rotparam.axis = _V(sin(A + 225*RAD), 0, cos(A + 225*RAD));
	if (Far) MeshgroupTransform(vis, mt6);

	mt1.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt1.nmesh = 0;//1;
	mt1.ngrp = -1;
	mt1.P.rotparam.angle = (float)(Psi);
	mt1.P.rotparam.ref = _V(R*(1 - cos(A)), 0, R*sin(A));
	mt1.P.rotparam.axis = _V(sin(A), 0, cos(A));
	if (Far) MeshgroupTransform(vis, mt1);

	mt2.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt2.nmesh = 1;//2;
	mt2.ngrp = -1;
	mt2.P.rotparam.angle = (float)(Psi);
	mt2.P.rotparam.ref = _V(R*(1 - cos(A + 90*RAD)), 0, R*sin(A + 90*RAD));
	mt2.P.rotparam.axis = _V(sin(A + 90*RAD), 0, cos(A + 90*RAD));
	if (Far) MeshgroupTransform(vis, mt2);

	mt3.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt3.nmesh = 2;//3;
	mt3.ngrp = -1;
	mt3.P.rotparam.angle = (float)(Psi);
	mt3.P.rotparam.ref = _V(R*(1 - cos(A + 180*RAD)), 0, R*sin(A + 180*RAD));
	mt3.P.rotparam.axis = _V(sin(A + 180*RAD), 0, cos(A + 180*RAD));
	if (Far) MeshgroupTransform(vis, mt3);

	mt4.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt4.nmesh = 3;//4;
	mt4.ngrp = -1;
	mt4.P.rotparam.angle = (float)(Psi);
	mt4.P.rotparam.ref = _V(R*(1 - cos(A + 270*RAD)), 0, R*sin(A + 270*RAD));
	mt4.P.rotparam.axis = _V(sin(A + 270*RAD), 0, cos(A + 270*RAD));
	if (Far) MeshgroupTransform(vis, mt4);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_launchpad::ServiceFermsRotation(double BetaZ, double dt)
{
	double eps = 1*RAD;
	double dBeta = BetaZ - Beta;
	double Beta0 = Beta;
	
	if (ABS(dBeta) < eps)
	{
		Beta = BetaZ;
		OmegaBeta = 0.0;
		if (!Ready03) STime = oapiGetSimTime();
		Ready03 = true;		
	}
	else
		if (!Ready03) Beta = Beta + OmegaBeta*dt*sign(dBeta);

	if (!vis) return;
	mt5.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt5.nmesh = 4;//5;
	mt5.ngrp = -1;
	mt5.P.rotparam.angle = (float)(Beta - Beta0);
	mt5.P.rotparam.ref = _V(R2*(1 - cos(A + 45*RAD)), 0, R2*sin(A + 45*RAD));
	mt5.P.rotparam.axis = _V(sin(A + 45*RAD), 0, cos(A + 45*RAD));
	if (Far) MeshgroupTransform(vis, mt5);
	
	mt6.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt6.nmesh = 5;//6;
	mt6.ngrp = -1;
	mt6.P.rotparam.angle = (float)(Beta - Beta0);
	mt6.P.rotparam.ref = _V(R2*(1 + cos(A + 45*RAD)), 0, -R2*sin(A + 45*RAD));
	mt6.P.rotparam.axis = _V(sin(A + 225*RAD), 0, cos(A + 225*RAD));
	if (Far) MeshgroupTransform(vis, mt6);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_launchpad::LaunchFermsRotation(double PsiZ , double dt)
{
	double epsilon = 0.05;
	double Psi0 = Psi;
	double dPsi = PsiZ - Psi;
	double eps = 1e-1;
	double h;
	
	if (!FirstStep)
	{
		FirstStep = true;
		oapiGetAltitude(hLauncher, &h0); 
	}
	

	// This is the final action that launchpad can do! 
	// Stop acting and clean up!
	if (ABS(dPsi) < eps)
	{
		Psi = PsiZ;
		Ready04 = true;
		sprintf(oapiDebugString(), "");
	}
	else
	{
		//OmegaPsi = OmegaPsi + epsilon*cos(30*RAD - Psi)*dt;
		//Psi = Psi + OmegaPsi*dt*sign(dPsi);
		oapiGetAltitude(hLauncher, &h);
		Psi = 0.08*(h - h0);
	}
	

	if (!vis) return;

	mt1.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt1.nmesh = 0;//1;
	mt1.ngrp = -1;
	mt1.P.rotparam.angle = (float)(Psi - Psi0);
	mt1.P.rotparam.ref = _V(R*(1 - cos(A)), 0, R*sin(A));
	mt1.P.rotparam.axis = _V(sin(A), 0, cos(A));
	if (Far) MeshgroupTransform(vis, mt1);

	mt2.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt2.nmesh = 1;//2;
	mt2.ngrp = -1;
	mt2.P.rotparam.angle = (float)(Psi - Psi0);
	mt2.P.rotparam.ref = _V(R*(1 - cos(A + 90*RAD)), 0, R*sin(A + 90*RAD));
	mt2.P.rotparam.axis = _V(sin(A + 90*RAD), 0, cos(A + 90*RAD));
	if (Far) MeshgroupTransform(vis, mt2);

	mt3.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt3.nmesh = 2;//3;
	mt3.ngrp = -1;
	mt3.P.rotparam.angle = (float)(Psi - Psi0);
	mt3.P.rotparam.ref = _V(R*(1 - cos(A + 180*RAD)), 0, R*sin(A + 180*RAD));
	mt3.P.rotparam.axis = _V(sin(A + 180*RAD), 0, cos(A + 180*RAD));
	if (Far) MeshgroupTransform(vis, mt3);

	mt4.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt4.nmesh = 3;//4;
	mt4.ngrp = -1;
	mt4.P.rotparam.angle = (float)(Psi - Psi0);
	mt4.P.rotparam.ref = _V(R*(1 - cos(A + 270*RAD)), 0, R*sin(A + 270*RAD));
	mt4.P.rotparam.axis = _V(sin(A + 270*RAD), 0, cos(A + 270*RAD));
	if (Far) MeshgroupTransform(vis, mt4);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_launchpad::BeginStart(void)
{	
	if ( (!setazimut) && (config == BALLISTIC) ) 
	{
		Alaunch = GetLaunchAzimut(phiT*RAD, lambdaT*RAD);
	}
	else
	{
		Alaunch = GetLaunchAzimut(0, 0);
	}
	hR7->SetTaget(phiT*RAD, lambdaT*RAD);
	hR7->SetHyroplatform(Alaunch, 90*RAD);
	hR7->SetOrbitParams(Ha, Hp, Inc);
	Ready01 = true;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_launchpad::StartLaunchOperations(void)
{
	if (begin == 0)
	{
		begin  = 1;
		sprintf(oapiDebugString(), "Done. Azimuth = %f", Alaunch*DEG);
	}
}



