//=============================================================================
//
//
//
//
//
//=============================================================================
#define		ORBITER_MODULE
#include	"math.h"
#include	"..\..\..\include\orbitersdk.h"
#include	"stdio.h"
#include	"../math/mymath.h"
#include	"../quaternion/vectors.h"
#include	"../quaternion/matrix.h"
#include	"../quaternion/quaternion.h"
#include	"BlockD.h"

HINSTANCE	h_DLL;
//double *p = new double[6];

const	double			mesh_ofs = 7.314;
const	double			EMPTY_MASS2 = 6100;
const	double			TANK_CAPACITY2 = 95800;
const	double			MAX_MAIN_THRUST2 = 24000*G;
const	double			MAX_CE_THRUST2 = 3467*G;
const	double			MAX_MAIN_ISP2[2] = {248*G, 315*G};
const	double			EX_LENGHT2 = 25.0;
const	double			EX_WIDTH2 = 0.5;
const	double			EX_CE_LENGHT2 = 3;
const	double			EX_CE_WIDTH2 = 0.1;

const		double		ANGLE_CE_MAX = 45*RAD;
const		double		ANGLE_CE_MAX_MAN = 30*RAD;
const		double		OMEGA_CE = 100*RAD;

const		VECTOR3		CS = {62.0, 62.0, 6.71};
const		VECTOR3		PMI = {175.61, 175.61, 10};
const		VECTOR3		CAMERA01 = {0.39, 0.39, -4.65 - mesh_ofs};
const		VECTOR3		CAMERA02 = {-0.39, 0.39, -4.65 - mesh_ofs};
const		VECTOR3		CAMERA03 = {-0.39, -0.39, -4.65 - mesh_ofs};
const		VECTOR3		CAMERA04 = {0.39, -0.39, -4.65 - mesh_ofs};
const		VECTOR3		CE01 = {1.1, 0, -4.85 - mesh_ofs};
const		VECTOR3		CE02 = {0, 1.1, -4.85 - mesh_ofs};
const		VECTOR3		CE03 = {-1.1, 0, -4.85 - mesh_ofs};
const		VECTOR3		CE04 = {0, -1.1, -4.85 - mesh_ofs};
const		VECTOR3		TD_POINT01 = {1, 0, -6 - mesh_ofs};
const		VECTOR3		TD_POINT02 = {-1, -1, -6 - mesh_ofs};
const		VECTOR3		TD_POINT03 = {-1, 1, -6 - mesh_ofs};

const double BB_SIDE = 1.52;  // 1.471 - original, 1.52 - a compromise, 1.56 - nominal top connection but body too far sideways
const double BB_Z = 14.337;

const		VECTOR3		APOINT01 = {BB_SIDE, 0.0, BB_Z - mesh_ofs};
VECTOR3		ADIR01 = {0, 0, -1};
const		VECTOR3		AROT01 = {0, 1, 0};

const		VECTOR3		APOINT02 = {0.0, BB_SIDE, BB_Z - mesh_ofs};
VECTOR3		ADIR02 = {0, 0, -1};
const		VECTOR3		AROT02 = {-1, 0, 0};

const		VECTOR3		APOINT03 = {-BB_SIDE, 0.0, BB_Z - mesh_ofs};
VECTOR3		ADIR03 = {0, 0, -1};
const		VECTOR3		AROT03 = {0, -1, 0};

const		VECTOR3		APOINT04 = {0.0, -BB_SIDE, BB_Z - mesh_ofs};
VECTOR3		ADIR04 = {0, 0, -1};
const		VECTOR3		AROT04 = {1, 0, 0};

const		VECTOR3		CE_AXIS01 = {-1, 0, 0};
const		VECTOR3		CE_AXIS02 = {0, 1, 0};
const		VECTOR3		CE_AXIS03 = {-1, 0, 0};
const		VECTOR3		CE_AXIS04 = {0, 1, 0};

const		VECTOR3		B_THRUST_REF = {2.28, 0, -4.65 - mesh_ofs};
const		VECTOR3		C_THRUST_REF = {0, 2.28, -4.65 - mesh_ofs};
const		VECTOR3		E_THRUST_REF = {-2.28, 0, -4.65 - mesh_ofs};
const		VECTOR3		D_THRUST_REF = {0, -2.28, -4.65 - mesh_ofs};

const		VECTOR3		B_CE_REF = {3.439, 0, -4.85 - mesh_ofs};
const		VECTOR3		C_CE_REF = {0, 3.439, -4.85 - mesh_ofs};
const		VECTOR3		D_CE_REF = {-3.439, 0, -4.85 - mesh_ofs};
const		VECTOR3		E_CE_REF = {0, -3.439, -4.85 - mesh_ofs};

const		VECTOR3		PAYLOAD_REF = {0, 0, 22.5 - mesh_ofs};
const		VECTOR3		PAYLOAD_DIR = {0, 0, 1};
const		VECTOR3		PAYLOAD_ROT = {1, 0, 0};
const		VECTOR3		ADAPTER_REF = {0, 0, 22.173 - mesh_ofs};

const		VECTOR3		LAUNCHPAD_REF = {0, 0, -5 - mesh_ofs};
const		VECTOR3		LAUNCHPAD_DIR = {0, 0, -1};
const		VECTOR3		LAUNCHPAD_ROT = {0, -1, 0};

const		int			CE01_NGROUP = 5;
const		int			CE02_NGROUP = 7;
const		int			CE03_NGROUP = 6;
const		int			CE04_NGROUP = 4;

const		int			FIRST_STAGE = 0;
const		int			SEPARATION = 1;
const		int			SECOND_STAGE = 2;
const		int			HEAD_SEPARATION = 3;
const		int			STOP = 4;

const		int			CFG_WARHEAD = 0;
const		int			CFG_PS_1 = 1;
const		int			CFG_3STAGE = 2;

const		float		dAmin = 1*RAD;

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
DLLCLBK	void InitModule(HINSTANCE hModule)
{
	h_DLL = hModule;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
DLLCLBK void ExitModule(HINSTANCE hModule)
{

}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void RotateVector(const VECTOR3 &Initial, const VECTOR3 &Angles, VECTOR3 &Result) 
{ 
	VECTOR3 ex = _V(1, 0, 0);
	VECTOR3 ey = _V(0, 1, 0);
	VECTOR3 ez = _V(0, 0, 1);
	quaternion	q, qi, v;

	q = _quaternion(Angles.x, _vector3(ex.x, ex.y, ex.z));
	qi = qinverse(q);
	v = _quaternion(PI, _vector3(Initial.x, Initial.y, Initial.z));
	v = (q*v)*qi;

	q = _quaternion(Angles.y, _vector3(ey.x, ey.y, ey.z));
	qi = qinverse(q);	
	v = (q*v)*qi;

	q = _quaternion(Angles.z, _vector3(ez.x, ez.y, ez.z));
	qi = qinverse(q);	
	v = (q*v)*qi;

	Result = _V(v.x, v.y, v.z);
} 

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void Ec2Eq (OBJHANDLE hPlanet,VECTOR3 &Location) 
{ 
	double   Obl =   oapiGetPlanetObliquity (hPlanet); 
	double   Tet =   oapiGetPlanetTheta (hPlanet); 
	VECTOR3  tmp = Location;
	
	Location.x = tmp.x;
	Location.y = tmp.y*cos(Obl) + tmp.z*sin(Obl);
	Location.z = -tmp.y*sin(Obl) + tmp.z*cos(Obl);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
R7_second_stage::R7_second_stage(OBJHANDLE hObj, int fmodel): VESSEL2(hObj, fmodel)
{
	hAdapter = oapiLoadMeshGlobal("r7_adapter");
	etex = oapiRegisterExhaustTexture("Exhaust2");
	ptex = oapiRegisterParticleTexture("Contrail3");


	memset(&psc, 0, sizeof(psc));
	psc.srcsize = 1.5;
	psc.srcrate = 1000;
	psc.v0 = 100;
	psc.srcspread = 0.01;
	psc.lifetime = 0.1;
	psc.growthrate = 5;
	psc.atmslowdown = 0.1;
	psc.ltype = PARTICLESTREAMSPEC::EMISSIVE;
	psc.levelmap = PARTICLESTREAMSPEC::LVL_PSQRT;
	psc.lmin = 0;
	psc.lmax = 0.5;
	psc.atmsmap = PARTICLESTREAMSPEC::ATM_PLIN;
	psc.amin = 0.0;
	psc.amax = 0.5;
	psc.tex = ptex;

	memset(&psc1, 0, sizeof(psc1));
	psc1.srcsize = 10;
	psc1.srcrate = 1000;
	psc1.v0 = 200;
	psc1.srcspread = 0.01;
	psc1.lifetime = 2;
	psc1.growthrate = 5;
	psc1.atmslowdown = 0.1;
	psc1.ltype = PARTICLESTREAMSPEC::EMISSIVE;
	psc1.levelmap = PARTICLESTREAMSPEC::LVL_PSQRT;
	psc1.lmin = 0;
	psc1.lmax = 0.5;
	psc1.atmsmap = PARTICLESTREAMSPEC::ATM_PLIN;
	psc1.amin = 0.0;
	psc1.amax = 0.5;
	//psc1.tex = ptex;

	angleCE01 = 0;
	angleCE02 = 0;
	angleCE03 = 0;
	angleCE04 = 0;
	FirstStageLevel = 0.0;
	FirstStageCELevel = 0.0;
	SecondStageLevel = 0.0;
	SecondStageCELevel = 0.0;
	massB = massC = massD = massE = 0;
	F1 = 0;
	p0 = 101252.94;
	alphaB = alphaC = alphaD = alphaE = 0*RAD;
	OmegaB = OmegaC = OmegaD = OmegaE = 0*RAD;
	LTime = 0;
	Time = StopTime = 0;
	StartFlag = false;
	Autopilot = true;
	Vdetach = 1;
	Azimut = 0*RAD;
	Alpha = 40*RAD;
	phiT = 45*RAD;
	lambdaT = 45*RAD;
	Vground = Vk = 0;
	Vw = 0;
	Stop1 = 0;
	Far = 0;
	signP0 = 0;
	signP1 = 0;
	np1 = np2 = 0;
	signY0 = 0;
	signY1 = 0;
	ny1 = ny2 = ntest = 0;
	Natt = 0;
	psi = 90*RAD;
	Omega = _V(0, 0, 0);
	debug_err_AZ = 0;
	PayLoadMass = 0;

//	PAY_ATTACH = _V(0, 0, 0);  %%% Vostok-Luna: hard-code instead of scenario
	PAY_ATTACH = _V(0, 0, -1.605);

	PAY_DIR = _V(0, 0, 0);
	PAY_ROT = _V(0, 0, 0);
	out = fopen("test.txt", "wt");
	nt = 0;
	Error = 0;
	dPitch = 90*RAD;
	
	PerigeeAlt = ApogeeAlt = Inc = 0;
	level3 = 0;
	Psi0 = PI/2;
	psi2 = t2 = Omega0 = 0;
	Vsep = 5;

	plevel = 0;

	killrot = sw = 0;

	// Vostok-Luna mod
	bVostokLuna = false;
	tA = 0;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::clbkSetClassCaps(FILEHANDLE cfg)
{
	SetSize(5.0);
	SetEmptyMass(EMPTY_MASS2);
	SetPMI(PMI);
	SetCrossSections(CS);
	SetCW(0.2, 0.2, 0.2, 0.2);
	SetCW (0.2, 0.2, 0.2, 0.2); 
	SetRotDrag (_V(3.0, 3.0, 3.0));
	SetLiftCoeffFunc(0);
	SetPitchMomentScale(1e-6);
	SetBankMomentScale(1e-6);
	SetSurfaceFrictionCoeff(1e5, 1e5);
	//SetTouchdownPoints(TD_POINT01, TD_POINT02, TD_POINT03);

	ph_main = CreatePropellantResource(TANK_CAPACITY2);

	th_camera[0] = CreateThruster(CAMERA01,
		                         _V(0, 0, 1),
								 MAX_MAIN_THRUST2,
								 ph_main,
								 MAX_MAIN_ISP2[1],
								 MAX_MAIN_ISP2[0]);

	th_camera[1] = CreateThruster(CAMERA02,
		                         _V(0, 0, 1),
								 MAX_MAIN_THRUST2,
								 ph_main,
								 MAX_MAIN_ISP2[1],
								 MAX_MAIN_ISP2[0]);

	th_camera[2] = CreateThruster(CAMERA03,
		                         _V(0, 0, 1),
								 MAX_MAIN_THRUST2,
								 ph_main,
								 MAX_MAIN_ISP2[1],
								 MAX_MAIN_ISP2[0]);

	th_camera[3] = CreateThruster(CAMERA04,
		                         _V(0, 0, 1),
								 MAX_MAIN_THRUST2,
								 ph_main,
								 MAX_MAIN_ISP2[1],
								 MAX_MAIN_ISP2[0]);

	th_ce01 = CreateThruster(CE01,
		                     _V(0, 0, 1),
				 			 MAX_CE_THRUST2,
							 ph_main,
							 MAX_MAIN_ISP2[1],
							 MAX_MAIN_ISP2[0]);

	th_ce02 = CreateThruster(CE02,
		                     _V(0, 0, 1),
				 			 MAX_CE_THRUST2,
							 ph_main,
							 MAX_MAIN_ISP2[1],
							 MAX_MAIN_ISP2[0]);

	th_ce03 = CreateThruster(CE03,
		                     _V(0, 0, 1),
				 			 MAX_CE_THRUST2,
							 ph_main,
							 MAX_MAIN_ISP2[1],
							 MAX_MAIN_ISP2[0]);

	th_ce04 = CreateThruster(CE04,
		                     _V(0, 0, 1),
				 			 MAX_CE_THRUST2,
							 ph_main,
							 MAX_MAIN_ISP2[1],
							 MAX_MAIN_ISP2[0]);

	thg_main = CreateThrusterGroup(th_camera, 4, THGROUP_MAIN);

	AddExhaust(th_camera[0], EX_LENGHT2, EX_WIDTH2, etex);
	AddExhaust(th_camera[1], EX_LENGHT2, EX_WIDTH2, etex);
	AddExhaust(th_camera[2], EX_LENGHT2, EX_WIDTH2, etex);
	AddExhaust(th_camera[3], EX_LENGHT2, EX_WIDTH2, etex);

	AddExhaust(th_ce01, EX_CE_LENGHT2, EX_CE_WIDTH2, etex);
	AddExhaust(th_ce02, EX_CE_LENGHT2, EX_CE_WIDTH2, etex);
	AddExhaust(th_ce03, EX_CE_LENGHT2, EX_CE_WIDTH2, etex);
	AddExhaust(th_ce04, EX_CE_LENGHT2, EX_CE_WIDTH2, etex);

	AddExhaustStream(th_camera[0], &psc);
	AddExhaustStream(th_camera[1], &psc);
	AddExhaustStream(th_camera[2], &psc);
	AddExhaustStream(th_camera[3], &psc);

	hBlockBAtt = CreateAttachment(false,
			   				      APOINT01,
							      ADIR01,
							      AROT01,
								  "ForBlockB",
							      false);

	hBlockCAtt = CreateAttachment(false,
			   				      APOINT02,
							      ADIR02,
							      AROT02,
								  "ForBlockC",
							      false);

	hBlockDAtt = CreateAttachment(false,
			   				      APOINT03,
							      ADIR03,
							      AROT03,
								  "ForBlockD",
							      false);

	hBlockEAtt = CreateAttachment(false,
			   				      APOINT04,
							      ADIR04,
							      AROT04,
								  "ForBlockE",
							      false);

	hHeadAtt = CreateAttachment(false,
								PAYLOAD_REF,
								PAYLOAD_DIR,
								PAYLOAD_ROT,
								"ForHead",
								false);

	hToLaunchpadAtt = CreateAttachment(true,
									   LAUNCHPAD_REF,
									   LAUNCHPAD_DIR,
									   LAUNCHPAD_ROT,
									   "ToLaunchpad",
									   false);		
	
	PSTREAM_HANDLE ps = AddParticleStream(&psc1, _V(0, 0, -60), _V(0, 0, -1), &plevel);
	//fprintf(out, "clbkSetClassCaps -> Stop1 = %i\n", Stop1);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::clbkLoadStateEx(FILEHANDLE scn, void *status)
{
	while (oapiReadScenario_nextline(scn, line))
	{
		if (!strnicmp (line, "AUTOPILOT", 9))
		{
			sscanf (line+9, "%i", &Autopilot);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}
		
		if (!strnicmp (line, "ANGLECE01", 9))
		{
			sscanf (line+9, "%f", &angleCE01);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "ANGLECE02", 9))
		{
			sscanf (line+9, "%f", &angleCE02);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "ANGLECE03", 9))
		{
			sscanf (line+9, "%f", &angleCE03);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "ANGLECE04", 9))
		{
			sscanf (line+9, "%f", &angleCE04);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "PHIT", 4))
		{
			sscanf (line+4, "%f", &phiT);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "LAMBDAT", 7))
		{
			sscanf (line+7, "%f", &lambdaT);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}
		
		if (!strnicmp (line, "ALPHAB", 6))
		{
			sscanf (line+6, "%f", &alphaB);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "ALPHAC", 6))
		{
			sscanf (line+6, "%f", &alphaC);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "ALPHAD", 6))
		{
			sscanf (line+6, "%f", &alphaD);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "ALPHAE", 6))
		{
			sscanf (line+6, "%f", &alphaE);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}
		
		if (!strnicmp (line, "OMEGAB", 6))
		{
			sscanf (line+6, "%f", &OmegaB);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "OMEGAC", 6))
		{
			sscanf (line+6, "%f", &OmegaC);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "OMEGAD", 6))
		{
			sscanf (line+6, "%f", &OmegaD);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "OMEGAE", 6))
		{
			sscanf (line+6, "%f", &OmegaE);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "STOP", 4))
		{
			sscanf (line+4, "%i", &Stop1);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "STARTFLAG", 9))
		{
			sscanf (line+9, "%i", &StartFlag);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "TIME", 4))
		{
			sscanf (line+4, "%f", &Time);
		}
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "STOPTIME", 8))
		{
			sscanf (line+8, "%f", &StopTime);
		}
		
		if (!strnicmp (line, "STAGE", 5))
		{
			sscanf (line+5, "%i", &stage);
		} 
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "FAR", 3))
		{
			sscanf (line+3, "%i", &Far);
		} 
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "PITCH", 5))
		{
			sscanf (line+5, "%f", &psi);
		} 
		else 
		{
			//ParseScenarioLineEx (line, status);
		}
		
		if (!strnicmp (line, "AZIMUTH", 7))
		{
			sscanf (line+7, "%f", &Azimut);
		} 
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "ALPHA", 5))
		{
			sscanf (line+5, "%f", &Alpha);
		} 
		else 
		{
			//ParseScenarioLineEx (line, status);
		}
		
		if (!strnicmp (line, "PSI0", 4))
		{
			float psi0;
			sscanf (line+4, "%f", &psi0);
			Psi0 = (double)psi0*RAD;
		}

		if (!strnicmp (line, "PSI2", 4))
		{
			float psi0;
			sscanf (line+4, "%f", &psi0);
			psi2 = (double)psi0;
		}

		if (!strnicmp (line, "T2", 2))
		{
			float t;
			sscanf (line+2, "%f", &t);
			t2 = (double)t;
		}

		if (!strnicmp (line, "OMEGA0", 6))
		{
			float omeg;
			sscanf (line+6, "%f", &omeg);
			Omega0 = (double)omeg*RAD;
		}

		if (!strnicmp (line, "ADAPTER", 7))
		{
			sscanf (line+7, "%i", &config);
		} 
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "MESHNAME", 8))
		{
			sscanf (line+8, "%s", &MeshName);
		} 
		
		if (!strnicmp (line, "PAYLOAD", 7))
		{
			sscanf (line+7, "%s", &PayLoadName);
		} 
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

/* %%% Vostok-Luna: hard-code
	if (!strnicmp (line, "APOINT", 6))
		{
			float x, y, z;
			sscanf (line+7, "%f %f %f", &x, &y, &z);
			PAY_ATTACH.x = (double)x;
			PAY_ATTACH.y = (double)y;
			PAY_ATTACH.z = (double)z;
		} 
		else 
		{
			//ParseScenarioLineEx (line, status);
		}*/

		if (!strnicmp (line, "PDIR", 4))
		{
			float x, y, z;
			sscanf (line+4, "%f %f %f", &x, &y, &z);
			PAY_DIR.x = (double)x;
			PAY_DIR.y = (double)y;
			PAY_DIR.z = (double)z;
		} 
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "PROT", 4))
		{
			float x, y, z;
			sscanf (line+4, "%f %f %f", &x, &y, &z);
			PAY_ROT.x = (double)x;
			PAY_ROT.y = (double)y;
			PAY_ROT.z = (double)z;
		} 
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "APOGEE", 6))
		{
			float ap;
			sscanf (line+6, "%f", &ap);
			ApogeeAlt = (double)ap;
		} 
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		if (!strnicmp (line, "PERIGEE", 7))
		{
			float pe;
			sscanf (line+7, "%f", &pe);
			PerigeeAlt = (double)pe;
		}

		if (!strnicmp (line, "VOSTOKLUNA", 10))
		{
			sscanf (line+10, "%d", &bVostokLuna);
		}

		if (!strnicmp (line, "INC", 3))
		{
			float i;
			sscanf (line+3, "%f", &i);
			Inc = (double)i;
		} 
		else 
		{
			//ParseScenarioLineEx (line, status);
		}

		ParseScenarioLineEx (line, status);
	}

	hH = oapiGetVesselByName(PayLoadName);
	if ( (stage != HEAD_SEPARATION) && (stage != STOP) && (hH) )
	{
		hHead = oapiGetVesselInterface(hH);
		ATTACHMENTHANDLE hPayloadAtt = hHead->CreateAttachment(true,
			                                                   PAY_ATTACH,
														       PAY_DIR,
														       PAY_ROT,
														       "ToR7",
														       false);
		AttachChild(hH, hHeadAtt, hPayloadAtt);
		PayLoadMass = hHead->GetMass();
	}

	hBlockA = oapiLoadMeshGlobal(MeshName);
	AddMesh(hBlockA);

	switch (config)
	{
		case CFG_WARHEAD:
		{
			AddMesh(oapiLoadMeshGlobal("r7_adapter"), &ADAPTER_REF);
			break;
		}
		case CFG_PS_1:
		{
			AddMesh(oapiLoadMeshGlobal("r7_adapter_ps1_new"), &ADAPTER_REF);
			//ReadPitchProg("./Config/R7/pitchprog.cfg");
			break;
		}
		case CFG_3STAGE:
		{
//			AddMesh(oapiLoadMeshGlobal("r7_adapter_frame2"), &ADAPTER_REF);
			ReadPitchProg("./Config/R7/pitchprog.cfg");
			break;
		}
	}
	
	hCE01 = oapiLoadMeshGlobal("r7_VL\\BlockDEngines");
	hCE02 = oapiLoadMeshGlobal("r7_VL\\BlockDEngines");
	hCE03 = oapiLoadMeshGlobal("r7_VL\\BlockDEngines");
	hCE04 = oapiLoadMeshGlobal("r7_VL\\BlockDEngines");
	AddMesh(hCE01, &CE01);
	AddMesh(hCE02, &CE02);
	AddMesh(hCE03, &CE03);
	AddMesh(hCE04, &CE04);
	//fprintf(out, "clbkLoadStateEx -> Stop1 = %i\n", Stop1);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::clbkSaveState(FILEHANDLE scn)
{
	VESSEL2::clbkSaveState(scn);
	oapiWriteScenario_int(scn, "STAGE", stage);
	oapiWriteScenario_float(scn, "TIME", Time);
	oapiWriteScenario_float(scn, "STOPTIME", StopTime);
	oapiWriteScenario_int(scn, "STARTFLAG", StartFlag);
	oapiWriteScenario_int(scn, "STOP", Stop1);
	oapiWriteScenario_float(scn, "ALPHAB", alphaB);
	oapiWriteScenario_float(scn, "ALPHAC", alphaC);
	oapiWriteScenario_float(scn, "ALPHAD", alphaD);
	oapiWriteScenario_float(scn, "ALPHAE", alphaE);
	oapiWriteScenario_float(scn, "OMEGAB", OmegaB);
	oapiWriteScenario_float(scn, "OMEGAC", OmegaC);
	oapiWriteScenario_float(scn, "OMEGAD", OmegaD);
	oapiWriteScenario_float(scn, "OMEGAE", OmegaE);
	oapiWriteScenario_float(scn, "PHIT", phiT);
	oapiWriteScenario_float(scn, "LAMBDAT", lambdaT);
	oapiWriteScenario_float(scn, "ANGLECE01", angleCE01);
	oapiWriteScenario_float(scn, "ANGLECE02", angleCE02);
	oapiWriteScenario_float(scn, "ANGLECE03", angleCE03);
	oapiWriteScenario_float(scn, "ANGLECE04", angleCE04);
	oapiWriteScenario_int(scn, "AUTOPILOT", Autopilot);
	oapiWriteScenario_int(scn, "FAR", Far);
	oapiWriteScenario_float(scn, "PITCH", psi);
	oapiWriteScenario_int(scn, "ADAPTER", config);
	oapiWriteScenario_float(scn, "AZIMUTH", Azimut);
	oapiWriteScenario_float(scn, "ALPHA", Alpha);
	if ( (config == CFG_3STAGE) && (config == CFG_3STAGE) )
	{
		oapiWriteScenario_float(scn, "PSI0", Psi0*DEG);
		oapiWriteScenario_float(scn, "PSI2", psi2);
		oapiWriteScenario_float(scn, "T2", t2);
		oapiWriteScenario_float(scn, "OMEGA0", Omega0*DEG);
	}

	oapiWriteScenario_string(scn, "MESHNAME", MeshName);
	if (hH)
	{
		oapiWriteScenario_string(scn, "PAYLOAD", PayLoadName);
		oapiWriteScenario_vec(scn, "APOINT", PAY_ATTACH);
		oapiWriteScenario_vec(scn, "PDIR", PAY_DIR);
		oapiWriteScenario_vec(scn, "PROT", PAY_ROT);
	}

	if (config != CFG_WARHEAD)
	{
		oapiWriteScenario_float(scn, "APOGEE", ApogeeAlt);
		oapiWriteScenario_float(scn, "PERIGEE", PerigeeAlt);
		oapiWriteScenario_float(scn, "INC", Inc);
	}

	if (bVostokLuna)
	{
		oapiWriteScenario_int(scn, "VOSTOKLUNA", bVostokLuna);
	}

	//fprintf(out, "clbkSaveState -> Stop1 = %i\n", Stop1);
	fclose(out);	
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::clbkPostStep(double simt, double SimDT, double mjd)
{
	// do not process timesteps if all work is done... 
	// unless some kind of touchdown event will be implemented in future
	if (stage == STOP)
		return;

	if ( (Natt == 0) && ( (stage == FIRST_STAGE) || (stage == SEPARATION) ) )
	{
		hB = GetAttachmentStatus(hBlockBAtt);		
		hBlockB = oapiGetVesselInterface(hB);
	
		hC = GetAttachmentStatus(hBlockCAtt);		
		hBlockC = oapiGetVesselInterface(hC);
	
		hD = GetAttachmentStatus(hBlockDAtt);		
		hBlockD = oapiGetVesselInterface(hD);
	
		hE = GetAttachmentStatus(hBlockEAtt);		
		hBlockE = oapiGetVesselInterface(hE);

		hblockB = (R7_first_stage*) hBlockB;
		hblockC = (R7_first_stage*) hBlockC;
		hblockD = (R7_first_stage*) hBlockD;
		hblockE = (R7_first_stage*) hBlockE;
		Natt++;
	}	
	
	VECTOR3 err = GetOrientErrors();

	if ( (StartFlag) && (!Stop1) ) 
	{
		if (GetHandle() == oapiGetFocusObject())
		/*sprintf(oapiDebugString(), "t = %f dB = %f dP = %f dY = %f",
			    Time, 
				err.x*DEG,
				err.y*DEG,
				err.z*DEG
				);*/
		sprintf(oapiDebugString(), "MET = %i | Bank Error = %f | Pitch Error = %f | Yaw Error = %f",
			    (int)(Time - 16), 
				err.x*DEG,
				err.y*DEG,
				err.z*DEG
				);		
	}

	
	ADIR01 = _V(sin(alphaB), 0, -cos(alphaB));
	ADIR02 = _V(0, sin(alphaC), -cos(alphaC));
	ADIR03 = _V(-sin(alphaD), 0, -cos(alphaD));
	ADIR04 = _V(0, -sin(alphaE), -cos(alphaE));

	SetAttachmentParams(hBlockBAtt,
						APOINT01,
						ADIR01,
						AROT01);

	SetAttachmentParams(hBlockCAtt,
						APOINT02,
						ADIR02,
						AROT02);

	SetAttachmentParams(hBlockDAtt,
						APOINT03,
						ADIR03,
						AROT03);

	SetAttachmentParams(hBlockEAtt,
						APOINT04,
						ADIR04,
						AROT04);
	
	if ( (StartFlag) && (stage != SECOND_STAGE) && (stage != STOP) && (stage != HEAD_SEPARATION) )
	{
		SetFirstStageMainThrust(FirstStageProg(Time));
		SetFirstStageCEThrust(FirstStageProg(Time));		
	}

	if ( (StartFlag) && (!Stop1) )
	{
		SetMainLevel(SecondStageProg(Time));
		SetCELevel(SecondStageProg(Time));		
	}

	switch (stage)
	{
	case FIRST_STAGE:
		{
			massB = hblockB->GetMass();
			massC = hblockC->GetMass();
			massD = hblockD->GetMass();
			massE = hblockE->GetMass();
			SetEmptyMass(massB + massC + massD + massE + EMPTY_MASS2 + PayLoadMass);
			
			double F1 = GetFirstStageForce();
			double Fce1 = GetFirstStageCEForce();
			AddForce(_V(0, 0, F1), B_THRUST_REF);
			AddForce(_V(0, 0, F1), C_THRUST_REF);
			AddForce(_V(0, 0, F1), D_THRUST_REF);
			AddForce(_V(0, 0, F1), E_THRUST_REF);

			AddForce(_V(0, Fce1*sin(angleCE01), Fce1*cos(angleCE01)), B_CE_REF);
			AddForce(_V(-Fce1*sin(angleCE02), 0, Fce1*cos(angleCE02)), C_CE_REF);
			AddForce(_V(0, Fce1*sin(angleCE03), Fce1*cos(angleCE03)), D_CE_REF);
			AddForce(_V(-Fce1*sin(angleCE04), 0, Fce1*cos(angleCE04)), E_CE_REF);
			
			double level01 = GetFirstStageLevel();
			if ( (Time > 120) && (level01 < 0.7) && (StartFlag) ) 
				stage = SEPARATION;

			break;
		}
	case SEPARATION:
		{
			massB = hblockB->GetMass();
			massC = hblockC->GetMass();
			massD = hblockD->GetMass();
			massE = hblockE->GetMass();
			SetEmptyMass(massB + massC + massD + massE + EMPTY_MASS2 + PayLoadMass);
			double mass = GetEmptyMass();			

			double F1 = GetFirstStageForce();			
			AddForce(ADIR01*(-F1), APOINT01);
			AddForce(ADIR02*(-F1), APOINT02);
			AddForce(ADIR03*(-F1), APOINT03);
			AddForce(ADIR04*(-F1), APOINT04);

			LateralBlocksRotation(SimDT);

			if (alphaB >= 7*RAD)
			{
				FirstStageSeparation();
				stage = SECOND_STAGE;
			}
			break;
		}
	case SECOND_STAGE:
		{
			SetEmptyMass(EMPTY_MASS2 + PayLoadMass);
			break;
		}
	case HEAD_SEPARATION:
		{
			SetEmptyMass(EMPTY_MASS2);
			OBJHANDLE hPayload = GetAttachmentStatus(hHeadAtt);
			
			if (hPayload)
			{
				if (GetHandle() == oapiGetFocusObject())
				{
					oapiSetFocusObject(hPayload);
				}

				DetachChild(hHeadAtt, Vdetach);
				PayLoadMass = 0;
				if (ntest == 0)
				{
					VECTOR3 VW;
					hHead->GetRelativeVel(oapiGetGbodyByName("Earth"), VW);
					Vw = length(VW);
					ntest++;					
				}
			}

			// set flag to stop processing timesteps, clean up message line.
			stage = STOP;
			double Fgase = 5000*G*exp(-(Time - StopTime)/1);
			double gamma = 10*RAD;
			AddForce(_V(0, -Fgase*sin(gamma), -Fgase*cos(gamma)), ADAPTER_REF);
			sprintf(oapiDebugString(), "");
			return;
		}
	}
	
	//fprintf(out, "clbkPostStep -> Stop1 = %i\n", Stop1);
	//if (StartFlag) DumpInFile(out, Time);

	if ( (!Stop1) && (StartFlag) )
		if (!Autopilot)
		{
			ManualControl(Time, SimDT);
			KillRot(Time, SimDT);
		}
		else
			ProgramStep(Time, SimDT);	

	if ( (!Stop1) && (StartFlag) )
		ThrusterProgram(Time, SimDT);
	
	InversTrack(Time, SimDT);	

	Time = Time + SimDT;
	
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::clbkVisualCreated(VISHANDLE visual, int refcount)
{
	if (refcount > 1) return;
	vis = visual;
	Far = 1;	
	//fprintf(out, "clbkVisualCreated -> Stop1 = %i\n", Stop1);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::clbkVisualDestroyed(VISHANDLE visual, int refcount)
{
	if (visual == vis) 
	{
		vis = NULL;
		Far = 0;
	}
	//fprintf(out, "clbkVisualDestroyed -> Stop1 = %i\n", Stop1);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
int R7_second_stage::clbkConsumeDirectKey(char *keystate)
{
	if (KEYDOWN(keystate, OAPI_KEY_S))
	{
		Start();
	}
	
	/*if (KEYDOWN(keystate, OAPI_KEY_O))
	{
		StartLaunchProg();
	}*/
	
	if (KEYDOWN(keystate, OAPI_KEY_J) && (!Autopilot) )
	{
		OBJHANDLE hPayload = GetAttachmentStatus(hHeadAtt);
		if (hPayload)
		{
			oapiSetFocusObject(hPayload);
			DetachChild(hHeadAtt, Vdetach);
			PayLoadMass = 0;
		}
		Stop();
		stage = STOP;
	}

	if (!Autopilot)
	{
		if (KEYDOWN(keystate, OAPI_KEY_MULTIPLY))
		{
			Stop();
			stage = STOP;			
		}		
	}

	return 0;
	//fprintf(out, "clbkConsumeDirectKey -> Stop1 = %i\n", Stop1);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
int R7_second_stage::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
	if ( (key == OAPI_KEY_C) && (!down) )
	{
		Autopilot = !Autopilot;
		//fprintf(out, "clbkConsumeBufferedKey -> Stop1 = %i\n", Stop1);
		return 1;
	}

	if ( (key == OAPI_KEY_NUMPAD5) && (!down) )
	{
		killrot = !killrot;
		//fprintf(out, "clbkConsumeBufferedKey -> Stop1 = %i\n", Stop1);
		return 1;
	}
	//fprintf(out, "clbkConsumeBufferedKey -> Stop1 = %i\n", Stop1);
	return 0;
}

//=============================================================================
//
//=============================================================================
DLLCLBK VESSEL *ovcInit(OBJHANDLE hvessel, int flightmodel)
{
	return new R7_second_stage(hvessel, flightmodel);
}

DLLCLBK void ovcExit(VESSEL *vessel)
{
	if (vessel) delete (R7_second_stage*) vessel;
}

//=============================================================================
//
//=============================================================================

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetCEAngle01(double angleZ, double dt)
{
	double angle0;
	double eps = 1e-2;
	
	angle0 = angleCE01;
	if (ABS(angleZ - angleCE01) < dAmin)
		angleCE01 = 0;//angleZ;
	else
		angleCE01 = angleCE01 + sign(angleZ - angleCE01)*4*fabs(angleZ - angleCE01)*dt;

	SetThrusterDir(th_ce01, _V(0, sin(angleCE01), cos(angleCE01)));

	if (!vis) return;
	mt01.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt01.nmesh = 1;
	mt01.ngrp = -1;//CE01_NGROUP;
	mt01.P.rotparam.angle = (float)(angleCE01 - angle0);
	mt01.P.rotparam.ref = _V(0, 0, 0);//CE01;
	mt01.P.rotparam.axis = CE_AXIS01;
	if (Far) MeshgroupTransform(vis, mt01);
}

void R7_second_stage::SetCEAngle02(double angleZ, double dt)
{
	double angle0;
	double eps = 1e-2;
	
	angle0 = angleCE02;
	if (ABS(angleZ - angleCE02) < dAmin)
		angleCE02 = 0;//angleZ;
	else
		angleCE02 = angleCE02 + sign(angleZ - angleCE02)*4*fabs(angleZ - angleCE02)*dt;

	SetThrusterDir(th_ce02, _V(-sin(angleCE02), 0, cos(angleCE02)));

	if (!vis) return;
	mt02.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt02.nmesh = 2;
	mt02.ngrp = -1;//CE02_NGROUP;
	mt02.P.rotparam.angle = (float)(angleCE02 - angle0);
	mt02.P.rotparam.ref = _V(0, 0, 0);//CE02;
	mt02.P.rotparam.axis = CE_AXIS02;
	if (Far) MeshgroupTransform(vis, mt02);
}

void R7_second_stage::SetCEAngle03(double angleZ, double dt)
{
	double angle0;
	double eps = 1e-2;
	
	angle0 = angleCE03;
	if (ABS(angleZ - angleCE03) < dAmin)
		angleCE03 = 0;//angleZ;
	else
		angleCE03 = angleCE03 + sign(angleZ - angleCE03)*4*fabs(angleZ - angleCE03)*dt;

	SetThrusterDir(th_ce03, _V(0, sin(angleCE03), cos(angleCE03)));

	if (!vis) return;
	mt03.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt03.nmesh = 3;
	mt03.ngrp = -1;//CE03_NGROUP;
	mt03.P.rotparam.angle = (float)(angleCE03 - angle0);
	mt03.P.rotparam.ref = _V(0, 0, 0);//CE03;
	mt03.P.rotparam.axis = CE_AXIS03;
	if (Far) MeshgroupTransform(vis, mt03);
}

void R7_second_stage::SetCEAngle04(double angleZ, double dt)
{
	double angle0;
	double eps = 1e-2;
	
	angle0 = angleCE04;
	if (ABS(angleZ - angleCE04) < dAmin)
		angleCE04 = 0;//angleZ;
	else
		angleCE04 = angleCE04 + sign(angleZ - angleCE04)*4*fabs(angleZ - angleCE04)*dt;

	SetThrusterDir(th_ce04, _V(-sin(angleCE04), 0, cos(angleCE04)));

	if (!vis) return;
	mt04.transform = MESHGROUP_TRANSFORM::ROTATE;
	mt04.nmesh = 4;
	mt04.ngrp = -1;//CE04_NGROUP;
	mt04.P.rotparam.angle = (float)(angleCE04 - angle0);
	mt04.P.rotparam.ref = _V(0, 0, 0);//CE04;
	mt04.P.rotparam.axis = CE_AXIS04;
	if (Far) MeshgroupTransform(vis, mt04);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetCELevel(double level)
{
	SetThrusterLevel(th_ce01, level);
	SetThrusterLevel(th_ce02, level);
	SetThrusterLevel(th_ce03, level);
	SetThrusterLevel(th_ce04, level);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetMainLevel(double level)
{
	SetThrusterGroupLevel(THGROUP_MAIN, level);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::StartLaunchProg(void)
{
	Time = 0;	
	StartFlag = true;
	LAzimut = Azimut;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
double R7_second_stage::FirstStageProg(double t)
{
	double Level;

	if ( (t >= 0) && (t <= 10.5) ) Level = 0.4761904762*t;
	if ( (t > 10.5) && (t <= 13.5) ) Level = -222.5000000+21.66666667*t;
	if ( (t > 13.5) && (t <= 14.5) ) Level = -11.00000000+6.000000000*t;
	if ( (t > 14.5) && (t <= 17.0) )	Level = 70.20000000+0.4000000000*t;	
	if ( (t > 17.0) && (t <= 17.8) ) Level = -114.2500000+11.25000000*t;
	if ( (t > 17.8) && (t <= 23.0) )	Level = 79.15384615+0.3846153846*t;
	if ( (t > 23.0) && (t <= 23.5) ) Level = -418.0000000+22.00000000*t;
	if ( (t > 23.5) && (t <= 27.5) ) Level = 93.12500000+0.2500000000*t;
	if ( (t > 27.5) && (t <= 126) )	Level = 100;
	if ( (t > 126) && (t <= 126.4) ) Level = 6400.000000-50.00000000*t;
	if ( (t > 126.4) && (t <= 126.6) ) Level = 1976.000000-15.00000000*t;
	if ( (t > 126.6) && (t <= 127) ) Level = 19700.00000-155.0000000*t;
	if ( (t > 127) && (t <= 127.4) ) Level = 3507.500000-27.50000000*t;
	if ( (t > 127.4) && (t <= 130) ) Level = 200.0000000-1.538461538*t;
	if ( t > 130 ) Level = 0;

	//plevel = Level/100;
	return Level/100;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetFirstStageMainThrust(double level)
{
	hblockB->SetMainLevel(level);
	hblockC->SetMainLevel(level);
	hblockD->SetMainLevel(level);
	hblockE->SetMainLevel(level);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetFirstStageCEThrust(double level)
{
	hblockB->SetCELevel(level);
	hblockC->SetCELevel(level);
	hblockD->SetCELevel(level);
	hblockE->SetCELevel(level);
}

double R7_second_stage::SecondStageProg(double t)
{
	double Level;

	if ( (t >= 0) && (t <= 16) ) Level = 0.3125000000*t;
	if ( (t > 16) && (t <= 19) ) Level = -341.6666667+21.66666667*t;
	if ( (t > 19) && (t <= 20) ) Level = -405.0000000+25.00000000*t;
	if ( (t > 20) && (t <= 21) ) Level = 35.00000000+3.000000000*t;
	if ( (t > 21) && (t <= 24) ) Level = 84.00000000+.6666666667*t;
	if (t > 24) Level =100;		
		
	//plevel = Level/100;
	return Level/100;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::FirstStageSeparation(void)
{
	hblockB->SetCELevel(0);
	hblockC->SetCELevel(0);
	hblockD->SetCELevel(0);
	hblockE->SetCELevel(0);
	FirstStageCELevel = 0;
	
	DetachChild(hBlockBAtt, Vsep);
	DetachChild(hBlockCAtt, Vsep);
	DetachChild(hBlockDAtt, Vsep);
	DetachChild(hBlockEAtt, Vsep);

	hblockB->SetAngularVel(_V(0, OmegaB - Vsep/15.6, 0));
	hblockC->SetAngularVel(_V(0, OmegaB - Vsep/15.6, 0));
	hblockD->SetAngularVel(_V(0, OmegaB - Vsep/15.6, 0));
	hblockE->SetAngularVel(_V(0, OmegaB - Vsep/15.6, 0));

	hblockB->OpenGate();
	hblockC->OpenGate();
	hblockD->OpenGate();
	hblockE->OpenGate();

	hblockB->AddForce(_V( 2000000,0,0), _V( 1, 0, 0));
	hblockC->AddForce(_V( 2000000,0,0), _V( 1, 0, 0));
	hblockD->AddForce(_V( 2000000,0,0), _V( 1, 0, 0));
	hblockE->AddForce(_V( 2000000,0,0), _V( 1, 0, 0));
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
double R7_second_stage::GetFirstStageForce(void)
{
	THRUSTER_HANDLE thr = hblockB->GetThrusterHandleByIndex(0);
	double level = hblockB->GetThrusterLevel(thr);
	double max_force = hblockB->GetThrusterMax(thr);

	return 4*max_force*level;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
double R7_second_stage::GetFirstStageCEForce(void)
{	
	THRUSTER_HANDLE thr = hblockB->GetThrusterHandleByIndex(4);
	double level = hblockB->GetThrusterLevel(thr);
	double max_force = hblockB->GetThrusterMax(thr);

	return 2*max_force*level;	
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
double R7_second_stage::GetFirstStageLevel(void)
{	
	THRUSTER_HANDLE thr = hblockB->GetThrusterHandleByIndex(0);
	double level = hblockB->GetThrusterLevel(thr);
	return level;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::LateralBlocksRotation(double dt)
{
	VECTOR3		pmi;
	double		L = 15.14;	

	hblockB->GetPMI(pmi);	
	
	OmegaB = OmegaB + ( GetFirstStageForce()*1.968 )*dt/(pmi.x + L*L)/massB;
	alphaB = alphaB + OmegaB*dt;	

	OmegaC = OmegaC + ( GetFirstStageForce()*1.968 )*dt/(pmi.x + L*L)/massC;
	alphaC = alphaC + OmegaC*dt;

	OmegaD = OmegaD + ( GetFirstStageForce()*1.968 )*dt/(pmi.x + L*L)/massD;
	alphaD = alphaD + OmegaD*dt;

	OmegaE = OmegaE + ( GetFirstStageForce()*1.968 )*dt/(pmi.x + L*L)/massE;
	alphaE = alphaE + OmegaE*dt;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetHyroplatform(double azimut, double alpha)
{
	Azimut = azimut;
	Alpha = alpha;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
VECTOR3 R7_second_stage::GetLongAxisDir(double azimut, double alpha)
{
	double	theta;
	VECTOR3	ea, PP, PK, PPK, tg;
	quaternion q1, qi1, v;

	theta = oapiGetPlanetObliquity(oapiGetGbodyByName("Earth"));
	ea = _V(0, cos(theta), sin(theta));
	oapiGetGlobalPos(GetHandle(), &PK);
	oapiGetGlobalPos(oapiGetGbodyByName("Earth"), &PP);
	PPK = PK - PP;
	PPK = PPK*(1/length(PPK));
	tg = crossp(PPK, ea);
	tg = tg*(1/length(tg));

	q1 = _quaternion(azimut, _vector3(PPK.x, PPK.y, PPK.z));
	qi1 = qinverse(q1);
	v = _quaternion(PI, _vector3(tg.x, tg.y, tg.z));
	q1 = (q1*v)*qi1;
	tg = _V(q1.x, q1.y, q1.z);

	q1 = _quaternion(PI/2 - alpha, _vector3(tg.x, tg.y, tg.z));
	qi1 = qinverse(q1);
	v = _quaternion(PI, _vector3(PPK.x, PPK.y, PPK.z));
	q1= (q1*v)*qi1;

	return _V(q1.x, q1.y, q1.z);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
VECTOR3 R7_second_stage::GetOrientErrors(void)
{
	VECTOR3	PK, ViP, ViPL, dir;
	double	dB, dP, dY;
	double	sin_dP, cos_dP;
	double	sin_dY, cos_dY;

	dir = GetLongAxisDir(Azimut, Alpha);
	dir = dir*(1/length(dir));
	
	oapiGetGlobalPos(GetHandle(), &PK);
	ViP = PK + dir;
	Global2Local(ViP, ViPL);

	dB = 0;

	sin_dP = ViPL.y/length(ViPL);
	cos_dP = sqrt(1 - sin_dP*sin_dP);
	dP = atan2(sin_dP, cos_dP);

	sin_dY = ViPL.x/sqrt(ViPL.x*ViPL.x + ViPL.z*ViPL.z);
	cos_dY = ViPL.z/sqrt(ViPL.x*ViPL.x + ViPL.z*ViPL.z);
	dY = atan2(sin_dY, cos_dY);

	return _V(dB, dP, dY);
} 

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
double R7_second_stage::u(double emax, double phi, double omega, double omega0)
{
	double omega1, domega, u1;
	
	omega1 = -sqrt(2*emax*ABS(phi))*sign(-phi) + omega0;
	domega = omega - omega1;
	u1 = sin(ANGLE_CE_MAX)*sign(domega);
	
	return asin(u1);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
double R7_second_stage::u2(double X, double XX, double w0, double K, double angle, double dt)
{
/*	double upr;
	double dXX;
	double eps = 0.001*RAD;
	double T = 2*dt;
	int n = 0;
	
	double amax = 1;
	do
	{
		dXX = XX - sqrt(2*K*fabs(X))*sign(X) - w0;

		upr = -sign(dXX);

		if (K != 0)
			T = abs(dXX/K);

		if (T < 10*dt) 
		{
			K = K/2;
			amax = amax/2;
			n++;
		}
	}
	while ( (T < 10*dt) && (n < 1000) );

	return upr = upr*amax;
*/
	double upr;
	double dXX;
	double eps = 0.001*RAD;
	double T = 2*dt;
	int n = 0;

	double amax = 1;
	
	double Tmax;
	if (oapiGetTimeAcceleration() == 10)
		Tmax = 10;
	else
		Tmax = 100;

	do
	{
		dXX = XX - sqrt(2*K*fabs(X))*sign(X) - w0;

		upr = -sign(dXX);

		if (K != 0)
			T = abs(dXX/K);

		if (T < Tmax*dt) 
		{
			K = K/2;
			amax = amax/2;
			n++;
		}
	}
	while ( (T < Tmax*dt) && (n < 1000) );

	return upr = upr*amax;

}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetPitch(double dP, double wp, double t, double dt)
{
	double	Ix;
	VECTOR3	pmi;
	double T1, T2, L;
	VECTOR3 omega;
	double	sin_d, cos_d, d;	

	GetPMI(pmi);
	L = ABS(CE01.z);
	Ix = GetMass()*pmi.x;

	GetAngularVel(omega);

	if (stage == FIRST_STAGE)
	{
		T1 = 2*GetFirstStageCEForce();
		T2 = 2*GetThrusterMax(th_ce01, GetAtmPressure())*GetThrusterLevel(th_ce01);

		d =  u2(dP, omega.x, wp, (T1 + T2)*L*sin(ANGLE_CE_MAX)/Ix, angleCE01, dt);
		
		SetPitchMoment(d, dt);

	}
	else
	{
		T1 = 0;
		T2 = 2*GetThrusterMax(th_ce01, GetAtmPressure())*GetThrusterLevel(th_ce01);

		d = u2(dP, omega.x, wp, (T1 + T2)*L*sin(ANGLE_CE_MAX)/Ix, angleCE01, dt);

		SetPitchMoment(d, dt);
	}
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetYaw(double dY, double wp, double t, double dt)
{
	double	Iy;
	VECTOR3	pmi;
	double T1, T2, L;
	VECTOR3 omega;
	double	sin_d, cos_d, d;	

	GetPMI(pmi);
	L = ABS(CE01.z);
	Iy = GetMass()*pmi.y;

	GetAngularVel(omega);

	if (stage == FIRST_STAGE)
	{
		T1 = 2*GetFirstStageCEForce();
		T2 = 2*GetThrusterMax(th_ce01, GetAtmPressure())*GetThrusterLevel(th_ce01);

		d = u2( dY, -omega.y, wp, (T1 + T2)*L*sin(ANGLE_CE_MAX)/Iy, angleCE02, dt);

		SetYawMoment(d, dt);
	}
	else
	{
		T1 = 0;
		T2 = 2*GetThrusterMax(th_ce01, GetAtmPressure())*GetThrusterLevel(th_ce01);

		d = u2( dY, -omega.y, wp, (T1 + T2)*L*sin(ANGLE_CE_MAX)/Iy, angleCE02, dt);
		
		SetYawMoment(d, dt);
	}
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
bool R7_second_stage::LaunchpadFree(void)
{
	double mass = GetMass();
	double max_force = GetThrusterMax(th_camera[0]);
	double level = GetThrusterLevel(th_camera[0]);
	double max_forceCE = GetThrusterMax(th_ce01);
	double levelCE = GetThrusterLevel(th_ce01);	
	double level1, level_ce1;

	if ( (stage == FIRST_STAGE) || (stage == SEPARATION) )
	{
		level1 = 4*GetFirstStageForce();
		level_ce1 = 4*GetFirstStageCEForce()*cos(angleCE01);
	}
	else
		level1 = level_ce1 = 0;
	
	double F = 4*max_force*level + 4*max_forceCE*levelCE*cos(angleCE01) + level1 + level_ce1;
	double P = mass*G;

	if (F > P)
		return true;
	else
		if (stage == FIRST_STAGE)
			return false;
		else
			true;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetTaget(double phi, double lambda)
{
	phiT = phi;
	lambdaT = lambda;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::ProgramStep(double t, double dt)
{
	double Airspeed = GetAirspeed();
	double SoundSpeed = 310;
	double azimut;
	double H = GetAltitude();
	VECTOR3	V, dir, PSS, er;
	double v;
	double sin_x, cos_x;
	double alpha;
	double vk;
	double dpsi;
	double w;
	double Re = 6.37101e6;
	double Me = 5.973698968e24;
	double e;
	double H0 = 228;
	double r, vr;
	
	GetRelativeVel(oapiGetGbodyByName("Earth"), V);
	GetRelativePos(oapiGetGbodyByName("Earth"), PSS);
	v = length(V);
	r = length(PSS);
	er = PSS*(1/r);
	vr = dotp(V, er);
	dir = GetLongAxisDir(Azimut, PI/2);
	cos_x = dotp(V, dir)/v;
	sin_x = sqrt(1 - cos_x*cos_x);
	alpha = PI/2 - atan2(sin_x, cos_x);
	
	switch (config)
	{
	case CFG_WARHEAD:
		{
			if (H < 20*1000) 
				psi = GetPitch();

			if (Airspeed >= 0.1*SoundSpeed)
			{
				vk = GetStopSpeed(phiT, lambdaT, H, alpha, azimut, dpsi);
			}
			else
			vk = 7000;

			Vk = vk;

			if ( (Airspeed >= 0.1*SoundSpeed) && (Airspeed < 0.8*SoundSpeed) )
			{
				SetHyroplatform(azimut, 75*RAD);
				w = 0;				
			}

			if ( (Airspeed >= 0.8*SoundSpeed) && (H <= 20*1000) )
			{
				SetHyroplatform(azimut, GetPitch() - GetAOA());
				GetAngularVel(Omega);
				w = 0;				
			}

			if (H >= 20*1000)
			{
				if (psi > 20*RAD)
				{
					psi = psi - 1*RAD*dt;
					w = -1*RAD;
				}
				else
				{
					psi = 20*RAD;
					w = 0;
				}		
				SetHyroplatform(azimut, psi);
			}

			if ( (v >= vk - 2.0) && (!Stop1) )
			{
				SetMainLevel(0);
				SetCELevel(0);
				double aoa = GetAOA();
				Vdetach = sqrt( vk*vk - v*v*sin(aoa)*sin(aoa) ) - v*cos(aoa);
				if (Vdetach < 0) Vdetach = 0.01;
				stage = HEAD_SEPARATION;
				Stop1 = 1;
			}
			VECTOR3 err = GetOrientErrors();
			if (Airspeed >= 0.1*SoundSpeed) 
			{
				SetPitch(err.y, w, t, dt);
				SetYaw(err.z, 0, t, dt);
			}
			break;
		}
	case CFG_PS_1:
		{	
			e = (ApogeeAlt - H)/(2*Re + ApogeeAlt + H);
			vk = sqrt(GGRAV*Me*(1 + e)/(Re + H));
			Vk = vk;			
						
			if (Airspeed < 0.1*SoundSpeed)
			{
				psi = PI/2;
				w = 0;
			}

			if ( (Airspeed >= 0.1*SoundSpeed) && (Airspeed < 0.7*SoundSpeed) )
			{
				psi = Psi0;
				w = 0;
			}

			if ( (Airspeed >= 0.7*SoundSpeed) && (t < 170) )
			{
				psi = GetPitch() - GetAOA();
				psi2 = GetPitch();
				t2 = t;
				w = 0;
			}

			if (t >= 160)
			{
				psi = psi2 - Omega0*(t - t2);
				w = 0;
			}

			if ( fabs(vr/v) <= sin(3*RAD) )
			{
				psi = VerticalSpeedControl(r, v, vr, t, dt);
				w = 0;
			}

			if (psi < -20*RAD) psi = -20*RAD;

			SetHyroplatform(GetAzimutCorrection(), psi);
			
			if ( (v >= Vk - 2.0) && (!Stop1) )
			{
				SetMainLevel(0);
				SetCELevel(0);
				double aoa = GetAOA();
				Vdetach = sqrt( Vk*Vk - v*v*sin(aoa)*sin(aoa) ) - v*cos(aoa);
				if (Vdetach < 0) Vdetach = 0.01;
				stage = HEAD_SEPARATION;
				Stop1 = 1;
			}
			VECTOR3 err = GetOrientErrors();
			if (Airspeed >= 0.1*SoundSpeed) 
			{
				SetPitch(err.y, w, t, dt);
				SetYaw(err.z, 0, t, dt);
			}
			break;
		}
	case CFG_3STAGE:
		{
			if (Airspeed < 0.1*SoundSpeed)
			{
				psi = PI/2;
				w = 0;
			}

			if ( (Airspeed >= 0.1*SoundSpeed) && (Airspeed < 0.7*SoundSpeed) )
			{
				psi = Psi0;
				w = 0;
			}

			if (bVostokLuna)
			{
				if (Airspeed >= 0.7*SoundSpeed) 
				{
					if (tA == 0)
					{
						// initialize pitch program calc vars
						tA = t;
						psiA = psi;
						const double PSI_K = 15*RAD;
						double Dpsi = psiA - PSI_K;
						double DT = 275-tA;
						psiRatio = Dpsi / DT;
					}
					
					psi = psiA - (t-tA) * psiRatio;
					w = 0;
				}
			}

			else
			{
				if ( (Airspeed >= 0.7*SoundSpeed) && (t < 170) )
				{
					psi = GetPitch() - GetAOA();
					psi2 = GetPitch();
					t2 = t;
					w = 0;
				}

				if (t >= 170)
				{
					psi = psi2 - Omega0*(t - t2);
					w = 0;
				}			
			}
			/*if (GetPropellantMass(ph_main)/TANK_CAPACITY2 < 0.02)
			{
				psi = GetPitch();
				level3 += 0.3*dt;
				if (level3 >= 1.0) level3 = 1.0;
				hHead->SetThrusterGroupLevel(THGROUP_MAIN, level3);
			}

			if ( (config == CFG_3STAGE) && (level3 >= 0.7) )
			{
				Stop();
			}*/
			
			SetHyroplatform(GetAzimutCorrection(), psi);
			VECTOR3 err = GetOrientErrors();
			if (Airspeed >= 0.1*SoundSpeed) 
			{
				SetPitch(err.y, w, t, dt);
				SetYaw(err.z, 0, t, dt);
			}
			break;
		}
	}	
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
double R7_second_stage::GetStopSpeed(double phi, double lambda, double Hk, double alpha, double &azimut, double &dpsi)
{
	double OMEGA0 = 2*PI/86164.09;
	double phi0, lambda0, r0;
	double tp, tp0;
	double lambda1;
	double eps = 1e-3;
	double e, E, E0, theta, theta0, _r0, sin_x, cos_x, Psi0, _v0, p, a, vk;
	double Re = 6.37101e6;
	double Me = 5.973698968e24;
	int n = 0;
	bool stop = false;

	VECTOR3 V;
	
	GetEquPos(lambda0, phi0, r0);
	tp = tp0 = 0;
	lambda1 = lambda;

	do
	{
		tp0 = tp;
		lambda = lambda1 + OMEGA0*tp;
		Psi0 = acos(sin(phi0)*sin(phi) + cos(lambda - lambda0)*cos(phi0)*cos(phi));
		_r0 = 1 + Hk/Re;
		_v0 = 2*(1 + tan(alpha)*tan(alpha))*tan(Psi0/2)*tan(Psi0/2)/( (1 + _r0)*tan(Psi0/2)*tan(Psi0/2) + 2*tan(alpha)*tan(Psi0/2) + _r0 - 1 );
		vk = sqrt(GGRAV*Me*_v0/(Re + Hk));

		azimut = -atan2(sin(lambda0 - lambda)*cos(phi), cos(phi0)*sin(phi) - sin(phi0)*cos(phi)*cos(lambda0 - lambda)); 
        if (azimut > (2*PI)) azimut -= 2*PI;

		VECTOR3 PSS, VSS;
		GetRelativePos(oapiGetGbodyByName("Earth"), PSS);
		GetRelativeVel(oapiGetGbodyByName("Earth"), VSS);
		VECTOR3 North = _V(0, 1, 0);
		Ec2Eq(oapiGetGbodyByName("Earth"), PSS);
		Ec2Eq(oapiGetGbodyByName("Earth"), VSS);
		VECTOR3 N1 = crossp(PSS, VSS); N1 = N1*(1/length(N1));
		VECTOR3 N2 = crossp(North, PSS); N2 = N2*(1/length(N2));
		double azimutV = asin(length(crossp(N1, N2)));
		if (azimut < 0) azimutV = -azimutV;
		if(azimut > 90*RAD) azimutV = 90*RAD + azimutV; 
        if(azimut < -90*RAD) azimutV = -90*RAD - azimutV;
		double err = azimutV - azimut;
		debug_err_AZ = err;
		
		double korr = length(VSS)*err/100.0;
		if(fabs(korr) > 5.0*RAD) korr=sign(korr)*5.0*RAD; 
		azimut -= korr;
		

		if ( vk >= sqrt(2*GGRAV*Me/(Re + Hk)) ) 
			stop = true;
		else
		{

			p = (Re + Hk)*_v0*cos(alpha)*cos(alpha);
			e = sqrt( 1 - (2 - _v0)*_v0*cos(alpha)*cos(alpha) );
			theta0 = acos( (p/(Re + Hk) - 1)/e );
			theta = acos( (p/Re - 1)/e );
			a = (Re + Hk)/(2 - _v0);
			E0 = 2*atan( tan(theta0/2)*sqrt((1 - e)/(1 + e)) );
			E = 2*PI - 2*atan( tan(theta/2)*sqrt((1 - e)/(1 + e)) );
			tp = sqrt(a)*a*(E - E0 - e*(sin(E) - sin(E0)))/sqrt(GGRAV*Me);
		
			n++;
			if (n > 100)
			{
				n = n;
				stop = true;
			}
		}

	}
	while ( (ABS(tp - tp0) >= eps) && (!stop) );	

	if (!stop)
		return vk;
	else
	{
		azimut = Azimut;
		return vk = 7000;
	}	
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetVground(double vground)
{
	Vground = vground;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
double R7_second_stage::GetFullThrust(void)
{
	double max_force = GetThrusterMax(th_camera[0]);
	double level = GetThrusterLevel(th_camera[0]);
	double max_forceCE = GetThrusterMax(th_ce01);
	double levelCE = GetThrusterLevel(th_ce01);
	double level1, level_ce1;

	if ( (stage == FIRST_STAGE) || (stage == SEPARATION) )
	{
		level1 = 4*GetFirstStageForce();
		level_ce1 = 4*GetFirstStageCEForce()*cos(angleCE01);
	}
	else
		level1 = level_ce1 = 0;
	
	double F = 4*max_force*level + 4*max_forceCE*levelCE*cos(angleCE01) + level1 + level_ce1;
	return F;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::DumpInFile(FILE *file, double t)
{
	VECTOR3 r, v, Omega;
	GetAngularVel(Omega);
	GetRelativePos(oapiGetGbodyByName("Earth"), r);
	GetRelativeVel(oapiGetGbodyByName("Earth"), v);
	Ec2Eq(oapiGetGbodyByName("Earth"), r);
	Ec2Eq(oapiGetGbodyByName("Earth"), v);
	if ( (int)(t/1) == nt )
	{
		//fprintf(file, "%f %f %f %f %f %f %f\n", t, r.x, r.y, r.z, v.x, v.y, v.z);
		fprintf(file, "%f %f %f\n", GetAltitude(), GetAtmPressure(), GetAtmDensity());
		nt++;
	}
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::ManualControl(double t, double dt)
{
	double lp, lpm, lpp;
	double ly, lym, lyp;
	double lb, lbm, lbp;

	lp = lpm = lpp = 0;
	ly = lym = lyp = 0;
	lb = lbm = lbp = 0;

	lpp = GetManualControlLevel(THGROUP_ATT_PITCHUP, MANCTRL_ATTMODE, MANCTRL_ANYDEVICE);
	lpm = GetManualControlLevel(THGROUP_ATT_PITCHDOWN, MANCTRL_ATTMODE, MANCTRL_ANYDEVICE);
	lp = lpp - lpm;

	lyp = GetManualControlLevel(THGROUP_ATT_YAWLEFT, MANCTRL_ATTMODE, MANCTRL_ANYDEVICE);
	lym = GetManualControlLevel(THGROUP_ATT_YAWRIGHT, MANCTRL_ATTMODE, MANCTRL_ANYDEVICE);
	ly = lyp - lym;

	lbp = GetManualControlLevel(THGROUP_ATT_BANKLEFT, MANCTRL_ATTMODE, MANCTRL_ANYDEVICE);
	lbm = GetManualControlLevel(THGROUP_ATT_BANKRIGHT, MANCTRL_ATTMODE, MANCTRL_ANYDEVICE);
	lb = lbp - lbm;

	if ( (fabs(lb) < 0.01) && (fabs(ly) < 0.01) )
	{
		SetPitchMoment(lp, dt);
	}

	if ( (fabs(lb) < 0.01) && (fabs(lp) < 0.01) )
	{
		SetYawMoment(-ly, dt);
	}

	if ( (fabs(ly) < 0.01) && (fabs(lp) < 0.01) )
	{
		SetBankMoment(lb, dt);
	}
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::Stop(void)
{
	SetMainLevel(0);
	SetCELevel(0);	
	stage = HEAD_SEPARATION;
	if (!Stop1) StopTime = Time;
	Stop1 = 1;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::ReadPitchProg(char *Name)
{
	NumProgLines = 0;
	int i = 0;
	FILE *f = fopen(Name, "rt");
	while (!feof(f))
	{
		float tmp;
		fscanf(f, "%f %f %f\n", &tmp, &tmp, &tmp);
		NumProgLines++;
	}
	fclose(f);

	PTime = new float[NumProgLines];
	Pitch = new float[NumProgLines];
	OmegaP = new float[NumProgLines];
	
	f = fopen(Name, "rt");
	for (i = 0; i <= NumProgLines - 1; i++)
	{
		fscanf(f, "%f %f %f\n", &PTime[i], &Pitch[i], &OmegaP[i]);
	}
	fclose(f);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
double R7_second_stage::GetPitchProg(double t)
{
	double a, b; 
	int i = (int)t;

	if (i > NumProgLines - 1) 
		i = NumProgLines - 1;

	a = (Pitch[i+1] - Pitch[i])/(PTime[i+1] - PTime[i]);
	b = (Pitch[i]*PTime[i+1] - Pitch[i+1]*PTime[i])/(PTime[i+1] - PTime[i]);

	return a*t + b;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
double R7_second_stage::GetOmegaPProg(double t)
{
	double a, b; 
	int i = (int)t;

	if (i > NumProgLines) 
		i = NumProgLines - 1;

	a = (OmegaP[i+1] - OmegaP[i])/(PTime[i+1] - PTime[i]);
	b = (OmegaP[i]*PTime[i+1] - OmegaP[i+1]*PTime[i])/(PTime[i+1] - PTime[i]);

	return a*t + b;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetOrbitParams(double Ha, double Hp, double inc)
{
	ApogeeAlt = Ha;
	PerigeeAlt = Hp;
	Inc = inc;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
float R7_second_stage::GetLaunchTime(void)
{
	return Time;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
int R7_second_stage::GetStartFlag(void)
{
	return StartFlag;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetLaunchParams(double psi0, double omega0, double Vk0)
{
	Psi0 = psi0;
	Omega0 = omega0;
    Vk = Vk0;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
double R7_second_stage::GetPayLoadMass(void)
{
	return PayLoadMass;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::GetOrbitParams(double &Ha, double &Hp, double &inc, double &azimut)
{
	Ha = (double)ApogeeAlt;
	Hp = (double)PerigeeAlt;
	inc = (double)Inc;
	azimut = (double)Azimut;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
double R7_second_stage::GetAzimutCorrection(void)
{
	double azimut;
	double phi0, lambda0, r0;
	GetEquPos(lambda0, phi0, r0);
	azimut = asin(cos(Inc)/cos(phi0));

	VECTOR3 PSS, VSS;
	GetRelativePos(oapiGetGbodyByName("Earth"), PSS);
	GetRelativeVel(oapiGetGbodyByName("Earth"), VSS);
	VECTOR3 North = _V(0, 1, 0);
	Ec2Eq(oapiGetGbodyByName("Earth"), PSS);
	Ec2Eq(oapiGetGbodyByName("Earth"), VSS);
	VECTOR3 N1 = crossp(PSS, VSS); N1 = N1*(1/length(N1));
	VECTOR3 N2 = crossp(North, PSS); N2 = N2*(1/length(N2));
	double azimutV = asin(length(crossp(N1, N2)));
	if (azimut < 0) azimutV = -azimutV;
	if(azimut > 90*RAD) azimutV = 90*RAD + azimutV; 
    if(azimut < -90*RAD) azimutV = -90*RAD - azimutV;
	double err = azimutV - azimut;
	//debug_err_AZ = err;
		
	double korr = length(VSS)*err/100.0;
	if(fabs(korr) > 5.0*RAD) korr=sign(korr)*5.0*RAD; 
	azimut -= korr;

	return azimut;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
int R7_second_stage::GetConfig(void)
{
	return config;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::GetProgParams(double &omega0)
{
	omega0 = Omega0;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
int R7_second_stage::GetStage(void)
{
	switch (stage)
	{
	case FIRST_STAGE:
		{
			return 1;
			break;
		}
	case SEPARATION:
		{
			return 1;
			break;
		}
	case SECOND_STAGE:
		{
			return 2;
			break;
		}
	case HEAD_SEPARATION:
		{
			return 2;
			break;
		}
	case STOP:
		{
			return 3;
			break;
		}
	default: 
		return -1;
	}
	
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
int R7_second_stage::GetAutopilot(void)
{
	return Autopilot;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
double R7_second_stage::VerticalSpeedControl(double r, double v, double vr, double t, double dt)
{
	double pitch, pitch0, dP;
	VECTOR3 T, P;
	double thr, p, a, g;
	double mass = GetMass();
	double tau = 10;
	double Kv, Ka;
	double ar, aG;
	double Ap;
	double vrZ;

	GetWeightVector(P);
	GetThrustVector(T);
	g = length(P)/mass;
	a = length(T)/mass;
	pitch0 = GetPitch();


	Kv = 1/tau/a/cos(pitch0);
	Ka = 1/a/cos(pitch0);
	aG = (v*v - vr*vr)/r - g;
	ar = aG + a*sin(GetPitch());
	GetApDist(Ap);

	vrZ = 0;

	dP = -Kv*(vr - vrZ) - 0.01*ar;
	if ( fabs(vr/v) >= sin(0.05*RAD) )
		pitch = dP;
	else
		pitch = -asin(aG/a);

	if (fabs(pitch) >= 20*RAD) pitch = 20*RAD*sign(pitch);		

	return pitch;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::ThrusterProgram(double t, double dt)
{
	switch (config)
	{
	case CFG_3STAGE:
		{
			if (GetPropellantMass(ph_main)/TANK_CAPACITY2 < 0.02)
			{
				psi = GetPitch();
				level3 += 0.3*dt;
				if (level3 >= 1.0) level3 = 1.0;
				hHead->SetThrusterGroupLevel(THGROUP_MAIN, level3);
			}

			if ( (config == CFG_3STAGE) && (level3 >= 0.7) )
			{
				Stop();
			}		
		}
	}
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::InversTrack(double t, double dt)
{
	double h = GetAltitude();	

	if (h > 9*1000)
	{
		plevel = plevel + 0.1*dt;
		if (plevel > 1.0) plevel = 1.0;
	}
	else
		plevel = 0;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetPitchMoment(double level, double dt)
{
	double angle = asin(sin(ANGLE_CE_MAX*level));
	
	SetCEAngle01(-angle, dt);
	SetCEAngle03(-angle, dt);
	
	if (stage == FIRST_STAGE)
	{
		hblockB->SetCEAngle(-angle, dt);
		hblockD->SetCEAngle(angle, dt);
	}
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetYawMoment(double level, double dt)
{
	double angle = asin(sin(ANGLE_CE_MAX*level));
	
	SetCEAngle02(angle, dt);
	SetCEAngle04(angle, dt);
	
	if (stage == FIRST_STAGE)
	{
		hblockC->SetCEAngle(angle, dt);
		hblockE->SetCEAngle(-angle, dt);
	}
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::SetBankMoment(double level, double dt)
{
	double angle = asin(sin(ANGLE_CE_MAX*level));
	
	SetCEAngle01(angle, dt);
	SetCEAngle03(-angle, dt);
	SetCEAngle02(angle, dt);
	SetCEAngle04(-angle, dt);
	
	if (stage == FIRST_STAGE)
	{
		hblockB->SetCEAngle(angle, dt);
		hblockC->SetCEAngle(angle, dt);
		hblockD->SetCEAngle(angle, dt);
		hblockE->SetCEAngle(angle, dt);
	}
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::KillRot(double t, double dt)
{
	VECTOR3 Omega, omg_ort;
	double omega;
	double eps = 1e-2*RAD;
	
	GetAngularVel(Omega);
	omega = length(Omega);

	if (killrot)
	{	
		if (omega < eps)
		{
			killrot = 0;
			SetAngularVel(_V(0, 0, 0));
		}
		else
		{
			omg_ort = Omega*(1/omega);
			switch (sw)
			{
			case 0:
				{
					SetPitchMoment(-omg_ort.x, dt);
					break;
				}
			case 1:
				{
					SetYawMoment(omg_ort.y, dt);
					break;
				}
			case 2:
				{
					SetBankMoment(omg_ort.z, dt);
					break;
				}
			}
			sw++;
			if (sw > 2) sw = 0;			
		}
	}	
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void R7_second_stage::Start(void)
{
	OBJHANDLE hObj = GetAttachmentStatus(hToLaunchpadAtt);
	if (hObj)
	{
		VESSEL *hvel = oapiGetVesselInterface(hObj);
		hLaunchpad = (R7_launchpad*) hvel;
		hLaunchpad->StartLaunchOperations();
	}
}