//=============================================================================
//
//
//
//
//
//=============================================================================
#ifndef		__R7_SECOND_STAGE_H
#define		__R7_SECOND_STAGE_H

class DLLEXPORT R7_first_stage: public VESSEL2
{
public:
	void SetMainLevel(double);
	void SetCELevel(double);
	void SetCEAngle(double, double);
	void OpenGate(void);
	double GetMaxThrust(void);
	double GetMaxCEThrust(void);
	double GetFuelMass(void);	
};

class DLLEXPORT R7_launchpad: public VESSEL2
{
public:
	void StartLaunchOperations(void);
};

class DLLEXPORT R7_second_stage: public VESSEL2
{
public:
	R7_second_stage(OBJHANDLE, int);
	~R7_second_stage(){};

	void clbkSetClassCaps(FILEHANDLE);
	void clbkLoadStateEx(FILEHANDLE, void *);
	void clbkSaveState(FILEHANDLE);
	void clbkPostStep(double, double, double);
	void clbkVisualCreated(VISHANDLE, int);
	void clbkVisualDestroyed(VISHANDLE, int);
	int clbkConsumeDirectKey(char *);
	int clbkConsumeBufferedKey(DWORD, bool, char *);	

	void StartLaunchProg(void);
	void SetHyroplatform(double, double);
	bool LaunchpadFree(void);
	void SetTaget(double, double);
	void SetVground(double);
	double GetFirstStageLevel(void);
	void SetOrbitParams(double Ha, double Hp, double inc);
	void SetLaunchParams(double psi0, double omega0, double Vk);
	float GetLaunchTime(void);
	int GetStartFlag(void);
	double GetPayLoadMass(void);
	void GetOrbitParams(double &Ha, double &Hp, double &inc, double &azimut);
	int GetConfig(void);
	void GetProgParams(double &omega0);
	int GetStage();
	int GetAutopilot();
	void Start(void);

private:
	OBJHANDLE				hB, hC, hD, hE, hH;
	VISHANDLE				vis;
	ATTACHMENTHANDLE		hBlockBAtt, hBlockCAtt, hBlockDAtt, hBlockEAtt;
	ATTACHMENTHANDLE		hHeadAtt, hChild;
	ATTACHMENTHANDLE		hToLaunchpadAtt;
	MESHHANDLE				hBlockA;
	MESHHANDLE				hAdapter;
	MESHHANDLE				hCE01, hCE02, hCE03, hCE04;
	VESSEL					*hBlockB;
	VESSEL					*hBlockC;
	VESSEL					*hBlockD;
	VESSEL					*hBlockE;
	VESSEL					*hHead;
	PROPELLANT_HANDLE		ph_main;
	THRUSTER_HANDLE			th_camera[4];
	THRUSTER_HANDLE			th_ce01;
	THRUSTER_HANDLE			th_ce02;
	THRUSTER_HANDLE			th_ce03;
	THRUSTER_HANDLE			th_ce04;
	THGROUP_HANDLE			thg_main;
	SURFHANDLE				etex, ptex;
	PARTICLESTREAMSPEC		psc, psc1;	

	R7_first_stage			*hblockB;
	R7_first_stage			*hblockC;
	R7_first_stage			*hblockD;
	R7_first_stage			*hblockE;

	R7_launchpad			*hLaunchpad;

	int						stage, config;

	double					angleCE01, angleCE02, angleCE03, angleCE04;
	double					FirstStageLevel, FirstStageCELevel;
	double					SecondStageLevel, SecondStageCELevel;
	double					massB, massC, massD, massE;
	double					F1;
	double					p0, p;
	float					alphaB, alphaC, alphaD, alphaE;
	float					OmegaB, OmegaC, OmegaD, OmegaE;
	double					LTime;
	float					PayLoadMass;
	double					Vdetach, Vsep;
	float					Azimut, Alpha;
	float					phiT, lambdaT;
	double					Vground, Vw;
	float					Time, StopTime;
	float					psi;
	double					debug_err_AZ;
	VECTOR3					Omega;
	VECTOR3					PAY_ATTACH;
	VECTOR3					PAY_DIR;
	VECTOR3					PAY_ROT;

	double  Psi0, Omega0, Vk, LAzimut;	
	double Error;
	double dPitch, alphaG;
	double T1;
	double level3;
	double ApogeeAlt, PerigeeAlt, Inc;
	double psi2, t2;
	double plevel;

	int						StartFlag;
	int						Autopilot;
	int						Stop1;
	int						Far;

	int						signP1, signP0;
	int						np1, np2, nt, Natt;

	int						signY1, signY0;
	int						ny1, ny2, ntest, NumProgLines;

	int						killrot, sw;

	FILE					*out;
	FILE					*prog;
	char					PayLoadName[255];
	char					MeshName[255];
	char*					line;
	float					*PTime;
	float					*Pitch;
	float					*OmegaP;

	// added for Vostok-Luna mod
	bool					bVostokLuna;	//+++
	float					tA;				//+--
	float					psiA;			//---
	float					psiRatio;		//---

	MESHGROUP_TRANSFORM		mt01, mt02, mt03, mt04;

	void SetCEAngle01(double, double);
	void SetCEAngle02(double, double);
	void SetCEAngle03(double, double);
	void SetCEAngle04(double, double);
	void SetCELevel(double);
	void SetMainLevel(double);

	double FirstStageProg(double);
	void SetFirstStageMainThrust(double);
	void SetFirstStageCEThrust(double);
	double SecondStageProg(double);
	void FirstStageSeparation(void);
	double GetFirstStageForce(void);
	double GetFirstStageCEForce(void);
	void LateralBlocksRotation(double);
	VECTOR3 GetLongAxisDir(double, double);
	VECTOR3 GetOrientErrors(void);
	double	u(double, double, double, double);
	double	u2(double , double, double, double, double, double);
	void SetPitch(double, double, double, double);
	void SetYaw(double, double, double, double);
	void ProgramStep(double, double);
	double GetStopSpeed(double, double, double, double, double &, double &);
	double GetFullThrust(void);
	void DumpInFile(FILE *, double);
	void ReadPitchProg(char *);
	void ManualControl(double t, double dt);
	void Stop(void);
	double GetOmegaPProg(double t);
	double GetPitchProg(double t);
	double GetAzimutCorrection(void);
	double GetCPitch(double t);
	double VerticalSpeedControl(double r, double v, double vr, double t, double dt);
	void ThrusterProgram(double t, double dt);
	void InversTrack(double t, double dt);
	void SetPitchMoment(double level, double dt);
	void SetYawMoment(double level, double dt);
	void SetBankMoment(double level, double dt);
	void KillRot(double t, double dt);	
};

#endif