#include "..\r7_vessel2m_VL\vessel2m.h"

// main engine parameters
const		double		TANK_CAPACITY = 6900;
const		double		MAIN_THRUST = 54500;
const		double		MAIN_ISP = 3173;
	// exhaust textures
const		double		EX_LENGTH = 6.0;
const		double		EX_WIDTH = 0.7;
	// thrust application point
const		VECTOR3		MAIN_REF = {0, 0, -0.739};
	// thrust direction
const		VECTOR3		MAIN_DIR = {0, 0, 1};

// pitch-yaw engines parameters
const		double		CE_THRUST = 2*25;
const		double		CE_ISP = 3173;
	// exhaust textures
const		double		EX_LENGTH_BIG = 1.0;
const		double		EX_WIDTH_BIG = 0.12;
	// thrust application points
const		VECTOR3		CE01_REF = {0, 1.468, -0.932};
const		VECTOR3		CE02_REF = {1.468, 0, -0.932};
const		VECTOR3		CE03_REF = {0, -1.468, -0.932};
const		VECTOR3		CE04_REF = {-1.468, 0, -0.932};
	// thrust directions
const		VECTOR3		CE01_DIR = {0, sin(30*RAD), cos(30*RAD)};
const		VECTOR3		CE02_DIR = {sin(30*RAD), 0, cos(30*RAD)};
const		VECTOR3		CE03_DIR = {0, -sin(30*RAD), cos(30*RAD)};;
const		VECTOR3		CE04_DIR = {-sin(30*RAD), 0, cos(30*RAD)};

// roll engines parameters
	// thrust application points
const		VECTOR3		CE05_REF = {0.042, 1.3, -0.835};
const		VECTOR3		CE06_REF = {1.3, -0.042, -0.835};
const		VECTOR3		CE07_REF = {-0.042, -1.3, -0.835};
const		VECTOR3		CE08_REF = {-1.3, 0.042, -0.835};
const		VECTOR3		CE09_REF = {-0.042, 1.3, -0.835};
const		VECTOR3		CE10_REF = {1.3, 0.042, -0.835};
const		VECTOR3		CE11_REF = {0.042, -1.3, -0.835};
const		VECTOR3		CE12_REF = {-1.3, -0.042, -0.835};
	// thrust directions
const		VECTOR3		CE05_DIR = {-1, 0, 0};
const		VECTOR3		CE06_DIR = {0, 1, 0};
const		VECTOR3		CE07_DIR = {1, 0, 0};
const		VECTOR3		CE08_DIR = {0, -1, 0};
const		VECTOR3		CE09_DIR = {1, 0, 0};
const		VECTOR3		CE10_DIR = {0, -1, 0};
const		VECTOR3		CE11_DIR = {-1, 0, 0};
const		VECTOR3		CE12_DIR = {0, 1, 0};

// physics
const		double		EMPTY_MASS = 1100;
const		VECTOR3		PMI_LUNNIK = {0.2, 0.2, 0.2};

// attachments
	// fairing attachment params on block E
const		VECTOR3		FLSPOINT = {0, 0, 1.763};
const		VECTOR3		FLSDIR = {0, 0, 1};
const		VECTOR3		FLSROT = {1, 0, 0};

	// payload attachment params on block E
const		VECTOR3		PAYPOINT = {0, 0, 1.09};
const		VECTOR3		PAYDIR = {0, 0, 1};
const		VECTOR3		PAYROT = {1, 0, 0};

	// camera
const double CAMERA_SPEED_RAD = 2.5*RAD; // rad/sec
const double CAMERA_Z_SIDE = 0.2; // z-component of initial dir unit vector for side-seeing camera, same as sin of the value
const double CAMERA_Z_TRAVEL_RAD = (PI / 2) - CAMERA_Z_SIDE;		// total travel span of z-component, from initial to 1.

	// adapters meshes offsets %%%
const		VECTOR3		ADAPTER_OFS = {0, 0, 0.878};
const		VECTOR3		ADAPTER02_OFS = {0, 0, 0.893};

// for orbit calcs
const double GRAVITYCONSTANT = 6.67259e-20*1e9;
const double MOON_PERIOD = 2360591.597;

// for detecting MECO approach
const double ARRIVAL_DELAY_ELLIPTIC = 3600 * 10;
const double ARRIVAL_DELAY_PARABOLIC = 3600 * 2;

// sound
const int SND_PYRO = 1;
const int SND_MOTOR = 2;





// second stage object declaration.
// We need it for forward referencing 
// until second stage is redone with VESSEL2M.
// Its usage is limited to just a couple of navigational parameters.
class DLLEXPORT R7_second_stage: public VESSEL2
{
public:
	void GetOrbitParams(double &Ha, double &Hp, double &inc, double &azimut) ;
	int GetAutopilot() ;
};





class R7_third_stage: public VESSEL2M
{
public:
	// constructor
	R7_third_stage(OBJHANDLE, int);

	// callbacks
	void clbkSetClassCaps(FILEHANDLE);
	void clbkPostCreation (void);
	void clbkPostStep(double, double, double);
	int clbkConsumeDirectKey(char *);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	bool clbkLoadGenericCockpit ();
	void clbkNavMode (int mode, bool active);

	// VESSEL2M overrides
	void clbkSaveState (FILEHANDLE scn);
	void ParseScenarioLineEx(char *line, void *status);
	void FinishConfiguration ();

private:
	// fuel tank
	PROPELLANT_HANDLE ph_main;

	// engine thrusters
	THRUSTER_HANDLE		th_main_All[13];
	THRUSTER_HANDLE		th_main[1];
	THRUSTER_HANDLE		th_pitch[2];
	THRUSTER_HANDLE		th_yaw[2];
	THRUSTER_HANDLE		th_bank_left[4];
	THRUSTER_HANDLE		th_bank_right[4];
	THGROUP_HANDLE		thg_main;

	// 2-nd stage interface %%% try to minimize its use or elimiate completely
	R7_second_stage		*hR7;						//+--

	// fairing
	ATTACHMENTHANDLE	hForFairing;				//+--
	OBJHANDLE hFairing;								//---
	VESSEL *hFairingI;								//---
	bool bFairingNotInitialized;					//+-- 
	bool bFairingAttached;							//+-- 
	double FairingMass;								//+--
	void InitializeFairing();
	void FairingSeparate();

	// payload
	char	PayloadName[255];				//+++
	VECTOR3 PayREF, PayDIR, PayROT;			//+++
	double PayloadMass;						//+--
	ATTACHMENTHANDLE	hForPayload;		//---
	void PayloadAttach();
	void PayloadSeparate();

	// comet
	enum CometStates {COMET_NONE, COMET_CHARGED, COMET_FIRING, COMET_DONE} 
		CometState;				//+++
	double CometStreamLevel;	//+++
	double CometTimer;			//+++
	void FireComet();

	// cameras
	enum CameraDirs {CAMERA_UP, CAMERA_DOWN, CAMERA_SIDE} 
		CameraDir;				//+++
	bool bMoveCamera;			//+--
	double CameraZ;				//--- z-component of the camera dir unit vector
	VECTOR3 CameraVector;		//---

	void ToggleOnboardCamera(CameraDirs Dir);
	void StartCameraRotation();
	void StopCameraRotation();
	void MoveCamera();

	// focus
	bool bSwitchFocus;								//+++

	// autopilot/manual controls
	bool bManualControl;						//+++
	void ToggleManualAndAutoplilot();
	void SetAutopilotControl();
	void SetManualControl();
	void ManualControl();

	// attitude control 
	// hyroplatform intermediate vars
	double Azimut;									//---
	double Alpha;									//---
	double Inc;										//+++
	bool bLockPitchOrYaw;							//+--
	double ArivalDelay;								//---
	double GetAzimutCorrection();
	VECTOR3 GetLongAxisDir(double azimut, double alpha);
	VECTOR3 GetOrientErrors();
	void SetHyroplatform(double, double);
	void SetPitchMoment(double level);
	void SetYawMoment(double level);
	void SetBankMoment(double level);
	double u2(double X, double XX, double w0, double K, double dt);
	void SetPitch(double dP, double wp, double t, double dt);
	void SetYaw(double dY, double wy, double t, double dt);
	void KillRot();
	void SlowRot();
	void FixBank();

	// guidance
	double SimDt;							//---
	double Met;								//+++
	void ProgramStep(double t, double dt);
	void SetMainLevel(double level);
	
	// moon trajectory
	bool bMoonNotCalculated;	//+-- 
	bool bMoonElliptic;			//-++     elliptic/parabolic flag. Calculated in CalcOtherTimes(), used in lots of places
	bool bMoonFlyby;			//+++
	double MoonTravelTime;		//-++  Calculated in CalcMoonTouchParams()
	char TrajectoryScn[256];	//---  uset to save/restore trajectory parameters

	// moon trajectory - moslty intermediate vars, no initialization or percistence in scn file
	// general planet params
	OBJHANDLE hEarth;			//+--
	OBJHANDLE hMoon;			//+--
	double MuEarth;				//+--
	double MuBoth;				//+--

	MATRIX3 TmEcliToMoon;		//-++
	double eMoon;				//---
	double pMoon;				//--- 
	double MoonTrueAnomaly;		//---
	double MoonCrossR;			//-++
	VECTOR3 VPos, VVel;			//---
	double Pos, Vel;			//---
	VECTOR3 VNormal;			//---
	VECTOR3 VNormal1;			//---
	double PeriArgument;		//---
	double WantedPeriArgument;	//---
	double MyTrueAnomaly;		//---
	double CurrentMoonCrossTime;//+--
	double MyCrossTrueAnomaly;	//-++
	double BurnTime;			//-++
	double BurnTimeElapsed;		//-++
	double BurnTimeRemaining;	//---
	double PeriArgumentStart;	//-++
	double TargetSemiMajorAxis;	//-++

	// moon trajectory calcs - one time
	void CalcGeneralPlanetParams(); // called from constructor
	void CalcGeneralMoonParams();
	void CalcMoonTouchParams();
	void CalcOtherTimes();
	void CalcEllipticPath();
	void CalcHyperbolicPath();
	// moon trajectory calcs - each step
	void CalcEllipticStep();
	void CalcHyperbolicStep();
	void CalcPeriArgument();
	void CalcWantedPeriArgument();
	void RecalcPosVelNormalsInMoonOrbitFrame();

	// sound
	int SoundLibID;					//---

	// debug output
	double dOutp1;				//---
	double dOutp2;				//---
	int iOutp1;					//---
	void DebugSomething();
};




inline MATRIX3 MakeTranslationMatrix( const VECTOR3 &X, const VECTOR3 &Y,const VECTOR3 &Z )
{
	MATRIX3 TM;

	TM.m11 = X.x;
	TM.m12 = X.y;
	TM.m13 = X.z;
	
	TM.m21 = Y.x;
	TM.m22 = Y.y;
	TM.m23 = Y.z;

	TM.m31 = Z.x;
	TM.m32 = Z.y;
	TM.m33 = Z.z;

	return TM;
}

// eccentricity vector, AKA Laplace vector normalized by Mu, points to pericenter
inline VECTOR3 VEccentricity(double Mu, const VECTOR3 VPos, const VECTOR3 VVel)
{
	VECTOR3 VE, VPart1, VPart2, M;
	double Pos;

	M = crossp(VPos, VVel);
	VPart1 = crossp(VVel, M) / Mu;

	Pos = length(VPos);
	VPart2 = VPos / Pos; 

	VE = VPart1 - VPart2;

	return VE;
}


inline double TrueAnomaly(VECTOR3 &VE, VECTOR3 &VPos, VECTOR3 &VVel)
{
	double E = length(VE);
	double Pos = length(VPos);

	double f = acos((VPos.x * VE.x + VPos.y * VE.y + VPos.z * VE.z) / (Pos * E));

	if ( dotp(VPos, VVel) < 0)
		f = 2*PI - f;

	return f;
}


inline double ArgOfPeri(VECTOR3 &VNormal, VECTOR3 &VE)
{
	double Normal = length(VNormal);
	double E = length(VE);

	double omega = acos((VNormal.x * VE.x + VNormal.y * VE.y + VNormal.z * VE.z) / (Normal * E));

	if (VE.z < 0) 
		omega = 2*PI - omega;

	return omega;
}


inline double EccentricAnomaly(double E, double f)
{
	double ea = 2 * atan(sqrt((1-E)/(1+E)) * tan(f/2));
	
	if (ea < 0)
		ea = 2*PI + ea;

	return ea;
}


inline double MeanAnomaly(double EccentricAnomaly, double E)
{
	return EccentricAnomaly - E * sin(EccentricAnomaly);
}


inline double TimeSincePeriapsis(double MeanAno, double T)
{
	return MeanAno / (2*PI) * T;
}



inline double TimeOfTrueAnomaly(double TrueAno, double E, double T)
{
	double ea = EccentricAnomaly(E, TrueAno);
	double ma = MeanAnomaly(ea, E);
	double tm = TimeSincePeriapsis(ma, T);
	return tm;
}


// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/vblr7/html/vagrpDerivedMath.asp
inline double atanh(double x)
{	
	return log( (1 + x)/(1 - x) ) / 2;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void Ec2Eq (OBJHANDLE hPlanet,VECTOR3 &Location) 
{ 
	double   Obl =   oapiGetPlanetObliquity (hPlanet); 
	double   Tet =   oapiGetPlanetTheta (hPlanet); 
	VECTOR3  tmp = Location;
	
	Location.x = tmp.x;
	Location.y = tmp.y*cos(Obl) + tmp.z*sin(Obl);
	Location.z = -tmp.y*sin(Obl) + tmp.z*cos(Obl);
}
