/*
	Intermediace vessel class for r7 vessel objects inheritance.
	Provides common additional functionality:
	- easy message exchange between all derived objects 
	(removes the need for structure awareness and cross-include of the header files);
    - crash, burn, wrecks, craters on Earth and Moon.
*/

#include "orbitersdk.h"
#include "OrbiterSoundSDK35.h"

#pragma warning(disable:4996)   // disable CRT string functions security warnings

// meaningful consts for attachment handling
const bool TOPARENT = true; //default
const bool TOCHILD = false;
const bool LOOSE = true; // deafult 
const bool NOTLOOSE = false; 

// crash variations consts
const bool EXPLODE = true;
const bool NO_EXPLODE = false;
const bool GENERIC_CRATER = true;

// on-screen message lifetime, sec
const double MSG_DISPLAY_TIME = 3;


class VESSEL2M: public VESSEL2 {
public:
	// constructor
	VESSEL2M (OBJHANDLE hObj, int fmodel);
	~VESSEL2M ();

	// structural speed and Q limits
	double VvCrash;		//+--
	double VhCrash;		//+--
	double VMaxBlast;	//+--
	double QMax;		//+--

	// configuration
	bool bAttached;		//+--
	bool bLandedPermanently;	//+--
	virtual void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	virtual void ParseScenarioLineEx(char *line, void *status);
	virtual void FinishConfiguration();

	void TimestepForVESSEL2M();
	virtual void Land();
	virtual void Explode();

	// focus
	int bFocus;			//+++
	bool bFocusOffByDefault;	//+--   fosus is ON by default in R7

	// message output
	void ShowAnnotationMessage (char* Message);

	// sending messages between VESSEL2M objects
	enum VesselMessage {VM_AUTOPILOT_OFF};
	void SendVesselMessage(OBJHANDLE hTarget, VesselMessage VM);
    virtual void GetVesselMessage(VesselMessage VM);

	// service functions
	double DRand(double from, double span);
	int IRand(int from, int  span);

private:
	// landing velocities, initialized in GetLandingVelocities(), used by other functions
	double V;	//---
	double Vv;	//---
	double Vh;	//---

	// landing-crashing-burning
	void GetLandingVelocities();
    bool bFuelExplode();
	void Crash();
	void Burn();
	void ReplaceWithWreck(bool bExplode = TRUE, bool bGenericCrater = FALSE);

	// dust cloud on low-speed impact, unlikely for R7 objects
	PARTICLESTREAMSPEC Dust;	//+--
	bool bGeneratingDust;		//+--
	double DustTimer;			//+--
	double DustLevel;			//+--
	void StartDust();
	void StopDust();

	// message output
	NOTEHANDLE MessagePort;							//---
	double msgStartTime;							//+--
	bool bMessageDisplayed;							//+--
};
